/*******************************************************************************
+
+  LEDA 5.0  
+
+
+  agd_demo.c
+
+
+  Copyright (c) 1995-2004
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                         GraphWinAGD.h
//
//    authors: David Alberts, Carsten Gutwenger, Gunnar Klau
//    date:    decembre 1999
//-------------------------------------------------------------------

#include <LEDA/graphics/graphwin.h>

using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cin;
using std::cout;
using std::cerr;
using std::endl;
using std::ostream;
using std::istream;
#endif


#include <AGD/FPPLayout.h>
#include <AGD/ConvexLayout.h>
#include <AGD/ConvexDrawLayout.h>
#include <AGD/MixedModelLayout.h>
#include <AGD/PlanarStraightLayout.h>
#include <AGD/PlanarDrawLayout.h>
#include <AGD/SchnyderLayout.h>
#include <AGD/SpringLayout.h>
#include <AGD/TutteLayout.h>
#include <AGD/VisibilityRepresentation.h>
#include <AGD/TreeLayout.h>
#include <AGD/SugiyamaLayout.h>
#include <AGD/PlanarizationGridLayout.h>
#include <AGD/PureOrthogonalLayout.h>
#include <AGD/QuasiOrthogonalLayout.h>
#include <AGD/OrthogonalLayout.h>
#include <AGD/StPlanarLayout.h>

#include <AGD/PlanAug.h>
#include <AGD/PlanarSubgraph.h>
#include <AGD/NaiveMaximalPlanarSubgraph.h>
#include <AGD/LEDAMakeBiconnected.h>
#include <AGD/LEDAMakeAcyclic.h>
#include <AGD/GreedyCycleRemoval.h>
#include <AGD/LongestPathRanking.h>
#include <AGD/CoffmanGrahamRanking.h>
#include <AGD/FastHierarchyLayout.h>
#include <AGD/ReducedDigraph.h>
#include <AGD/LongestPathCompaction.h>
#include <AGD/FlowCompaction.h>

#include <AGD/generators.h>
#include <AGD/LeveledGraph.h>

class GraphWinAGD : public GraphWin
{
public:

	GraphWinAGD();

	virtual ~GraphWinAGD() { }

protected:
	void optionsConvex();
	void optionsConvexDraw();
	virtual void optionsStraight();
	virtual void optionsPlanarDraw();
	virtual void optionsMixedModel();
	virtual void optionsOvid(Ovid &O);
	void optionsSpring();
	void optionsTreeLayout();
	virtual void optionsSugiyama();
	virtual void optionsPlanarization();
	void optionsGlobal();
	void optionsPlanSub();
	void optionsPlanAug();
	void optionsStPlanarGrid();
	void optionsStPlanar();

	FPPLayout fpp;
	ConvexLayout convex;
	ConvexDrawLayout convex_draw;
	PlanarStraightLayout straight;
	PlanarDrawLayout planar_draw;
	MixedModelLayout mixed;
	SchnyderLayout schnyder;
	SpringLayout spring;
	TutteLayout tutte;
	VisibilityRepresentation visib_rep;
	SugiyamaLayout sugiyama;
	TreeLayout tree_layout;
	PlanarizationGridLayout planarization;
	PureOrthogonalLayout pure_ortho;
	OrthogonalLayout ortho;
	QuasiOrthogonalLayout quod;
	StPlanarGridLayout st_planar_grid;
	StPlanarLayout st_planar;

	PlanAug plan_aug;
	LEDAMakeBiconnected make_bic;
	PlanarSubgraph plan_sub;
	NaiveMaximalPlanarSubgraph naive_maximal_plan_sub;
	LEDAMakeAcyclic make_acyclic;
	GreedyCycleRemoval greedy_cycle_removal;
	LongestPathRanking hierarchical;
	CoffmanGrahamRanking cgr;
	FastHierarchyLayout fhl;
	ReducedDigraph reduced_digraph;
	LongestPathCompaction C_longest_paths;
	FlowCompaction C_flow;

public:
	static void callFppLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.fpp);
	}
	static void callConvexLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.convex);
	}
	static void callConvexDrawLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.convex_draw);
	}
	static void callPlanarStraightLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.straight);
	}
	static void callPlanarDrawLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.planar_draw);
	}
	static void callMixedModelLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.mixed);
	}
	static void callPureOrthoLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.pure_ortho);
	}
	static void callOrthoLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.ortho);
	}
	static void callQuodLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.quod);
	}
	static void callSchnyderLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.schnyder);
	}
	static void callTutteLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.tutte.bbox (DRect(GW.get_xmin(), GW.get_xmax(),
			-GW.get_ymax(), -GW.get_ymin()));
		callLayout (gwAGD, gwAGD.tutte);
	}
	static void callSpringLayout (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callLayout (gwAGD, gwAGD.spring);
	}
	static void callVisibilityRepresentation (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout (gwAGD, gwAGD.visib_rep);
	}
	static void callSugiyamaLayout(GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		if (callLayout(gwAGD, gwAGD.sugiyama))
			cout << gwAGD.sugiyama.number_of_crossings() << " crossings" << endl;
	}
	static void callTreeLayout(GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callLayout(gwAGD, gwAGD.tree_layout);
	}
	static void callPlanarizationLayout(GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		if (callGridLayout(gwAGD, gwAGD.planarization))
			cout << gwAGD.planarization.number_of_crossings() << " crossings" << endl;
	}
	static void callStPlanarGrid (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callGridLayout2 (gwAGD, gwAGD.st_planar_grid);
	}
	static void callStPlanar (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callLayout (gwAGD, gwAGD.st_planar);
	}


	static void callRandomPlanarGraph (GraphWin& GW) {
		gen_random_graph (GW, random_planar_graph,
			"Create Random Planar Graph");
	}
	static void callRandomPlanarTricGraph (GraphWin& GW) {
		gen_random_graph (GW, planar_triconnected_graph,
			"Create Triconnected Planar Graph");
	}
	static void callRandomPlanarBicGraph (GraphWin& GW);
	static void callRandomGraph (GraphWin& GW);
	static void callRandomGraph2 (GraphWin& GW) {
		gen_random_graph (GW, random_graph_3,
			"Create Random Graph (2)");
	}
	static void callRandomFourGraph (GraphWin& GW) {
		gen_random_graph (GW, random_four_graph,
			"Create Random Four-Graph");
	}
	static void callRandomTree (GraphWin& GW);
	static void callRandomStPlanarGraph(GraphWin &gw1);
	static void callRandomHierarchy (GraphWin& GW);


	static void callPlanAug (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callAugment (gwAGD, gwAGD.plan_aug);
	}
	static void callMakeBic (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callAugment (gwAGD, gwAGD.make_bic);
	}

	static void callPlanSub (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callSubgraph (gwAGD, gwAGD.plan_sub);
	}

	static void callNaiveMaximalPlanSub (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callSubgraph (gwAGD, gwAGD.naive_maximal_plan_sub);
	}
	static void callMakeAcyclic (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callSubgraph (gwAGD, gwAGD.make_acyclic);
	}
	static void callGreedyCycleRemoval (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callSubgraph (gwAGD, gwAGD.greedy_cycle_removal);
	}
	static void callReducedDigraph (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		callSubgraph (gwAGD, gwAGD.reduced_digraph);
	}

	static void callOptionsStPlanarGrid (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsStPlanarGrid();
	}
	static void callOptionsStPlanar (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsStPlanar();
	}
	static void callOptionsConvex (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsConvex();
	}
	static void callOptionsConvexDraw (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsConvexDraw();
	}
	static void callOptionsStraight (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsStraight();
	}
	static void callOptionsPlanarDraw (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsPlanarDraw();
	}
	static void callOptionsMixedModel (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsMixedModel();
	}
	static void callOptionsPureOrtho (GraphWin &GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsOvid(gwAGD.pure_ortho);
	}

	static void callOptionsOrtho (GraphWin &GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsOvid(gwAGD.ortho);
	}

	static void callOptionsQuod (GraphWin &GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsOvid(gwAGD.quod);
	}
	static void callOptionsSpring (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsSpring();
	}
	static void callOptionsTreeLayout(GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsTreeLayout();
	}
	static void callOptionsPlanarization (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsPlanarization();
	}
	static void callOptionsSugiyama (GraphWin &GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsSugiyama();
	}

	static void callOptionsGlobal (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsGlobal();
	}

	static void callOptionsPlanSub (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsPlanSub();
	}

	static void callOptionsPlanAug (GraphWin& GW) {
		GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
		gwAGD.optionsPlanAug();
	}

	static bool callLayout (GraphWinAGD& GW, LayoutModule& L);
	static bool callGridLayout (GraphWinAGD& GW, GridLayoutModule& GL);
	static bool callGridLayout2 (GraphWinAGD& GW, GridLayoutModule& L);
	static bool callAugment (GraphWinAGD& GW, AugmentationModule& A);
	static bool callSubgraph (GraphWinAGD& GW, SubgraphModule& S);

	static void callHelpAbout(GraphWin& GW);

protected:
	LeveledGraph LG;

	static string get_error_message(AgdKey p);

	int planarization_layouter, planarization_planarizer, planarization_inserter;
	int sug_ranking, sug_cross_min, sug_cycle_removal, sug_layout;
	int the_compactor;

	int the_mapper;
	double the_min_over_x, the_min_over_y;
	double the_sep_x, the_sep_y;
	bool the_pres_embedding;

	bool rh_planar, rh_single, rh_long_edges;
	bool rg_no_rev, rg_lpf, rg_no_par;
	bool rbg_multi, rbg_map;
	int rt_max_deg, rt_max_width;
	bool rst_multi;
	double rst_stprob;

	int layout_menu, generators_menu, options_menu,
		graph_menu, augment_menu, subgraph_menu;

	void gen_pre(int n, int m);
	void gen_post();

	int openPanel(panel &P);
	static void embed_random(const graph& G, node_array<double>& x,
		node_array<double>& y);
	static void gen_random_graph(GraphWin& gw, void (*f)(graph&,int,int), string s);
	static void gen_random_graph1(GraphWin& gw, void (*f)(graph&,int), string s);

	static void random_bic_graph (graph& G, int num_nodes);
	static void make_simple (graph& G);
	static void random_graph_2 (graph& G, int num_nodes, int num_edges);
	static void random_graph_3 (graph& G, int num_nodes, int num_edges);
	static void set_N(node v, bool b, const graph& G, node_array<bool>& N);
	static void random_four_graph (graph& G, int n, int m);
};


enum argType { endProg, noLoad, loadGw, loadGml };

argType process_args(int argc, char *argv[], string &filename);


//-------------------------------------------------------------------
//
//                         GraphWinAGD.c
//
// This program demonstrates the layout algorithms of the
// AGD-Library unsing LEDA's GraphWin as graph editor.
//
//    authors: David Alberts, Carsten Gutwenger, Gunnar Klau
//    date:    decembre 1999
//-------------------------------------------------------------------

#include <AGD/GraphWinInterface.h>
#include <AGD/GraphWinAnimInterface.h>
#include <AGD/AGDLayout.h>
#include <AGD/GraphCopyGLI.h>
#include <AGD/GraphGLI.h>

#include <AGD/DfsRanking.h>
#include <AGD/BarycenterHeuristic.h>
#include <AGD/MedianHeuristic.h>
#include <AGD/WeightedMedianHeuristic.h>
#include <AGD/GreedyInsertHeuristic.h>
#include <AGD/GreedySwitchHeuristic.h>
#include <AGD/SplitHeuristic.h>
#include <AGD/SiftingHeuristic.h>

#include <AGD/SubgraphPlanarizer.h>
#include <AGD/LeveledGraph.h>

#include <AGD/MaxAllMapper.h>
#include <AGD/MaxEachMapper.h>
#include <AGD/FixedGridMapper.h>
#include <AGD/FixedSizeMapper.h>

#include <AGD/TopologyModule.h>

#include <AGD/ShortestPathInserter.h>

//-------------------------------------------------------------------
//                 processing of command line arguments
//-------------------------------------------------------------------

argType process_args(int argc, char *argv[], string &filename)
{
	if (argc > 1) {
		string opts = argv[1];

		if (opts == "-h") {
			cout << "agd_demo [opts]" << endl;
			cout << "agd_opt_demo [opts]" << endl << endl;
			cout << "possible options:" << endl;
			cout << "  -h           shows this help" << endl;
			cout << "  -v           shows version information" << endl;
			cout << "  -gw  <name>  loads file <name> in gw-format" << endl;
			cout << "  -gml <name>  loads file <name> in gml-format" << endl;
			cout << "if no options are given, GraphWin starts with empty graph."<< endl;
			return endProg;

		} else if (opts == "-v") {
			cout << "AGD " << AGD::version_string() <<
				", LEDA " << AGD::version2string(__LEDA__) <<
				", GraphWin " << GraphWin::version() << endl;
			return endProg;

		} else if (opts == "-gw") {
			if (argc == 2) {
				cout << "missing filename!" << endl;
				return endProg;
			}
			filename = argv[2];
			return loadGw;

		} else if (opts == "-gml") {
			if (argc == 2) {
				cout << "missing filename!" << endl;
				return endProg;
			}
			filename = argv[2];
			return loadGml;

		} else {
			filename = opts;
			return loadGw;
		}
	}

	return noLoad;
}

//-------------------------------------------------------------------
//                 main()-function for agd_demo
//-------------------------------------------------------------------

int agd_demo_main(int argc, char *argv[])
{
	string filename;
	argType opt = process_args(argc,argv,filename);

	if (opt != endProg) {
		GraphWinAGD GW;
		GW.set_frame_label("AGD Demo");
		GW.display();

		switch(opt) {
		case loadGw:
			if (GW.read_gw(filename) != 0)
				cout << "couldn't open file " << filename << " !" << endl;
			GW.zoom_graph();
			break;

		case loadGml:
			if (GW.read_gml(filename) != 0)
				cout << "couldn't open file " << filename << " !" << endl;
			GW.zoom_graph();
			break;

                default:
			break;
		}

		GW.edit();
	}

	return 0;
}


//-------------------------------------------------------------------
//                            constructor
//-------------------------------------------------------------------

GraphWinAGD::GraphWinAGD()
{
	// set some  GraphWin options to appropriate values
	set_zoom_objects(true);
	set_node_shape (rectangle_node);

	// create menues for AGD
	layout_menu     = add_menu ("AGD Layout");

	add_simple_call (callPlanarizationLayout, planarization.name(), layout_menu);

	int planar_poly_layout_menu     = add_menu ("Planar Polyline", layout_menu);
	add_simple_call (callPureOrthoLayout, pure_ortho.name(), planar_poly_layout_menu);
	add_simple_call (callQuodLayout, quod.name(), planar_poly_layout_menu);
	add_simple_call (callOrthoLayout, ortho.name(), planar_poly_layout_menu);
	add_separator(planar_poly_layout_menu);
	add_simple_call (callMixedModelLayout, mixed.name(), planar_poly_layout_menu);

	int planar_straight_layout_menu = add_menu ("Planar Straight-Line", layout_menu);
	add_simple_call (callFppLayout, fpp.name(), planar_straight_layout_menu);
	add_simple_call (callSchnyderLayout, schnyder.name(),
		planar_straight_layout_menu);
	add_separator (planar_straight_layout_menu);
	add_simple_call (callPlanarStraightLayout, straight.name(),
		planar_straight_layout_menu);
	add_simple_call (callPlanarDrawLayout, planar_draw.name(),
		planar_straight_layout_menu);
	add_separator (planar_straight_layout_menu);
	add_simple_call (callConvexLayout, convex.name(), planar_straight_layout_menu);
	add_simple_call (callConvexDrawLayout, convex_draw.name(),
		planar_straight_layout_menu);
	add_separator (planar_straight_layout_menu);
	add_simple_call (callVisibilityRepresentation, visib_rep.name(),
		planar_straight_layout_menu);

	add_separator(layout_menu);
	add_simple_call (callSugiyamaLayout, sugiyama.name(), layout_menu);

	int force_layout_menu         = add_menu ("Force-Directed", layout_menu);
	add_simple_call (callSpringLayout, spring.name(), force_layout_menu);
	add_simple_call (callTutteLayout, tutte.name(), force_layout_menu);

	int specialized_layout_menu     = add_menu ("Specialized", layout_menu);
	add_simple_call (callTreeLayout, tree_layout.name(), specialized_layout_menu);
	add_simple_call (callStPlanarGrid, "dominance layout", specialized_layout_menu);
	add_simple_call (callStPlanar, "st-planar", specialized_layout_menu);

	// create menu entries for graph generators
	generators_menu = add_menu ("AGD Generators");
	add_simple_call (callRandomPlanarGraph, "planar", generators_menu);
	add_simple_call (callRandomPlanarBicGraph, "planar biconnected",
		generators_menu);
	add_simple_call (callRandomPlanarTricGraph, "planar triconnected",
		generators_menu);
	add_separator(generators_menu);
	add_simple_call (callRandomGraph, "random", generators_menu);
	add_simple_call (callRandomGraph2, "random (2)", generators_menu);
	add_separator(generators_menu);
	add_simple_call (callRandomFourGraph, "four-graph", generators_menu);
	add_separator(generators_menu);
	add_simple_call (callRandomTree, "tree", generators_menu);
	add_simple_call (callRandomStPlanarGraph, "st-planar", generators_menu);
	add_simple_call (callRandomHierarchy, "hierarchical", generators_menu);

	// create menu entries for augmentation and subgraph algorithms
	graph_menu      = add_menu ("AGD Graph");
	augment_menu    = add_menu ("Augmentation", graph_menu);
	subgraph_menu   = add_menu ("Subgraph", graph_menu);
	add_simple_call (callMakeBic, "biconnected", augment_menu);
	add_simple_call (callPlanAug, "planar biconnected", augment_menu);
	add_simple_call (callPlanSub, "planar", subgraph_menu);
	add_simple_call (callNaiveMaximalPlanSub, "maximal planar", subgraph_menu);
	add_simple_call (callMakeAcyclic, "maximal acyclic", subgraph_menu);
	add_simple_call (callGreedyCycleRemoval, "GreedyCycleRemoval", subgraph_menu);
	add_simple_call (callReducedDigraph, "ReducedDigraph", subgraph_menu);

	// create menu entries for setting optional parameters
	options_menu    = add_menu ("AGD Options");

	add_simple_call (callOptionsPlanarization,"Planarization",options_menu );

	int planar_poly_options_menu   = add_menu ("Planar Polyline", options_menu);
	add_simple_call (callOptionsPureOrtho,  "PureOrthogonalLayout", planar_poly_options_menu);
	add_simple_call (callOptionsQuod,       "QuasiOrthogonalLayout", planar_poly_options_menu);
	add_simple_call (callOptionsOrtho,      "OrthogonalLayout", planar_poly_options_menu);
	add_separator(planar_poly_options_menu);
	add_simple_call (callOptionsMixedModel, "Mixed-Model",    planar_poly_options_menu);

	int planar_straight_options_menu = add_menu ("Planar Straight-Line", options_menu);
	add_simple_call (callOptionsConvex,     "Convex",         planar_straight_options_menu);
	add_simple_call (callOptionsConvexDraw, "ConvexDraw",     planar_straight_options_menu);
	add_separator(planar_straight_options_menu);
	add_simple_call (callOptionsStraight,   "PlanarStraight", planar_straight_options_menu);
	add_simple_call (callOptionsPlanarDraw, "PlanarDraw",     planar_straight_options_menu);

	add_simple_call (callOptionsSugiyama,   "Sugiyama",       options_menu );

	int force_options_menu = add_menu ("Force-Directed", options_menu);
	add_simple_call (callOptionsSpring,     "Spring Embedder",force_options_menu);

	int specialized_options_menu = add_menu ("Specialized", options_menu);
	add_simple_call (callOptionsTreeLayout, "Tree Layout",  specialized_options_menu);
	add_simple_call (callOptionsStPlanarGrid, "dominance layout", specialized_options_menu);
	add_simple_call (callOptionsStPlanar, "st-planar", specialized_options_menu);

	add_separator(options_menu);
	int augment_options_menu = add_menu ("Augmentation", options_menu);
	add_simple_call (callOptionsPlanAug,"planar biconnected",augment_options_menu );

	int subgraph_options_menu = add_menu ("Subgraph", options_menu);
	add_simple_call (callOptionsPlanSub,"planar",subgraph_options_menu );

	add_simple_call (callOptionsGlobal,"Global",options_menu );

	int help_menu = get_menu("Help");
	add_separator(help_menu);
	add_simple_call (callHelpAbout, "About AGD", help_menu);
	
	the_mapper = 0;
	the_min_over_x = 5;  the_min_over_y = 5;
	the_sep_x      = 15; the_sep_y      = 15;
	the_pres_embedding = true;

	quod.verify(false);
	ortho.verify(false);
	pure_ortho.verify(false);
	the_compactor = 1;

	tree_layout.sib_separation(20.0);
	tree_layout.subtree_separation(20.0);
	tree_layout.level_separation(20.0);

	fhl.node_distance(20.0);
	fhl.layer_distance(20.0);
	sugiyama.set_comp_coord(fhl);

	planarization.set_planar_layouter(quod);
	planarization_layouter = 0; planarization_planarizer = 0;
	planarization_inserter = 0;

	sug_ranking = 0; sug_cross_min = 0; sug_cycle_removal = 0; sug_layout = 0;

	rh_planar = false; rh_single = false; rh_long_edges = true;
	rg_no_rev = true; rg_lpf = true; rg_no_par = true;
	rbg_multi = false; rbg_map = false;
	rt_max_deg = 0; rt_max_width = 0;
	rst_multi = false; rst_stprob = 0.5;
}


//-------------------------------------------------------------------
//            build error message for violated property
//-------------------------------------------------------------------

string GraphWinAGD::get_error_message(AgdKey p)
{
	if (p == key::planar)
		return "The graph is not planar.";
	if (p == key::simple)
		return "The graph ist not simple.";
	if (p == key::connected)
		return "The graph is not connected.";
	if (p == key::biconnected)
		return "The graph is not biconnected.";
	if (p == key::triconnected)
		return "The graph is not triconnected.";
	if (p == key::four_graph)
		return "The graph is not a four-graph.";
	if (p == key::no_self_loops)
		return "The graph has self loops.";
	if (p == key::directed)
		return "The graph is not directed.";
	if (p == key::undirected)
		return "The graph is not undirected.";
	if (p == key::no_reversals)
		return "The graph has reversal edges.";
	if (p == key::acyclic)
		return "The graph is not acyclic.";
	if (p == key::tree)
		return "The graph does not represent a tree.";
	if (p == key::forest)
		return "The graph does not represent a forest.";
	if (p == key::single_source)
		return "The graph has not a single source.";
	if (p == key::single_sink)
		return "The graph has not a single sink.";
	if (p == key::bipartite)
		return "The graph is not bipartite.";
	if (p == key::st_graph)
		return "The graph is not an st-graph.";
	if (p == key::st_planar)
		return "The graph is not st-planar.";
	return "property <" + p.info() + "> is not satisfied.";
}


//-------------------------------------------------------------------
//                 generic calling functions
//-------------------------------------------------------------------

bool GraphWinAGD::callLayout (GraphWinAGD& GW, LayoutModule& L) {
	AgdKey p;
	GraphWinAnimInterface A (GW); // an interface for our GraphWin
	const graph &G = GW.get_graph();

	// can L draw the graph displayed in GW ?
	if (L.check(G, p)) {
		cout << "\ncalling " << L.name() << endl;
		L.call (G, A); // yes, we can call L
		GW.fill_window();

		return true;
	} else {
		GW.acknowledge (get_error_message(p)); // no, so we display the error message
	}

	return false;
}

bool GraphWinAGD::callGridLayout (GraphWinAGD& GW, GridLayoutModule& L)
{
	AgdKey p;
	graph &G = GW.get_graph();

	if (L.check(G, p)) {
		cout << "calling " << L.name() << endl;

		GraphWinAnimInterface gwi(GW);

		face f0 = nil;
		TopologyModule T;
		PlaneGraphCopy *pg = nil;
		GridLayoutInfo *gli = nil;
		GridLayout gl;

		if (GW.the_pres_embedding &&
			L.postcondition(0).member(key::PlaneGraphCopy_callable) && T.check(G, p))
		{
			pg = &T.call(G, gwi, f0);
			L.call_grid(*pg, f0, gl);
			gli = new GraphCopyGLI(gl,*pg);

		} else {
			L.call_grid(G,gl);
			gli = new GraphGLI(gl,L.bounding_box());
		}

		IRect ir = L.bounding_box();
		cout << "grid size: " << ir.width() << " * " << ir.height() << endl;

		int gs = 2*GW.get_grid_dist();
		if (GW.the_mapper == 3 && gs > 0) {
			FixedSizeMapper M(gs,gs,
				GW.the_min_over_x,GW.the_min_over_y,
				GW.the_sep_x, GW.the_sep_y);

			M.call(*gli,gwi);

		} else if (GW.the_mapper == 2 && gs > 0) {
			FixedGridMapper M(gs,gs,
				GW.the_min_over_x,GW.the_min_over_y,
				GW.the_sep_x, GW.the_sep_y);

			M.call(*gli,gwi);

		} else if (GW.the_mapper == 1) {
			MaxEachMapper M(40,40,GW.the_min_over_x,GW.the_min_over_y,
				GW.the_sep_x, GW.the_sep_y);

			M.call(*gli,gwi);

		} else {
			MaxAllMapper M(40,GW.the_min_over_x,GW.the_sep_x);

			M.call(*gli,gwi);
		}

		delete pg;
		delete gli;

		GW.set_flush(true);
		GW.zoom_graph();
		
		return true;

	} else {
		GW.acknowledge (get_error_message(p));
	}

	return false;
}


bool GraphWinAGD::callGridLayout2 (GraphWinAGD& GW, GridLayoutModule& L)
{
	AgdKey p;
	graph &G = GW.get_graph();

	if (L.check(G, p)) {
		cout << "calling " << L.name() << endl;

		GraphWinAnimInterface gwi(GW);

		GridLayout gl(G);
		L.call_grid(G, gl);

		IRect ir = L.bounding_box();
		cout << "grid size: " << ir.width() << " * " << ir.height() << endl;

		GraphGLI gli(gl);

		int gs = 2*GW.get_grid_dist();
		if (GW.the_mapper == 3 && gs > 0) {
			FixedSizeMapper M(gs,gs,
				GW.the_min_over_x,GW.the_min_over_y,
				GW.the_sep_x, GW.the_sep_y);

			M.call(gli,gwi);

		} else if (GW.the_mapper == 2 && gs > 0) {
			FixedGridMapper M(gs,gs,
				GW.the_min_over_x,GW.the_min_over_y,
				GW.the_sep_x, GW.the_sep_y);

			M.call(gli,gwi);

		} else if (GW.the_mapper == 1) {
			MaxEachMapper M(40,40,GW.the_min_over_x,GW.the_min_over_y,
				GW.the_sep_x, GW.the_sep_y);

			M.call(gli,gwi);

		} else {
			MaxAllMapper M(40,GW.the_min_over_x,GW.the_sep_x);

			M.call(gli,gwi);
		}

		GW.set_flush(true);
		GW.zoom_graph();
		
		return true;
	} else {
		GW.acknowledge (get_error_message(p)); // no, so we display the error message
	}

	return false;
}


bool GraphWinAGD::callAugment (GraphWinAGD& GW, AugmentationModule& A) {
	AgdKey p;

	// can A be called for the graph displayed in GW ?
	if (A.check(GW.get_graph(), p)) {
		cout << "\ncalling " << A.name() << endl;

		list<edge> e_list;
		edge e;

		A.call (GW.get_graph(), e_list); // yes, we can call A

		GW.update_graph();

		GW.deselect_all_edges();
		forall (e, e_list)
			GW.select(e);

		cout << A.num_added_edges() << " edge(s) added." << endl;

		return true;

	} else {
		GW.acknowledge (get_error_message(p)); // no, so we display the error message
	}

	return false;
}


bool GraphWinAGD::callSubgraph (GraphWinAGD& GW, SubgraphModule& S)
{
	graph &G = GW.get_graph();
	AgdKey p;

	// can S be called for the graph displayed in GW ?
	if (S.check(G, p)) {
		cout << "\ncalling " << S.name() << endl;
		S.call (G); // yes, we can call S
		GW.update_graph();

		cout << S.num_del_edges() << " edge(s) deleted." << endl;

		return true;

	} else {
		GW.acknowledge (get_error_message(p)); // no, so we display the error message
	}

	return false;
}


//-------------------------------------------------------------------
//                 setting options of algorithms
//-------------------------------------------------------------------

int GraphWinAGD::openPanel(panel &P)
{
	int xleft,xright,ytop,ybot;

	get_window().frame_box(xleft,ytop,xright,ybot);

	//Modification Thomas Ziegler 4.4.2001 due to problems with CC5.0
	//on sparc solaris
	/*
	if (xleft+xright > window::screen_width())
		return P.open(-(xleft-1),ytop);
	else
		return P.open(xright+1,ytop);
		*/
	return P.open(0,0);
}

void GraphWinAGD::optionsOvid(Ovid& L) {

	int  output_level   = L.output_level();
	bool verify         = L.verify();
	bool traditional    = L.traditional();
	bool compact_layout = L.compact_layout();
	int compactor = the_compactor; 

	panel P ("Options for " + L.name());

	P.int_item  ("output level",  output_level);
	P.bool_item ("verify",  verify);
	P.bool_item ("traditional", traditional);
	P.choice_item ("compaction", compactor, "longest paths", "flow");
	P.bool_item("compact further", compact_layout);

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");

	int result = openPanel(P);
	if (result != cancel) {

		L.output_level(output_level);
		L.verify (verify);
		L.traditional(traditional);
		L.compact_layout(compact_layout);

		the_compactor = compactor;
		switch (compactor) {
			case 0: L.set_compactor(C_longest_paths); break;
			case 1: L.set_compactor(C_flow); break;
		}
	}

	if (result == call) {
		callGridLayout(*this,L);
	}
}

void GraphWinAGD::optionsPlanSub()
{
	int runs=4;

	panel P ("Options for Planar Subgraph");
	P.choice_item("Algorithm:",planarization_planarizer,"PlanarSubgraph", "NaiveMaximalPlanarSubgraph");
	P.int_item ("runs:", runs);

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");
	int result = openPanel(P);

	if (result != cancel) {
		if (planarization_planarizer==0) {
			//cout << "plan_sub.runs(" << runs <<")" << endl; 
			plan_sub.runs(runs);
		}
		else {
			//cout << "naive_maximal_plan_sub.runs(" << runs <<")" << endl; 
			naive_maximal_plan_sub.runs(runs);
		}
	}

	if (result == call) {
		if (planarization_planarizer==0) {
			//cout << "callPlanSub(*this)" << endl;
			callPlanSub(*this);
		}
		else {
			//cout << "callNaiveMaximalPlanSub(*this)" << endl;
			callNaiveMaximalPlanSub(*this);
		}
	}
}

void GraphWinAGD::optionsPlanAug()
{
	bool size_strategy = plan_aug.size_strategy();
	bool use_blabels = plan_aug.use_blabels();

	panel P ("Options for Planar Biconnected");

	P.bool_item ("size strategy:", size_strategy);
	P.bool_item ("use b-labels:", use_blabels);

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");

	int result = openPanel(P);
	if (result != cancel) {
		plan_aug.size_strategy(size_strategy);
		plan_aug.size_strategy(use_blabels);
	}
	if (result == call) {
		callPlanAug(*this);
	}
}

void GraphWinAGD::optionsPlanarization()
{
	int layouter = planarization_layouter;
	int inserter = planarization_inserter;
	int planarizer= planarization_planarizer;

	panel P ("Options for Planarization");
	P.choice_item ("Layout:", layouter, "QuasiOrtho", "Ortho", "Mixed-Model");
	P.choice_item("Algorithm:",planarizer,"PlanarSubgraph", "NaiveMaximalPlanarSubgraph");
	P.choice_item("Edge Reinsertion: ",inserter, "Basic", "Shortest First", "Remove and Reinsert", "Extended Remove and Reinsert");
	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");
	int result = openPanel(P);

	if (result != cancel) {
		planarization_planarizer=planarizer;
		SubgraphPlanarizer subPlan;
		switch (planarizer) {
		case 0:
			//cout << "subPlan.set_subgraph(plan_sub)" << endl;
			subPlan.set_subgraph(plan_sub);
			break;
		case 1:
			//cout << "subPlan.set_subgraph(naive_maximal_plan_sub);" << endl;
			subPlan.set_subgraph(naive_maximal_plan_sub);
			break;
		}

		planarization_inserter=inserter;
		ShortestPathInserter shortest_path_inserter;
		switch (inserter) {
		case 0:
			//cout << "shortest_path_inserter.set_to_basic()" << endl;
			shortest_path_inserter.set_to_basic();
			break;
		case 1:
			//cout << "shortest_path_inserter.set_to_shortest_first()" << endl;
			shortest_path_inserter.set_to_shortest_first();
			break;
		case 2:
			//cout << "shortest_path_inserter.set_to_rr()" << endl;
			shortest_path_inserter.set_to_rr();
			break;
		case 3:
			//cout << "shortest_path_inserter.set_to_err()" << endl;
			shortest_path_inserter.set_to_err();
			break;
		}
		subPlan.set_inserter(shortest_path_inserter);

		planarization.set_planarizer(subPlan);

		planarization_layouter = layouter;
		switch (layouter) {
		case 0:
			//cout << "planarization.set_planar_layouter(quod)" << endl;
			planarization.set_planar_layouter(quod);
			break;
		case 1:
			//cout << "planarization.set_planar_layouter(ortho)" << endl;
			planarization.set_planar_layouter(ortho);
			break;
		case 2:
			//cout << "planarization.set_planar_layouter(mixed)" << endl;
			planarization.set_planar_layouter(mixed);
			break;
		}
	}
	if (result == call) {
		//cout << "callPlanarizationLayout(*this)" << endl;
		callPlanarizationLayout(*this);
	}
}

void GraphWinAGD::optionsConvex()
{
	bool   size_opt   = convex.size_opt();
	double base_ratio = convex.base_ratio();

	panel P ("Options for Convex");

	P.bool_item ("size_opt:",   size_opt);
	P.real_item ("base_ratio:", base_ratio);

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");

	int result = openPanel(P);
	if (result != cancel) {
		convex.size_opt   (size_opt);
		convex.base_ratio (base_ratio);
	}
	if (result == call) {
		callConvexLayout(*this);
	}
}

void GraphWinAGD::optionsConvexDraw()
{
	bool   size_opt   = convex_draw.size_opt();
	bool   side_opt   = convex_draw.side_opt();
	double base_ratio = convex_draw.base_ratio();

	panel P ("Options for ConvexDraw");

	P.bool_item ("size_opt:",   size_opt);
	P.bool_item ("side_opt:",   side_opt);
	P.real_item ("base_ratio:", base_ratio);

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");

	int result = openPanel(P);
	if (result != cancel) {
		convex_draw.size_opt   (size_opt);
		convex_draw.side_opt   (side_opt);
		convex_draw.base_ratio (base_ratio);
	}
	if (result == call) {
		callConvexDrawLayout(*this);
	}
}


void GraphWinAGD::optionsStraight()
{
	bool   size_opt   = straight.size_opt();
	bool   max_outer  = straight.max_outer();
	double base_ratio = straight.base_ratio();

	panel P ("Options for PlanarStraight");

	P.bool_item ("size_opt:",   size_opt);
	P.bool_item ("max_outer:",  max_outer);
	P.real_item ("base_ratio:", base_ratio);

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");

	int result = openPanel(P);
	if (result != cancel) {
		straight.size_opt   (size_opt);
		straight.max_outer  (max_outer);
		straight.base_ratio (base_ratio);
	}
	if (result == call) {
		callPlanarStraightLayout(*this);
	}
}

void GraphWinAGD::optionsPlanarDraw()
{
	bool   size_opt   = planar_draw.size_opt();
	bool   side_opt   = planar_draw.side_opt();
	bool   max_outer  = planar_draw.max_outer();
	double base_ratio = planar_draw.base_ratio();

	panel P ("Options for PlanarDraw");

	P.bool_item ("size_opt:",   size_opt);
	P.bool_item ("side_opt:",   side_opt);
	P.bool_item ("max_outer:",  max_outer);
	P.real_item ("base_ratio:", base_ratio);

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");

	int result = openPanel(P);
	if (result != cancel) {
		planar_draw.size_opt   (size_opt);
		planar_draw.side_opt   (side_opt);
		planar_draw.max_outer  (max_outer);
		planar_draw.base_ratio (base_ratio);
	}
	if (result == call) {
		callPlanarDrawLayout(*this);
	}
}

void GraphWinAGD::optionsMixedModel()
{
	//int    co_type      = (int)mixed.co_type();
	double base_ratio   = mixed.base_ratio();

	panel P ("Options for Mixed-Model");

	//P.choice_item ("co_type:", co_type, "triconnected", "biconnected");
	P.real_item ("base ratio:", base_ratio);

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");

	int result = openPanel(P);
	if (result != cancel) {
		//mixed.co_type      ((CanonicalOrderType)(co_type));
		mixed.base_ratio   (base_ratio);
	}
	if (result == call) {
		callMixedModelLayout(*this);
	}
}

void GraphWinAGD::optionsGlobal()
{
	bool   pres_embedding = the_pres_embedding;
	int    mapper         = the_mapper;
	double min_over_x     = the_min_over_x;
	double min_over_y     = the_min_over_y;
	double sep_x          = the_sep_x;
	double sep_y          = the_sep_y;

	panel P ("Global Options");

	P.text_item("\\bf\\blue Grid Mapper");
	P.choice_item ("mapper:", mapper, "MaxAll", "MaxEach", "FixedGrid", "FixedSize");
	P.real_item ("min. overhang (x-coord.):", min_over_x);
	P.real_item ("min. overhang (y-coord.):", min_over_y);
	P.real_item ("separation (x-coord.):", sep_x);
	P.real_item ("separation (y-coord.):", sep_y);

	P.text_item("\\bf\\blue Plane Algorithms");
	P.bool_item ("preserve embedding:",  pres_embedding);

	P.button ("Ok");
	int cancel = P.button ("Cancel");

	if (P.open() != cancel) {
		the_pres_embedding = pres_embedding;
		the_mapper         = mapper;
		the_min_over_x     = min_over_x;
		the_min_over_y     = min_over_y;
		the_sep_x          = sep_x;
		the_sep_y          = sep_y;
	}
}


void GraphWinAGD::optionsSugiyama()
{
	int ranking       = sug_ranking;
	int cross_min     = sug_cross_min;
    //  int layout        = sug_layout;
	int cycle_removal = sug_cycle_removal;
	int width = cgr.width();
	int  runs       = sugiyama.runs();
	int  fails      = sugiyama.fails();
	bool init_order = sugiyama.init_order();
	bool transpose  = sugiyama.transpose();

	double node_dist      = fhl.node_distance();
	double level_dist     = fhl.layer_distance();
	bool fixed_level_dist = fhl.fixed_layer_distance();

	panel P ("Options for Sugiyama");

	P.text_item("\\bf\\blue Rank Assignment");
	P.choice_item ("method:", ranking, "Longest-Path", "Coffman-Graham", "DFS");
	P.int_item ("width:", width);
	P.choice_item ("cycle removal:", cycle_removal, "DFS", "Greedy");

	P.text_item("\\bf\\blue Crossing Minimization");
	P.choice_item ("method:", cross_min, "Barycenter", "Median", "Weighted-Median", "Split", "Sifting");
	P.int_item ("runs:", runs);
	P.int_item ("fails:", fails);
	P.bool_item ("initial order:", init_order);
	P.bool_item ("transpose:", transpose);

	P.text_item("\\bf\\blue Computation of Coordinates");
	P.real_item ("node distance:", node_dist);
	P.real_item ("level distance:", level_dist);
	P.bool_item ("fixed level distance:", fixed_level_dist);

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");

	int result = openPanel(P);
	if (result != cancel) {
		sugiyama.runs(runs);
		sugiyama.fails(fails);
		sugiyama.init_order(init_order);
		sugiyama.transpose(transpose);

		cgr.width(width);

		fhl.node_distance(node_dist);
		fhl.layer_distance(level_dist);
		fhl.fixed_layer_distance(fixed_level_dist);

		sugiyama.set_comp_coord(fhl);

		const SubgraphModule* subgraph = 0;

		sug_cycle_removal = cycle_removal;
		switch(cycle_removal) {
		case 0:
			subgraph = &make_acyclic;
			break;
		case 1:
			subgraph = &greedy_cycle_removal;
			break;
		}
		
		sug_ranking = ranking;
		switch (ranking) {
		case 0:
			hierarchical.set_subgraph(*subgraph);
			sugiyama.set_ranking(hierarchical);
			break;
		case 1:
			cgr.set_subgraph(*subgraph);
			sugiyama.set_ranking(cgr);
			break;
		case 2:
			sugiyama.set_ranking(DfsRanking());
			break;
		}
		
		sug_cross_min = cross_min;
		switch (cross_min) {
		case 0:
			sugiyama.set_cross_min(BarycenterHeuristic());
			break;
		case 1:
			sugiyama.set_cross_min(MedianHeuristic());
			break;
		case 2:
			sugiyama.set_cross_min(WeightedMedianHeuristic());
			break;
		case 3:
			sugiyama.set_cross_min(SplitHeuristic());
			break;
		case 4:
			sugiyama.set_cross_min(SiftingHeuristic());
			break;
		}
	}
	if (result == call) {
		callSugiyamaLayout(*this);
	}
}

void GraphWinAGD::optionsSpring()
{
	bool   noise = spring.noise();
	int    iter  = spring.iterations();

	panel P ("Options for Spring Embedder");

	P.bool_item ("noise:", noise);
	P.int_item  ("iterations:", iter);

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");

	int result = openPanel(P);
	if (result != cancel) {
		spring.noise (noise);
		spring.iterations (iter);
	}
	if (result == call) {
		callSpringLayout(*this);
	}
}


void GraphWinAGD::optionsTreeLayout()
{
	double sib_separation     = tree_layout.sib_separation();
	double level_separation   = tree_layout.level_separation();
	double subtree_separation = tree_layout.subtree_separation();

	panel P ("Options for TreeLayout");

	P.double_item ("sib_separation:", sib_separation);
	P.double_item ("level_separation:", level_separation);
	P.double_item ("subtree_separation:", subtree_separation);

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");

	int result = openPanel(P);
	if (result != cancel) {
		tree_layout.sib_separation(sib_separation);
		tree_layout.level_separation(level_separation);
		tree_layout.subtree_separation(subtree_separation);
	}
	if (result == call) {
		callTreeLayout(*this);
	}
}

void GraphWinAGD::optionsStPlanarGrid()
{
	int compaction = (int)st_planar_grid.compaction();

	panel P ("Options for StPlanarGridLayout");

	P.choice_item ("compaction:", compaction, "none", "symmetry preserving", "minimum area");

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");

	int result = openPanel(P);
	if (result != cancel) {
		st_planar_grid.compaction((StPlanarGridLayout::CompactionType)compaction);
	}
	if (result == call) {
		callStPlanarGrid(*this);
	}
}

void GraphWinAGD::optionsStPlanar()
{
	int compaction = (int)st_planar.compaction();
	double separation = st_planar.separation();

	panel P ("Options for StPlanarLayout");

	P.choice_item ("compaction:", compaction, "none", "symmetry preserving", "minimum area");
	P.real_item("separation:", separation);

	P.button ("Ok");
	int call   = P.button ("Call");
	int cancel = P.button ("Cancel");

	int result = openPanel(P);
	if (result != cancel) {
		st_planar.compaction((StPlanarGridLayout::CompactionType)compaction);
		st_planar.separation(separation);
	}
	if (result == call) {
		callStPlanar(*this);
	}
}


void GraphWinAGD::callHelpAbout(GraphWin& GW)
{
	panel P;

	P.text_item("\\bf\\blue AGD-Demo");
	P.text_item("");
	P.text_item("");
	P.text_item("");
	P.text_item("This demo program demonstrates the layout algorithms \
of AGD for graphs using LEDA's graph editor GraphWin.");
	P.text_item("");
	P.text_item("");
	P.text_item("AGD version: " + AGD::version_string());
	P.text_item("");
	P.text_item("LEDA version: " + AGD::version2string(__LEDA__));
	P.text_item("");
	P.text_item(string("GraphWin version: %lf", GraphWin::version()));
	P.text_item("");
	P.text_item("");
	P.text_item("");
	P.text_item("AGD is distributed by the Algorithmic Solutions Software GmbH.");

	P.button("ok");

	GW.open_panel(P);
}


//-------------------------------------------------------------------
//                      creating random graphs
//-------------------------------------------------------------------

//LeveledGraph GraphWinAGD::LG;

//int GraphWinAGD::anim_status;
//bool GraphWinAGD::rh_planar = false, GraphWinAGD::rh_single = false, GraphWinAGD::rh_long_edges = true;
//bool GraphWinAGD::rg_no_rev = true, GraphWinAGD::rg_lpf = true, GraphWinAGD::rg_no_par = true;
//bool GraphWinAGD::rbg_multi = false, GraphWinAGD::rbg_map = false;
//int GraphWinAGD::rt_max_deg = 0, GraphWinAGD::rt_max_width = 0;

/*
void GraphWinAGD::wait (GraphWinAGD& GW, panel& P, float delay, int QUIT)
{
	float T1, T2;
	int val;
	double x, y;

	used_time (T1);
	do {
		T2 = T1;
		P.get_event(val,x,y);
	} while (used_time(T2) < delay && GW.anim_status != QUIT);
}

void GraphWinAGD::anim_action (int n)
{
	anim_status = n;
}*/
/*
void GraphWinAGD::show_results(const AGDModule& M, window& results)
{
	if (&M == &sugiyama && sugiyama_infos) {
		results.del_messages();
		results.message ("Used heuristic: " +
			sugiyama.name_of_heuristic(sugiyama.used_heuristic()));
		results.message ("Number of crossings: " +
			sugi_to_string(sugiyama.number_of_crossings()));
		results.message ("Number of runs: " +
			sugi_to_string(sugiyama.number_of_runs()));
		results.message( "Next fixed layer: " +
			sugi_to_string(sugiyama.fixed_layer()));
		results.message("Next changeable layer: " +
			sugi_to_string(sugiyama.changeable_layer()));
	}
}*/
/*
void GraphWinAGD::animate(GraphWinAGD& GW, LayoutModule& L)
{
	//GraphWinAGD &gwAGD = (GraphWinAGD&)GW;
	//SugiyamaLayout &sugi = GW.sugiyama;
	graph& G = GW.get_graph();
	GraphWinInterface gwi(GW);

	string msg;
	if (L.check(G,msg) == false) {
		GW.acknowledge(msg);
		return;
	}

	// the graph may not get changed during animation!
	GW.disable_calls();

	// first call start_animation, if false is returned,
	// animation is not possible
	if (L.start_animation(G,gwi) == true) {

		// create a control panel for animation
		panel P("Animation Control Panel");

		P.buttons_per_line (5);
		const int
			SINGLE = P.button ("SINGLE", &anim_action),
			PLAY   = P.button ("PLAY"  , &anim_action),
			FAST   = P.button ("FAST"  , &anim_action),
			STOP   = P.button ("STOP"  , &anim_action),
			QUIT   = P.button ("QUIT"  , &anim_action);
		const float delay = 0.5f, delay_fast = 0.15f;

		P.display();

		// perform animation until the user presses the Quit button,
		// or no more steps are possible
		bool cont = true;
		anim_status = STOP;
		int val;
		double x, y;

		window results(200.0f, 100.0f, "Results");

		while (cont && anim_status != QUIT) {
			while (P.get_event(val,x,y) != no_event) ;

			if (anim_status == SINGLE || anim_status == PLAY ||
				anim_status == FAST)
			{
				cont = L.next_step();
				fit_in_window (GW, gwi);
				//GW.show_results(L,results);

				if (anim_status == SINGLE)
					anim_status = STOP;
				else {
					wait (GW, P, (anim_status == PLAY) ?
						delay : delay_fast, QUIT);
				}
			}
		}

		// call stop animation only if animation is still possible
		if (cont) {
		  L.stop_animation();
		  fit_in_window( GW, gwi ); // MiK
		}
		if (cont == false && anim_status != QUIT)
			GW.acknowledge ("Animation is finished.");
	}

	// now, we can enable calls again
	GW.enable_calls();
}
*//*
void GraphWinAGD::fit_in_window (GraphWinAGD& GW, LayoutInterface& A)
{
	const double eps = 0.000001;

	double xmin = GW.get_xmin();
	double xmax = GW.get_xmax();
	double ymin = GW.get_ymin();
	double ymax = GW.get_ymax();

	DRect bbox(A.area());

	//A.get_layout_size (bbox);

	if (bbox.left() >= xmin-eps)
		if (bbox.right() <= xmax+eps)
			if (-bbox.bottom() >= ymin-eps)
				if (-bbox.top() <= ymax+eps)
					return;

	double r = (xmax-xmin) / (ymax-ymin);

	double h = bbox.bottom() - bbox.top();

	double xm = leda_max (bbox.right(), h*r+bbox.left());

	GW.win_init (bbox.left(), xm, -bbox.bottom());
}*/

void GraphWinAGD::make_simple (graph& G)
{
	Delete_Loops (G);
	Make_Simple (G);

	edge_array<edge> rev(G);
	Is_Bidirected (G, rev);
	list<edge> del_edges;
	edge e;

	forall_edges (e, G) {
		if (rev[e] != nil) {
			del_edges.append(e);
			rev[rev[e]] = nil;
		}
	}

	forall (e, del_edges) G.del_edge(e);
}

void GraphWinAGD::random_graph_2 (graph& G, int num_nodes, int num_edges)
{
	random_graph (G, num_nodes, num_edges);

	make_simple(G);
}

void GraphWinAGD::random_graph_3 (graph& G, int num_nodes, int num_edges)
{
	random_planar_graph (G, num_nodes, num_edges);

	int i, m = num_edges/10;

	for (i = 1; i <= m; i++)
		G.new_edge(G.choose_node(),G.choose_node());

	make_simple(G);
}

void GraphWinAGD::set_N(node v, bool b, const graph& G, node_array<bool>& N)
{
	edge e;

	forall_inout_edges(e,v)
		N[G.opposite(v,e)] = b;
}

void GraphWinAGD::random_four_graph (graph& G, int n, int m)
{
	G.clear();

	array<node> T(n);
	node_array<bool> N(G,n,false);

	int i, j, k;
	random_source S;

	for (i = 0; i < n; i++)
		T[i] = G.new_node();

	for (k = 1; n >= 1 && k <= m; k++) {
		i = S(0,n-1);
		j = S(0,n-1);

		N[T[i]] = true;
		set_N(T[i],true,G,N);

		if (N[T[j]] == true) {
			int j1 = j;
			do {
				j = (j+1) % n;
			} while (N[T[j]] && j1!= j);
		}
		if (N[T[j]]) T[i] = T[--n];
		else {
			G.new_edge(T[i],T[j]);
			if (degree(T[i]) == 4) T[i] = T[--n];
			if (j >= n) j = i;
			if (degree(T[j]) == 4) T[j] = T[--n];
		}
		N[T[i]] = false;
		set_N(T[i],false,G,N);
	}
}

void GraphWinAGD::callRandomHierarchy (GraphWin& gw1)
{
	GraphWinAGD& gw = (GraphWinAGD&)gw1;
	if (!gw.check_init_graph_handler()) return;

	panel P("Create Random Hierarchy");
	int n = gw.get_gen_nodes(),
		m = gw.get_gen_edges();

	P.int_item ("nodes", n);
	P.int_item ("edges", m);
	P.bool_item ("planar", gw.rh_planar);
	P.bool_item ("single source", gw.rh_single);
	P.bool_item ("long edges", gw.rh_long_edges);
	P.button ("ok", 1);
	P.button ("cancel", 0);

	if (gw.openPanel(P) == 0 || n < 0 || m < 0) return;

	gw.gen_pre(n,m);

	bool old_flush = gw.set_flush(false); 

	random_hierarchy(gw.LG,n,m,gw.rh_planar,gw.rh_single,gw.rh_long_edges);
	gw.set_graph(gw.LG);

	node_array<int> rank(gw.LG);

	node v;
	forall_nodes(v,gw.LG)
		rank [v] = gw.LG.level(v);

	AgdKey p;
	GraphWinInterface A (gw);

	if (gw.sugiyama.check(gw.LG, p)) {
		gw.sugiyama.ext_call (gw.LG, rank, A);
		gw.fill_window();
	}

	gw.set_flush(old_flush);
	gw.redraw();
	gw.call_init_graph_handler();
}

void GraphWinAGD::gen_pre(int n, int m)
{
	set_gen_nodes (n);
	set_gen_edges (m);
	graph& G = get_graph();

	G.clear();
	update_graph();
	//old_flush=set_flush(false); 
	//clear_graph();
}

void GraphWinAGD::gen_post()
{
	graph& G=get_graph();
	bool old_flush = set_flush(false); 

	update_graph();

	node_array<double> x(G);
	node_array<double> y(G);
	embed_random(G,x,y);
	update_graph();
	adjust_coords_to_win(x,y);
	set_layout(x,y);
	embed_edges();
	set_flush(old_flush);
	redraw();
	call_init_graph_handler();
}


void GraphWinAGD::callRandomGraph(GraphWin &gw1)
{
	GraphWinAGD& gw = (GraphWinAGD&)gw1;
	if (!gw.check_init_graph_handler()) return;

	panel P("Create Random Graph");
	int n = gw.get_gen_nodes(),
		m = gw.get_gen_edges();
	
	P.int_item ("nodes", n);
	P.int_item ("edges", m);
	P.bool_item ("no reversals", gw.rg_no_rev);
	P.bool_item ("no self-loops", gw.rg_lpf);
	P.bool_item ("no parallel edges", gw.rg_no_par);

	P.button ("ok", 1);
	P.button ("cancel", 0);

	if (gw.openPanel(P) == 0 || n < 0 || m < 0) return;

	gw.gen_pre(n,m);
	random_graph (gw.get_graph(), n, m, gw.rg_no_rev, gw.rg_lpf, gw.rg_no_par);
	gw.gen_post();
}

void GraphWinAGD::callRandomTree(GraphWin &gw1)
{
	GraphWinAGD& gw = (GraphWinAGD&)gw1;
	if (!gw.check_init_graph_handler()) return;

	panel P("Create Random Tree");
	int n = gw.get_gen_nodes();
	
	P.int_item ("nodes", n);
	P.int_item ("max. degree", gw.rt_max_deg);
	P.int_item ("max. width", gw.rt_max_width);

	P.button ("ok", 1);
	P.button ("cancel", 0);

	if (gw.openPanel(P) == 0 || n < 0) return;

	bool old_flush=gw.set_flush(false); 
	gw.gen_pre(n,gw.get_gen_edges());

	random_tree (gw.get_graph(), n, gw.rt_max_deg, gw.rt_max_width);

	gw.update_graph();
	GraphWinInterface A (gw);
	A.flush(false);
	gw.tree_layout.call(gw.get_graph(),A);
	gw.fill_window();

	gw.set_flush(old_flush);
	gw.redraw();
}

void GraphWinAGD::callRandomPlanarBicGraph(GraphWin &gw1)
{
	GraphWinAGD& gw = (GraphWinAGD&)gw1;
	if (!gw.check_init_graph_handler()) return;

	panel P("Create Biconnected Planar Graph");
	int n = gw.get_gen_nodes(),
		m = gw.get_gen_edges();
	
	P.int_item ("nodes", n);
	P.int_item ("edges", m);
	P.bool_item ("multi-edges", gw.rbg_multi);
	P.bool_item ("planar map", gw.rbg_map);

	P.button ("ok", 1);
	P.button ("cancel", 0);

	if (gw.openPanel(P) == 0 || n < 0 || m < 0) return;

	gw.gen_pre(n,m);
	planar_biconnected_graph (gw.get_graph(), n, m, gw.rbg_multi, gw.rbg_map);
	gw.gen_post();
}

void GraphWinAGD::callRandomStPlanarGraph(GraphWin &gw1)
{
	GraphWinAGD& gw = (GraphWinAGD&)gw1;
	if (!gw.check_init_graph_handler()) return;

	panel P("Create st-Planar Graph");
	int n = gw.get_gen_nodes(),
		m = gw.get_gen_edges();
	
	P.int_item ("nodes", n);
	P.int_item ("edges", m);
	P.bool_item ("multi-edges", gw.rst_multi);
	P.double_item ("st probability", gw.rst_stprob);

	P.button ("ok", 1);
	P.button ("cancel", 0);

	if (gw.openPanel(P) == 0 || n < 0 || m < 0) return;

	bool old_flush=gw.set_flush(false); 
	gw.gen_pre(n,m);
	st_planar_graph (gw.get_graph(), n, m, gw.rst_multi, gw.rst_stprob);

	gw.update_graph();
	GraphWinInterface A (gw);
	A.flush(false);
	gw.st_planar.call(gw.get_graph(),A);
	gw.fill_window();

	gw.set_flush(old_flush);
	gw.redraw();
}

void GraphWinAGD::gen_random_graph(GraphWin& gw1, void (*f)(graph&,int,int), string s)
{
	GraphWinAGD& gw = (GraphWinAGD&)gw1;
	if (!gw.check_init_graph_handler()) return;

	panel P(s);
	int n = gw.get_gen_nodes(),
		m = gw.get_gen_edges();

	P.int_item ("nodes", n);
	P.int_item ("edges", m);
	P.button ("ok", 1);
	P.button ("cancel", 0);

	if (gw.openPanel(P) == 0 || n < 0 || m < 0) return;

	gw.gen_pre(n,m);
	f (gw.get_graph(), n, m);
	gw.gen_post();
}

void GraphWinAGD::gen_random_graph1(GraphWin& gw1, void (*f)(graph&,int), string s)
{
	GraphWinAGD& gw = (GraphWinAGD&)gw1;
	if (!gw.check_init_graph_handler()) return;

	panel P(s);
	int n = gw.get_gen_nodes();

	P.int_item ("nodes", n);
	P.button ("ok", 1);
	P.button ("cancel", 0);

	if (gw.openPanel(P) == 0 || n < 0) return;

	gw.gen_pre(n,gw.get_gen_edges());

	f (gw.get_graph(), n);

	gw.gen_post();
}


void GraphWinAGD::embed_random(const graph& G, node_array<double>& x,
	node_array<double>& y) 
{
	random_source ran;
	node v;
	forall_nodes(v,G)  ran >> x[v] >> y[v];
}

//-------------------------------------------------------------------
//
//                         agd_demo.c
//
// This program demonstrates the layout algorithms of the
// AGD-Library unsing LEDA's GraphWin as graph editor.
//
//    authors: David Alberts, Carsten Gutwenger, Gunnar Klau
//    date:    decembre 1999
//-------------------------------------------------------------------



//int agd_demo_main(int argc, char *argv[]);

#if defined(WINMAIN)

int main() {
  int    argc;
  char** argv;
  get_command_line(argc,argv);

#else

int main(int argc, char *argv[]) {

#endif

	return agd_demo_main(argc,argv);
}

