/*******************************************************************************
+
+  AGD 1.2
+
+
+  CGraphWin.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                         ClusterGraphWin.h
//
//    author: Dirk Luetke-Huettmann
//    date:   july 1999
//-------------------------------------------------------------------


#ifndef AGD_CLUSTER_GRAPHWIN_H
#define AGD_CLUSTER_GRAPHWIN_H 1

#include <LEDA/graphics/graphwin.h>
#include <LEDA/system/stream.h>
#include <LEDA/system/stream.h>
#include <AGD/ClusterGraph.h>


/*{\Manpage {CGraphWin} {} {GraphWin for Cluster Graphs} {CGW} }*/

/*{\Mbaseclasses

\begin{hierarchy}{7}{2.5}
\hdclass{0}{1.5}{6}{GraphWin}
\arhook{6}{2}{7}
\hdclass{1}{0}{6.5}{CGraphWin}
\end{hierarchy}
}*/

class __exportAGD CGraphWin;

// static void dummy_handler(leda::string) {};

enum cgw_cluster_style { cgw_rectangle, cgw_circle };


/////////////////////////////////////////////////////////////////////////
//
// class: CLUSTER_INFO

class __exportAGD cluster_info {
  friend class CGraphWin;

public:
  cluster_info();
    
  int get_index() { return index;};

  void set_x(double xcoord) { x = xcoord; };
  void set_y(double ycoord) { y = ycoord; };
  void set_width(double width) { w = width; };
  void set_height(double height) { h = height; };

  void set_label(const leda::string &l) { label = l;} ;
  const leda::string &get_label() { return label;};

  void set_cluster_style(cgw_cluster_style style) { cluster_style = style; };

  leda::gen_polygon get_region() { return cluster_region; } ; 
  void set_region(leda::gen_polygon cl_region) {cluster_region = cl_region; };

private:
  int index;
  Cluster* cl;

  leda::string label;

  leda::color text_fg_color;
  leda::color cluster_color;
  leda::color cluster_fill_color;
  leda::color cluster_label_color;
  cgw_cluster_style cluster_style;
  leda::line_style linestyle;

  leda::gen_polygon cluster_region;

  double x,y,w,h;
  int borderwidth;
};

/////////////////////////////////////////////////////////////////////////////
//
// class: CGRAPHWIN


class __exportAGD CGraphWin : public leda::GraphWin {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is an extended |GraphWin| that
gives the user the ability to edit a clustergraph.}*/

public:

  /*{\Mcreation CGW 5.5}*/
  
  CGraphWin(const char* name);
  /*{\Mcreate creates an instance |\Mvar| of type |\Mname| and sets the window
    label to |name|.}*/

  /*{\Moperations}*/

  ClusterGraph& get_clustergraph();
  /*{\Mop returns the reference to the underlieing clustergraph |CG|.}*/

  void clear_clustertree();
  /*{\Mop removes all clusters from the cluster hierarchy. All nodes of 
    |\Mvar| are made members of the root-cluster.}*/ 

  void print_gml(ostream &);

  void restore_dim() {win_init(_xmin, _xmax, _ymin);};

  Cluster* find_bounding_cluster(leda::point p);

  void insert_to_cli_cl_map(cluster_info* cli, Cluster* cl_node) { cli_cl_map[cl_node] = cli;};

  cluster_info* get_cluster_info(Cluster* cl) {return cli_cl_map[cl];};

  void fill_window();
  void get_bounding_box(double&, double&, double&, double&);

  void zoom_graph();

  void setup_node(leda::node v);

  void redraw();

  void redraw_clusters();

private:
  ClusterGraph* cg_p;
  Cluster* rc_p;

  int get_next_index();
  leda::list<int> missing_ids;

  //leda::string graphname;
  
  double _xmin, _xmax, _ymin;
  int cl_index_counter;

  leda::map<Cluster*, cluster_info*> cli_cl_map;

  // initializes the callback functions for menu and mouse actions
  void set_handler_functions();
  void init_window();

  void init_clustertree(Cluster* cl_parent);

  void gml_print_cluster_tree(Cluster* cl_parent, leda::string indent, ostream &out);

  void load_gml_handler();
  void save_gml_handler();

  static void read_file(leda::string filename);
  static void save_file(leda::string filename);

  void read_cluster(leda::file_istream& fp, Cluster* cl_parent);  
  void read_cluster_gml(const leda::string &filename);

  //const leda::string &get_graphname() {return graphname;} ;
  void set_gml_filename(const leda::string &name);

  void rec_redraw_cluster(Cluster* cl_parent);
  Cluster* rec_fb_cluster(Cluster* cl_parent, leda::point p);

  void get_extreme_coords(double& xmin, double& ymin, 
			  double& xmax, double& ymax, 
			  double& r1, double& r2);
  
  void get_extreme_coords(double& xmin, double& ymin, 
			  double& xmax, double& ymax);
  
  void adjust_zoom_rect(double& x0, double& y0, double& x1, double& y1);
  
  static void create_cluster(leda::GraphWin& gw);

  static void delete_cluster(leda::GraphWin& gw);

  static void load_gml(leda::GraphWin& gw);

  static void save_gml(leda::GraphWin& gw);
};

#endif
