/*******************************************************************************
+
+  AGD 1.2
+
+
+  IRect.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                           IRect.h
//
//    author: Carsten Gutwenger
//    date:   june 1997
//-------------------------------------------------------------------


#if defined(_MSC_VER)
#pragma once
#endif

#ifndef _AGD_IRECT_H
#define _AGD_IRECT_H

#include <AGD/IPoint.h>


/*{\Manpage {IRect} {} {Integer Rectangles} {ir} }*/

class __exportAGD IRect {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a rectangle
$(left,right,top,bottom)$ in a grid coordinate system.
That is $left\leq right$ and $bottom\leq top$.}*/

public:

	/*{\Mcreation}*/

	IRect () {
		l = r = t = b = 0;
	}
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname| initialized to
	the rectangle $(0,0,0,0)$.}*/

	IRect (int xmin, int xmax, int ymin, int ymax) {
		l = xmin; r = xmax;
		t = ymax; b = ymin;
	}
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname| initialized to
	the rectangle $(xmin,xmax,ymin,ymax)$.}*/

	IRect (const IPoint& ll, const IPoint& ur) {
		l = ll.xcoord();
		r = ur.xcoord();
		b = ll.ycoord();
		t = ur.ycoord();
	}
	/*{\Mcreate  creates an instance |\Mvar| of type |\Mname| initialized to
	the rectangle with lower left corner |ll| and upper right corner |ur|.}*/


	/*{\Moperations}*/

	IPoint lower_left () const {
		return IPoint (l, b);
	}
	/*{\Mop returns the lower left corner of |\Mvar|.}*/

	IPoint upper_right () const {
		return IPoint (r, t);
	}
	/*{\Mop returns the upper right corner of |\Mvar|.}*/

	/*{\Moptions nextwarning=no }*/
	const int& left () const {
		return l;
	}
	int& left () {
		return l;
	}
	/*{\Mop returns the x-coordinate of the lower left corner
	of |\Mvar|.}*/

	/*{\Moptions nextwarning=no }*/
	const int& right () const {
		return r;
	}
	int& right () {
		return r;
	}
	/*{\Mop returns the x-coordinate of the upper right corner
	of |\Mvar|.}*/

	/*{\Moptions nextwarning=no }*/
	const int& top () const {
		return t;
	}
	int& top () {
		return t;
	}
	/*{\Mop returns the y-coordinate of the upper right corner
	of |\Mvar|.}*/

	/*{\Moptions nextwarning=no }*/
	const int& bottom () const {
		return b;
	}
	int& bottom () {
		return b;
	}
	/*{\Mop returns the y-coordinate of the lower left corner
	of |\Mvar|.}*/

	/*{\Moptions nextwarning=no }*/
	int width () const {
		return r-l;
	}
	/*{\Mop returns the width $(= |right|-|left|)$ of |\Mvar|.}*/

	int height () const {
		return t-b;
	}
	/*{\Mop returns the height $(= |top|-|bottom|)$ of |\Mvar|.}*/

	int area() const {
		return width() * height();
	}
	/*{\Mop returns the area $(= |width|() * |height|())$ of |\Mvar|.}*/

private:
	int l, r, t, b;
};


#endif
