/*******************************************************************************
+
+  AGD 1.2
+
+
+  LEDAMakeAcyclic.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                         LEDAMakeAcyclic.h
//
//
// author: Carsten Gutwenger
// date:   march 1998
//-------------------------------------------------------------------


#ifndef _AGD_LEDA_MK_ACYCLIC_H
#define _AGD_LEDA_MK_ACYCLIC_H


#include <AGD/SubgraphModule.h>


/*{\Manpage {LEDAMakeAcyclic} {} {LEDA's Make Acyclic} {S} }*/

/*{\Mbaseclasses
\begin{hierarchy}{10}{4}
\hdclass{0}{3}{5.5}{AGDModule}
\arhook{5.5}{3.5}{6.5}
\hdclass{1}{1.5}{6}{SubgraphModule}
\arhook{7}{2}{8}
\hdclass{2.5}{0}{6}{LEDAMakeAcyclic}
\end{hierarchy}
}*/


class __exportAGD LEDAMakeAcyclic : public SubgraphModule {

/*{\Mdefinition
The module |\Mname| is a front end for LEDA's Make\_Acyclic()
function. It computes an acyclic subgraph of an
input graph G.

\horizline
\commoninfo{Make Acyclic}{LEDA's Make Acyclic}{}{S. N\"aher}{}{}

\begin{preposttab}
precondition & $\{ \emph{directed} \} $ \\
postcondition(|PRE|) &  $\{ \emph{maximal acyclic, directed} \} $
\end{preposttab}

\horizline
}*/

  public:

    /*{\Mcreation S 5}*/
    
    LEDAMakeAcyclic();
    /*{\Mcreate creates an instance |\Mvar| of type |\Mname|.}*/
    
    
    ~LEDAMakeAcyclic() { }
    
    /*{\Moperations 1.2 3.5}*/

    leda::string name () const {
	return "Make Acyclic";
    }

    leda::string long_name () const {
	return "LEDA's Make Acyclic";
    }

    leda::string impl_author () const {
	return "S. N\204her";
    }


    AGDModule *clone() const;

	/*{\Moptions warnings=no}*/

	/*{\Mtext\stdstart{1.5cm}
	\headerline{Standard Interface (Inherited Methods)}
	The detailed description of these methods can be found in the
	manual entries of the base class (|SubgraphModule|).
	}*/

	/* inherited
	bool check (const graph& G, AgdKey &p) const;
	*/
	/*{\Mop}*/

	/* inherited
	bool call (const graph& G, list<edge>& L);
	*/
	/*{\Mop}*/

	/* inherited
	bool call (graph& G);
	*/
	/*{\Mop}*/

	/* inherited
	bool call (GraphCopy& G, list<edge>& L);
	*/
	/*{\Mop}*/

	/* inherited
	int num_del_edges() const;
	*/
	/*{\Mop}*/

	/*{\Mtext \stdend}*/
	
  protected:
    void do_call (const leda::graph& G, leda::list<leda::edge>& feedback);
};


#endif
