/*******************************************************************************
+
+  AGD 1.2
+
+
+  OrthoRepGridConcept.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



#if defined(_MSC_VER)
#pragma once
#endif

//#line 1473 "Lweb/OrthoRep.lw"
#ifndef _AGD_ORGC_CONCEPT_H
#define _AGD_ORGC_CONCEPT_H

#include <AGD/impl/GCGridConceptBase.h>

class OrthogonalRepresentation;

/*{\Manpage {OrthoRepGridConcept} {} {Grid Concept for Orthogonal Representations} {C} }*/

/*{\Mbaseclasses

\begin{hierarchy}{12.5}{2.5}
  \hdclass{0}{1.5}{9.5}{PlaneGraphCopyGridConcept} \arhook{9.5}{2}{10.5}
  \hdclass{4}{0}{7}{OrthoRepGridConcept}
\end{hierarchy} }*/


class __exportAGD OrthoRepGridConcept : public GCGridConceptBase {

/*{\Mdefinition
The class |\Mname| represents an implementation concept for grid layout
algorithms which work on an orthogonal representation and access the grid
layout via an object of type |GridLayout|. The graph is stored as an orthogonal representation |H|, which can be modified by the algorithm. 
}*/

  OrthogonalRepresentation *_porp;
//  list<Label> *_Lp;
  
  bool new_graph, new_layout;
  
  // forbid copying
  OrthoRepGridConcept (const OrthoRepGridConcept &) { }
  OrthoRepGridConcept &operator=(const OrthoRepGridConcept &) {
    return *this;
  }
  
public:
  
  OrthoRepGridConcept(const leda::graph &G, 
                                      LayoutInterface &A);      
  OrthoRepGridConcept(const leda::graph &G, 
                                      leda::face f, LayoutInterface &A);
  OrthoRepGridConcept(OrthogonalRepresentation &H, 
                                      LayoutInterface &A);

  OrthoRepGridConcept(const leda::graph &G, GridLayout &gl);

  OrthoRepGridConcept(const leda::graph &G,
                                      leda::face f, GridLayout &gl);

//  OrthoRepGridConcept(const leda::graph &G,
//                                    leda::face f, GridLayout &gl,
//                                    leda::list<Label>& Lambda);

  OrthoRepGridConcept(OrthogonalRepresentation &H, 
                                      GridLayout &gl);


  ~OrthoRepGridConcept();


/*{\Moperations}*/
/*{\Mtext |\Mname| implements the constructors and operations which are
  required for template argument |GRID_CONCEPT| of |GridLayoutModuleImpl|.
  Furthermore, the following operations are provided:
  The class inherits all the operations from |PlaneGraphCopyGridConcept|. Additionally, it provides the following operation:}*/

	static void set_postcondition(leda::list<AgdKey> &post);

  void get_node_grid_size (GridCoordinateMapper &gcm,
              const LayoutInterface &A);
  void get_node_grid_size(const GridLayout &gl) {
    GCGridConceptBase::get_node_grid_size(gl);
  }
  
  void update (GridCoordinateMapper &gcm, LayoutInterface &A);
  void update (GridLayout &gl);
  
  
  //list<Label> &get_labels() { 
  //  return *_Lp;
  //}

  OrthogonalRepresentation &get_representation() {
    return *(OrthogonalRepresentation *)(_porp);
  }
  /*{\Mop returns the orthogonal representation |H|.}*/
  
  
  /*{\Moptions warnings=no}*/
  
  /*
    GridLayout &get_layout() const;
  */
  /*{\Mop returns the grid layout of |G|.}*/
  
  /*
    void set_zero_size();
        */
        /*{\Mop sets the grid width and height of all nodes to $0$.}*/

        /*
        void set_bbox (const IRect &bbox);
        */
        /*{\Mop sets the bounding box of the grid layout to |bbox|.}*/
};

#endif

