/*******************************************************************************
+
+  AGD 1.2
+
+
+  PlaneClusterGraphCopy.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                         PlaneClusterGraphCopy.h
//
//    author: Dirk Luetke-Huettmann
//    date:   june 1999
//-------------------------------------------------------------------


#ifndef _AGD_PLANARIZED_CLUSTER_GRAPH_H
#define _AGD_PLANARIZED_CLUSTER_GRAPH_H

#include <AGD/ClusterGraph.h>
#include <AGD/PlaneGraphCopy.h>
#include <LEDA/core/map.h>

/*{\Manpage {PlaneClusterGraphCopy} {} {Plane Cluster Graph Copies} {PCG} }*/

/*{\Mbaseclasses

\begin{hierarchy}{8}{5.5}
\hdclass{0}{4.5}{5}{graph}
\arhook{5}{5}{6}
\hdclass{1.5}{3}{5}{GraphCopy}
\arhook{6.5}{3.5}{7.5}
\hdclass{2.5}{1.5}{5.5}{PlaneGraphCopy}
\arhook{8}{2}{9}
\hdclass{1.5}{0}{8.0}{PlaneClusterGraphCopy}
\end{hierarchy}
}*/


class __exportAGD PlaneClusterGraphCopy : public PlaneGraphCopy {

/*{\Mdefinition
An instanz |\Mvar| of |\Mtype| represents a copy of a planar clustergraph.}*/

public:

	/*{\Mcreation PCG 6}*/
        PlaneClusterGraphCopy ();

	PlaneClusterGraphCopy(const PlaneClusterGraphCopy& pcg);
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| initialized
	to a copy of |pcg|.}*/

	PlaneClusterGraphCopy(const ClusterGraph& CG);
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| initialized
	to a copy of |CG|.}*/

	PlaneClusterGraphCopy(const ClusterGraph& CG, leda::list<leda::edge>& E);
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname| initialized
	to a copy of $CG$ without the edges in $E$. \precond Each $e\in E$
	is an edge in $CG$.}*/

	~PlaneClusterGraphCopy();


	/*{\Moperations 2 3.5}*/

	PlaneClusterGraphCopy& operator= (const PlaneClusterGraphCopy& PCG);

	Cluster* new_cluster(void);
	/*{\Mop creates a new cluster as a child of the root-cluster and returns it.}*/

	Cluster* new_cluster(Cluster* cl);
	/*{\Mop creates a new cluster as a child of cluster |cl| and returns it.}*/

	leda::node new_node(Cluster* cl);
	/*{\Mop creates a new node and makes it a member of cluster |cl|.}*/

	Cluster* get_rootcluster(void) const {
	  return rc_p; 
	};
	/*{\Mop returns the root-cluster.}*/

	Cluster* cluster_of(leda::node v) const;
	/*{\Mop returns the cluster node |v| belongs too.}*/ 
	
	Cluster* original(Cluster* cl);
	/*{\Mop returns the corresponding cluster for |cl| in the original clustergraph.}*/ 

	const ClusterGraph& original() const {
	  return *cg_p;
	}
	/*{\Mop returns the underlying clustergraph.}*/
	
	leda::edge original(leda::edge e) const {
	  return PlaneGraphCopy::original(e);
	} 

	leda::list<Cluster*> all_clusters() const {
	  leda::list<Cluster *> all;
          _all_clusters(rc_p,all);
          return all;
        }
	/*{\Mop returns a list of all cluster.}*/

	leda::list<Cluster*> all_subcluster(Cluster* cl) const {
	  leda::list<Cluster *> subcl;
	  _all_clusters(cl, subcl);
	  return subcl;
	}
	/*{\Mop returns a list of all subcluster of $T(c)$.}*/

	bool is_member(Cluster* cl, leda::node v) const; 
	/*{\Mop returns |true| if node $v$ is a member of a subcluster of |cl|,
	  otherwise |false|.}*/

	Cluster* copy(Cluster* cl);
	/*{\Mop returns the copy of cluster |cl|. Precondition is that |cl| is a cluster in the original graph.}*/
	
	leda::node copy(leda::node v) {
	  return PlaneGraphCopy::copy(v);
	}

	ClusterGraph* get_clustergraph() const {
	  return cg_p;
	}

private:
	ClusterGraph* cg_p;
	Cluster* rc_p;

	leda::map<Cluster*, Cluster*> cl_copy;
	leda::map<Cluster*, Cluster*> cl_orig;
	
	void init_clustertree(Cluster* cl) ;

	void post_new_node_handler(leda::node);
	void pre_del_node_handler(leda::node);	
	
	void _all_clusters(Cluster *c, leda::list<Cluster *> &all) const;
};

#endif
