/*******************************************************************************
+
+  AGD 1.2
+
+
+  PlaneGraphCopyGridConcept.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                      PlaneGraphCopyGridConcept.h
//
//    author: Carsten Gutwenger
//    date:   octobre 1998
//-------------------------------------------------------------------


#ifndef _AGD_PGC_CONCEPT_H
#define _AGD_PGC_CONCEPT_H


#include <AGD/impl/GCGridConceptBase.h>

class PlaneGraphCopy;


/*{\Manpage {PlaneGraphCopyGridConcept} {} {Grid Concept for Plane Graph Copies} {C} }*/

class __exportAGD PlaneGraphCopyGridConcept : public GCGridConceptBase {

/*{\Mdefinition
The class |\Mname| represents an implementation concept for grid layout
algorithms which work on a planar map and access the grid
layout via an object of type |GridLayout|. The graph is stored as a plane graph
copy |PG|, which can be modified by the algorithm. The following transformations
are performed by the update functions:
\begin{itemize}
\item If the polyline of an edge $e$ is empty and the polyline of its
reversal is not empty, the reversed polyline of the reversal edge
is chosen as polyline for $e$. Thus, it is sufficient to assign a
polyline to an edge or its reversal, no matter which one corresponds
to an edge in the original graph.
\item The polylines of splitted edges are concatenated.
\item Unnecessary bend points are removed.
\end{itemize}
A computed bounding box can be passed to the grid concept using
set\_bbox(). Otherwise, the bounding box is computed on demand.
}*/

	PlaneGraphCopy *_pgcp;
	leda::face       _ext_face;

	bool new_graph, new_layout;

	// forbid copying
	PlaneGraphCopyGridConcept (const PlaneGraphCopyGridConcept &) { }
	PlaneGraphCopyGridConcept &operator=(const PlaneGraphCopyGridConcept &) {
		return *this;
	}

public:

	PlaneGraphCopyGridConcept (const leda::graph &G, LayoutInterface &A);	
	PlaneGraphCopyGridConcept (const leda::graph &G, leda::face f, LayoutInterface &A);
	PlaneGraphCopyGridConcept (PlaneGraphCopy &PG, leda::face f, LayoutInterface &A);

	PlaneGraphCopyGridConcept (const leda::graph &G, GridLayout &gl);
	PlaneGraphCopyGridConcept (const leda::graph &G, leda::face f, GridLayout &gl);
	PlaneGraphCopyGridConcept (PlaneGraphCopy &PG, leda::face f, GridLayout &gl);


	~PlaneGraphCopyGridConcept();


	/*{\Moperations}*/
	/*{\Mtext |\Mname| implements the constructors and operations which are
	required for template argument |GRID_CONCEPT| of |GridLayoutModuleImpl|.
	Furthermore, the following operations are provided:}*/

	void get_node_grid_size (GridCoordinateMapper &gcm,
		const LayoutInterface &A);
	void get_node_grid_size(const GridLayout &gl) {
		GCGridConceptBase::get_node_grid_size(gl);
	}

	void update (GridCoordinateMapper &gcm, LayoutInterface &A);
	void update (GridLayout &gl);


	PlaneGraphCopy &get_graph() {
		return *(PlaneGraphCopy *)(_gcp);
	}
	/*{\Mop returns the plane graph copy |PG|.}*/

	leda::face get_exterior_face() {
		return _ext_face;
	}
	/*{\Mop returns the exterior face of |PG| or |nil|.}*/

	/*{\Moptions warnings=no}*/

	/*
	GridLayout &get_layout() const;
	*/
	/*{\Mop returns the grid layout of |G|.}*/

	/*
	void set_zero_size();
	*/
	/*{\Mop sets the grid width and height of all nodes to $0$.}*/

	/*
	void set_bbox (const IRect &bbox);
	*/
	/*{\Mop sets the bounding box of the grid layout to |bbox|.}*/

	static void set_postcondition(leda::list<AgdKey> &post);
};


#endif
