/*******************************************************************************
+
+  AGD 1.2
+
+
+  PreCond.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                           PreCond.h
//
//
//    author: Carsten Gutwenger
//    date:   octobre 1998
//-------------------------------------------------------------------


#if defined(_MSC_VER)
#pragma once
#endif

#ifndef _AGD_PRE_COND_H
#define _AGD_PRE_COND_H

#include <AGD/CondBase.h>


/*{\Manpage {PreCond} {} {Preconditions} {P} }*/

/*{\Mbaseclasses

\begin{hierarchy}{6.5}{2.5}
\hdclass{0}{1.5}{5}{CondBase}
\arhook{5}{2}{6}
\hdclass{1.5}{0}{5}{PreCond}
\end{hierarchy}
}*/

class __exportAGD PreCond : public CondBase {

/*{\Mdefinition
An instance |\Mvar| of type |\Mname| is a precondition of a module $M$.
It is represented by a collection of key sets $S_1,\ldots,S_n$.
The key set $S_i$ corresponds to the $i$th input parameter of $M$,
i.e., $n$ is the number of input parameters.
}*/

public:

	/*{\Mcreation}*/

	PreCond(int n = 1) : CondBase(1,n) { }
	/*{\Mcreate creates an instance |\Mvar| of type |\Mname|
	initialized to a precondition for $n$ input parameters.
	Each key set is initialized to the empty set.}*/

	PreCond(const PreCond& p) : CondBase(p) { }

	~PreCond() { }

	/*{\Moperations}*/
	/*{\Mtext |\Mname| inherits the operations of its base class
	|CondBase|.}*/

	PreCond &operator=(const PreCond &c) {
		CondBase::operator=(c);
		return *this;
	}
};



#endif
