/*******************************************************************************
+
+  AGD 1.2
+
+
+  CompactionPair.h
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



//#line 50 "Lweb/CompactionPair.lw"
#ifndef _AGD_COMPACTIONPAIR_H
#define _AGD_COMPACTIONPAIR_H

#include <AGD/OrthogonalRepresentation.h>
#include <AGD/GridLayout.h>

class CompactionPair {
  
  public:

  enum node_dir { unknown_node_dir, horizontal, vertical };
  
  CompactionPair(const OrthogonalRepresentation &H);
  CompactionPair(GridLayout &L);
  ~CompactionPair() {};
  
  void compute_x_coords() { compute_coords(vertical); };
  void compute_y_coords() { compute_coords(horizontal); };
  void compute_coords(node_dir d);

  void compute_coords_properly(node_dir d);
  void compute_x_coords_properly() { compute_coords_properly(vertical); }
  void compute_y_coords_properly() { compute_coords_properly(horizontal); }
  
  double x_coord(leda::node v) const { return _coord[_ver[v]];  }
  double y_coord(leda::node v) const { return _coord[_hor[v]];  }
  
  void print(ostream &o = cout) const;

  protected:

  enum arc_type { unknown_arc, basic_arc, visibility_arc };
  enum node_type { unknown_node_type, graph_segment, label_segment };
  
  void create_nodes(node_dir d);
  void create_basic_arcs(direction d);

  void create_visibility_arcs(const GridLayout &L, node_dir d);

  void add_vertices(leda::node n, leda::node v, node_dir d,
                    leda::node_array<bool> &marked);
  
  void set_coords(const PlaneGraphCopy &DC, const leda::edge_array<int> &flow);
  void process_node(leda::node n, double coord, const PlaneGraphCopy &DC, 
                    const leda::edge_array<double> &length,
                    leda::node_array<bool> &processed);
  
  private:

  const leda::graph *PP; // pointer to pertinent graph 
  leda::edge_array<direction> _dir; // directions in the pertinent graph
  leda::graph DDxy; 
  leda::node_array<leda::node> _hor, _ver;
  leda::node_map<double> _coord; // the coordinate
  leda::node_map<leda::node> left_, right_, bottom_, top_;
  leda::node_map<leda::list<leda::node> > _members;
  
  leda::edge_map<double> _weight;
  leda::edge_map<arc_type> _a_type;
  leda::node_map<node_type> _n_type;
  leda::node_map<node_dir> _n_dir;
  
  void _init();
  void _removeDoubleArcs();

};

#endif

