/*******************************************************************************
+
+  LEDA 4.4.1  
+
+
+  misc.h
+
+
+  Copyright (c) 1995-2003
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                            misc.h
//
//    author: Carsten Gutwenger
//    date:   february 1997
//-------------------------------------------------------------------


#ifndef _AGD_IMPL_MISC_H
#define _AGD_IMPL_MISC_H

#include <AGD/AGD.h>
#include <LEDA/core/array.h>
#include <LEDA/core/list.h>


template<class E>
void init (leda::array<E> &A, E x)
{
   int i;
   for (i = A.low(); i <= A.high(); i++)
      A[i] = x;
}

template<class E>
leda::list_item get (const leda::list<E> &L, leda::list_item it_start, int pos)
{
   if (L.empty()) return nil;
	leda::list_item it = it_start;
	int k = pos % L.length();

	for (int i = 1; i <= k; i++, it = L.cyclic_succ (it)) ;
	return it;
}

template<class E>
void reverse_list (leda::list<E>& L)
{
	leda::list_item it;
	leda::list<leda::list_item> items;

	forall_items (it, L)
		items.append(it);

	forall (it, items)
		L.move_to_front (it);
}

template<class E>
void permute(leda::array<E>& A)
{
	leda::random_source& rs = AGD::get_random_source();
	int low = A.low(), high = A.high();

	for (int i = low; i <= high; ++i) {
		int j = rs(low, high);
		leda::swap(A[i], A[j]);
	}
}

template<class E>
void permute(leda::list<E>& L)
{
	leda::random_source& rs = AGD::get_random_source();
	int sz = L.size();
	leda::list_item* A = LEDA_NEW_VECTOR(leda::list_item, sz);

	for (int i = 0; i < sz; ++i) {
		int j = rs(0, sz-1);
		leda::swap(A[i], A[j]);
	}
	L.permute(A);

	LEDA_DEL_VECTOR(A);
}

#endif
