//-------------------------------------------------------------------
//
//                      agd_face_map.h
//
//    author: Carsten Gutwenger
//    date:   july 1999
//-------------------------------------------------------------------


#ifndef AGD_FA_MAP_H
#define AGD_FA_MAP_H


#include <LEDA/graph/graph.h>


template <class E>
class agd_face_map : public graph_map {

	E def;

	void clear_entry(GenPtr& x) const {
		E *p = (E *) x;
		delete p;
	}

	void copy_entry(GenPtr& p) const {
		const E &x = *(E *)p;
		p = (GenPtr) new E(x);
	}

	void init_entry(GenPtr& p) const {
		p = (GenPtr) new E(def);
	}

public:

	agd_face_map() : graph_map(nil,0,2) {
		leda_init_default(def);
		init_def_entry();
	}

	agd_face_map(const leda_graph& G) : graph_map(&G,G.max_face_index()+1,2) {
		leda_init_default(def);
		init_def_entry();
		init_table();
	}


	agd_face_map(const leda_graph& G, E x) : graph_map(&G,G.max_face_index()+1,2), def(x) {
		init_def_entry();
		init_table();
	}

	agd_face_map(const agd_face_map<E>& M) : graph_map(M), def(M.def) {
		init_def_entry();
	}

	agd_face_map<E> &operator=(const agd_face_map<E>& M) {
		clear_def_entry();
		graph_map::operator=(M);
		def = M.def;
		init_def_entry();
		return *this;
	}

	~agd_face_map() {
		clear_table();
		clear_def_entry();
	}

	const E& operator[](leda_face v) const {
		return *(const E *)graph_map::map_read(v);
	}

	E& operator[](leda_face v) {
		return *(E *)graph_map::map_access(v);
	}

	void init() {
		graph_map::init(nil,0,2);
	}

	void init (const leda_graph &G) {
		graph_map::init(&G,G.max_face_index()+1,2);
		init_table();
	}

	void init(const leda_graph &G, E x) {
		graph_map::init(&G,G.max_face_index()+1,2);
		def = x;
		init_table();
	}

};


#endif
