//-------------------------------------------------------------------
//
//                      agd_node_map.h
//
//    author: Carsten Gutwenger
//    date:   july 1999
//-------------------------------------------------------------------


#ifndef AGD_ND_MAP_H
#define AGD_ND_MAP_H


#include <LEDA/graph/graph.h>


template <class E>
class agd_node_map : public graph_map {

	E def;

	void clear_entry(GenPtr& x) const {
		E *p = (E *) x;
		delete p;
	}

	void copy_entry(GenPtr& p) const {
		const E &x = *(E *)p;
		p = (GenPtr) new E(x);
	}

	void init_entry(GenPtr& p) const {
		p = (GenPtr) new E(def);
	}

public:

	agd_node_map() : graph_map(nil,0,0) {
		leda_init_default(def);
		init_def_entry();
	}

	agd_node_map(const leda_graph& G) : graph_map(&G,G.max_node_index()+1,0) {
		leda_init_default(def);
		init_def_entry();
		init_table();
	}


	agd_node_map(const leda_graph& G, E x) : graph_map(&G,G.max_node_index()+1,0), def(x) {
		init_def_entry();
		init_table();
	}

	agd_node_map(const agd_node_map<E>& M) : graph_map(M), def(M.def) {
		init_def_entry();
	}

	agd_node_map<E> &operator=(const agd_node_map<E>& M) {
		clear_def_entry();
		graph_map::operator=(M);
		def = M.def;
		init_def_entry();
		return *this;
	}

	~agd_node_map() {
		clear_table();
		clear_def_entry();
	}

	const E& operator[](leda_node v) const {
		return *(const E *)graph_map::map_read(v);
	}

	E& operator[](leda_node v) {
		return *(E *)graph_map::map_access(v);
	}

	void init() {
		graph_map::init(nil,0,0);
	}

	void init (const leda_graph &G) {
		graph_map::init(&G,G.max_node_index()+1,0);
		init_table();
	}

	void init(const leda_graph &G, E x) {
		graph_map::init(&G,G.max_node_index()+1,0);
		def = x;
		init_table();
	}

};


#endif
