/************************************************************************

         Filename   :   PQdirected.h
         Version    :   1.0.1
         Author     :   Christoph Buchheim
         Modified   :   Sebastian Leipert
         at         :   May 1998
         Language   :   C++

************************************************************************/

#ifndef PQDIRECTED_H
#define PQDIRECTED_H


#ifdef NOAGD 

#include "PQTree.h"
#include "pqNode.h"
#include "internalNode.h"
#include "leaf.h"
#include "indicator.h"
#include "leafKey.h"
#include "pqNodeKey.h"
#include "basicList.h"
#include "pqStack.h"

#else

#include "AGD/impl/PQTree/PQTree.h"
#include "AGD/impl/PQTree/pqNode.h"
#include "AGD/impl/PQTree/internalNode.h"
#include "AGD/impl/PQTree/leaf.h"
#include "AGD/impl/planarity/indicator.h"
#include "AGD/impl/PQTree/leafKey.h"
#include "AGD/impl/PQTree/pqNodeKey.h"
#include "AGD/impl/planarity/basicList.h"
#include "AGD/impl/PQTree/pqStack.h"

#endif


template<class T,class X,class Y>
class PQdirected : public PQTree<T,X,Y> {

typedef PQTree<T,X,Y> baseclass;
public:
  
PQdirected();
// ~PQdirected();
void addition(X,int,leafKey<T,X,Y>**,basicList<T>*,pqStack<X>*,pqStack<X>*);


protected:
  
pqStack<pqNode<T,X,Y>*> pertinentInd;

  
void saveFront(pqNode<T,X,Y>*,basicList<T>*,pqStack<X>*,pqStack<X>*,int=0);

  
pqNode<T,X,Y> *clientSibLeft(pqNode<T,X,Y>*) const;
pqNode<T,X,Y> *clientSibRight(pqNode<T,X,Y>*) const;
pqNode<T,X,Y> *clientLeftEndmost(pqNode<T,X,Y>*) const;
pqNode<T,X,Y> *clientRightEndmost(pqNode<T,X,Y>*) const;
pqNode<T,X,Y> *clientNextSib(pqNode<T,X,Y>*,pqNode<T,X,Y>*) const;

char *clientPrintType(pqNode<T,X,Y>*);
int clientPrintNodeCategorie(pqNode<T,X,Y>*);

void removeChildFromSiblings(pqNode<T,X,Y>*);


};

template<class T,class X,class Y>
PQdirected<T,X,Y>::PQdirected() : PQTree<T,X,Y>() {};

/*
template<class T,class X,class Y>
PQdirected<T,X,Y>::~PQdirected() {};
*/

template<class T,class X,class Y>
void PQdirected<T,X,Y>::addition(X label,int numberOfElements,
                                 leafKey<T,X,Y>** ArrayOfElements,
                                 basicList<T> *keyList,
                                 pqStack<X> *dirLeft,pqStack<X> *dirRight) {
  pqNode<T,X,Y> *_newSubtree=NULL;
  if(numberOfElements>1) {
    _newSubtree=new internalNode<T,X,Y>(baseclass::_identificationNumber++,P_NODE,EMPTY);
    addNewLeavesToTree((internalNode<T,X,Y>*)_newSubtree,
                       ArrayOfElements,numberOfElements);
  }
  else if(numberOfElements==1) _newSubtree=
    new leaf<T,X,Y>(baseclass::_identificationNumber++,EMPTY,ArrayOfElements[0]);
  if(baseclass::_pertinentRoot->status()==PARTIAL) {
    pqNode<T,X,Y> *_child,*_left,*_right;
    _child=fullChildrenStack(baseclass::_pertinentRoot)->pop();
    fullChildrenStack(baseclass::_pertinentRoot)->push(_child);
    _left=PQTree<T,X,Y>::clientSibLeft(_child);
    _right=_child->getNextSib(_left);
    while(_left&&(_left->status()==FULL||_left->type()==IND)) {
      _right=_child; _child=_left;
      _left=_child->getNextSib(_right);
    }
    while(_child&&(_child->status()==FULL||_child->type()==IND)) {
      _right=_child->getNextSib(_left);
      saveFront(_child,keyList,dirLeft,dirRight,
        PQTree<T,X,Y>::clientSibLeft(_child)==_left);
      removeChildFromSiblings(_child);
      _child=_right;
    }
    if(_newSubtree) {
      addNodeToNewParent(baseclass::_pertinentRoot,_newSubtree,_left,_right);
      _left=_newSubtree;
    }
    pqNodeKey<T,X,Y> *_info=new pqNodeKey<T,X,Y>(label);
    indicator<T,X,Y> *_newInd=
      new indicator<T,X,Y>(baseclass::_identificationNumber++,_info);
    addNodeToNewParent(baseclass::_pertinentRoot,_newInd,_left,_right);
    _newInd->changeSiblings(_left,NULL);
    _newInd->changeSiblings(_right,NULL);
    _newInd->putSibling(_left,LEFT);
    _newInd->putSibling(_right,RIGHT);
  }
  else {
    saveFront(baseclass::_pertinentRoot,keyList,dirLeft,dirRight);
    if(_newSubtree) {
      exchangeNodes(baseclass::_pertinentRoot,_newSubtree);
      if(baseclass::_root==baseclass::_pertinentRoot) baseclass::_root=_newSubtree;
    }
    else if(baseclass::_root==baseclass::_pertinentRoot)
      baseclass::_root=new internalNode<T,X,Y>(baseclass::_identificationNumber++,P_NODE,EMPTY);
    else removeChildFromSiblings(baseclass::_pertinentRoot);
  }
  while(!pertinentInd.stackEmpty()) delete pertinentInd.pop();
}

template<class T,class X,class Y>
void PQdirected<T,X,Y>::saveFront(pqNode<T,X,Y> *_saveRoot,
                                  basicList<T> *keyFront,
                                  pqStack<X> *dirLeft,pqStack<X> *dirRight,
                                  int direction) {
  pqNode<T,X,Y> *_firstSon,*_nextSon,*_oldSib,*_newSib;
  X label;
  switch(_saveRoot->type()) {
    case P_NODE: if(_saveRoot->referenceChild()) {
                   _firstSon=_saveRoot->referenceChild();
                   saveFront(_firstSon,keyFront,dirLeft,dirRight);
                   _nextSon=PQTree<T,X,Y>::clientSibRight(_firstSon);
                   while(_nextSon!=_firstSon) {
                     saveFront(_nextSon,keyFront,dirLeft,dirRight);
                     _nextSon=PQTree<T,X,Y>::clientSibRight(_nextSon);
                   }
                 }
                 destroyNode(_saveRoot);
                 break;
    case Q_NODE: _oldSib=NULL;
                 _nextSon=PQTree<T,X,Y>::clientLeftEndmost(_saveRoot);
                 while(_nextSon) {
                   saveFront(_nextSon,keyFront,dirLeft,dirRight,
                     PQTree<T,X,Y>::clientSibLeft(_nextSon)==_oldSib);
                   _newSib=_nextSon->getNextSib(_oldSib);
                   _oldSib=_nextSon;
                   _nextSon=_newSib;
                 }
                 destroyNode(_saveRoot);
                 break;
    case LEAF:   keyFront->in(*(_saveRoot->getKey()->userStructKey()));
                 destroyNode(_saveRoot);
                 break;
    case IND:    label=(*(((indicator<T,X,Y>*)_saveRoot)
                       ->getNodeInfo()->userStructInfo()));
                 if(direction) dirRight->push(label);
                 else dirLeft->push(label);
                 pertinentInd.push(_saveRoot);
  }
}

template<class T,class X,class Y>
pqNode<T,X,Y> *PQdirected<T,X,Y>::clientSibLeft(pqNode<T,X,Y> *nodePtr)
const {
  pqNode<T,X,Y> *holdSib,*oldSib=nodePtr;
  nodePtr=PQTree<T,X,Y>::clientSibLeft(oldSib);
  while(nodePtr&&nodePtr->type()==IND) {
    holdSib=oldSib; oldSib=nodePtr;
    nodePtr=oldSib->getNextSib(holdSib);
  }
  return nodePtr;
}

template<class T,class X,class Y>
pqNode<T,X,Y> *PQdirected<T,X,Y>::clientSibRight(pqNode<T,X,Y> *nodePtr)
const {
  pqNode<T,X,Y> *holdSib,*oldSib=nodePtr;
  nodePtr=PQTree<T,X,Y>::clientSibRight(oldSib);
  while(nodePtr&&nodePtr->type()==IND) {
    holdSib=oldSib; oldSib=nodePtr;
    nodePtr=oldSib->getNextSib(holdSib);
  }
  return nodePtr;
}

template<class T,class X,class Y>
pqNode<T,X,Y> *PQdirected<T,X,Y>::clientLeftEndmost(pqNode<T,X,Y> *nodePtr)
const {
  pqNode<T,X,Y> *left=PQTree<T,X,Y>::clientLeftEndmost(nodePtr);
  if(!left||left->type()!=IND) return left;
  else return clientNextSib(left,NULL);
}

template<class T,class X,class Y>
pqNode<T,X,Y> *PQdirected<T,X,Y>::clientRightEndmost(pqNode<T,X,Y> *nodePtr)
const {
  pqNode<T,X,Y> *right=PQTree<T,X,Y>::clientRightEndmost(nodePtr);
  if(!right||right->type()!=IND) return right;
  else return clientNextSib(right,NULL);
}

template<class T,class X,class Y>
pqNode<T,X,Y> *PQdirected<T,X,Y>::clientNextSib(pqNode<T,X,Y> *nodePtr,
                                       pqNode<T,X,Y> *other) const {
  pqNode<T,X,Y> *left=clientSibLeft(nodePtr);
  pqNode<T,X,Y> *right=clientSibRight(nodePtr);
  if(left!=other) return left;
  else if(right!=other) return right;
  else return NULL;
}

template<class T,class X,class Y>
char *PQdirected<T,X,Y>::clientPrintType(pqNode<T,X,Y> *nodePtr) {
  if(nodePtr->type()==IND) return (char*)"DIRECTION INDICATOR";
  else return PQTree<T,X,Y>::clientPrintType(nodePtr);
}

template<class T,class X,class Y>
int PQdirected<T,X,Y>::clientPrintNodeCategorie(pqNode<T,X,Y> *nodePtr) {
  if(nodePtr->type()==IND) return IND_CAT;
  else return PQTree<T,X,Y>::clientPrintNodeCategorie(nodePtr);
}

template<class T,class X,class Y>
void PQdirected<T,X,Y>::removeChildFromSiblings(pqNode<T,X,Y> *nodePtr) {
  if(nodePtr->type()!=IND) {
    pqNode<T,X,Y> *left=clientSibLeft(nodePtr);
    pqNode<T,X,Y> *right=clientSibRight(nodePtr);
    pqNode<T,X,Y> *parent=nodePtr->parent();
    if(left&&!right) left->parent(parent);
    if(right&&!left) right->parent(parent);
  }
  PQTree<T,X,Y>::removeChildFromSiblings(nodePtr);
}



#endif
