// l_matcher.h: interface for the l_matcher class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _AGD_IMPL_L_MATCHER_H
#define _AGD_IMPL_L_MATCHER_H

#include <LEDA/core/h_array.h>
#include <LEDA/core/map2.h>

#include "bctree.h"
#include "label.h"

enum MATCH_STRATEGY {SIZE, CONNECT_CONDITION};
enum PLANAR_STATUS {N_CHECKED, N_PLANAR, PLANAR_, N_PLANAR_CHECKED};

int MAKE_CONNECTED (graph&, bctree& , list<edge> & );

class l_matcher
{
public:
	int nPlanarityTests ();

	inline bool use_blabels () {
		return use_blabels_;
	}

	inline void use_blabels (bool u) {
		use_blabels_ = u;
	}

	inline void set_strategy (MATCH_STRATEGY s) {
		strategy_ = s;
	}

	inline MATCH_STRATEGY strategy() {
		return strategy_;
	}

	l_matcher (graph& G, MATCH_STRATEGY MS);
	l_matcher();
	virtual ~l_matcher();
	list<edge> augment (bool use_blabels, MATCH_STRATEGY strat);
private:
	void update_dfs_numbering ();
	int nTests_;
	block connect_inside_label (label& l, block anc);
	int number_of_nodes (bc_node b);
	void explore_subtree (bc_node start, bc_node tabu, list<cutpoint>& container);
	void explore_planarity (cutpoint c1, cutpoint c2);
	void change2clabel (label& l, cutpoint new_head);
	bool try_extend (label& l);
	void check_DFS_numbers (bc_node bcn, bc_node parent);
	bc_node find_last_before (block p, bc_node bcn);

	inline label is_label (bc_node bcn) {
		if ( !is_label_.defined (bcn))
			return nil;
		return labels_.inf (is_label_[bcn]);
	}

	bool are_adj (node v, node w);
	bool PLANAR (cutpoint c1, cutpoint c2, bool explore_planarity = true, bool add_infos = true);
	list_item insert_label (label l);

	inline void delete_pendant (block p) {
		pendants_.del_item (pendants_.search (p));
		belongs_to_.undefine (p);
	}

	inline int number_of_labels () {
		return labels_.size ();
	}

	label make_blabel (label c1, label c2, block b, cutpoint head);
	label add_pendant (label l, block p);

	inline void remove_pendant (block p, label l = nil) {
		belongs_to_.undefine (p);
		list_item li = pendants_.search (p);
		pendants_.del_item (li);
		l->remove_pendant (p);
	}

	block connect_labels (label first, label sec);
	void delete_label (label& l, bool remove_pendants = true);
	block connect_inside_pendant (block p, block anc, cutpoint c);
	label new_label (cutpoint cp, block p, STOP_CAUSE why_stop);
	label is_head (cutpoint cp);
	label reduce_chain (block p, cutpoint start, label l_old = nil);

	inline edge connect_pendants (block p1, block p2) {
		edge e = G_->new_edge (BC_.adj_to_cutpoint_node (p1), BC_.adj_to_cutpoint_node (p2));
		result_.push (e);
	return e;
	}

	block join_pendants (label& l);
	bool concond_strategy (label &first, label &sec);
	bool size_strategy (label &first, label &sec);
	bool connect_condition (label a, label b);

	inline bool is_ready() {
		return labels_.empty ();
	}

	bool MASSIVE_;
	STOP_CAUSE follow_path (cutpoint cp, cutpoint &last);
	list<edge> terminate ();
	void decrease_size (label l, list<block> pendants);

	bool use_blabels_;
	bctree BC_;
	MATCH_STRATEGY strategy_;
	graph *G_;
	list <label> labels_;
	h_array <bc_node, list_item> is_label_;
	list<block> pendants_;
	list<edge> result_;
	h_array<block, label> belongs_to_;
	map2 <cutpoint, cutpoint, PLANAR_STATUS> planar_;
};

#endif 
