/*******************************************************************************
+
+  AGD 1.2
+
+
+  GraphWinInterface.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


//-------------------------------------------------------------------
//
//                     GraphWinInterface.c
//
//    author: Carsten Gutwenger, revised by David Alberts
//    date:   june 1997
//-------------------------------------------------------------------


#include <AGD/GraphWinInterface.h>
#include <AGD/GraphWinAnimInterface.h>

using namespace leda;


//-------------------------------------------------------------------
//                         GraphWinInterface
//-------------------------------------------------------------------

GraphWinInterface::GraphWinInterface (GraphWin& GW) :
	LayoutInterface(GW.get_graph()), _flush(true)
{
	gw_p = &GW;
	old_flush = gw_p->set_flush(false);
}

GraphWinInterface::~GraphWinInterface()
{
	gw_p->set_flush(old_flush);
}

void GraphWinInterface::init(GraphWin &GW)
{
	set_graph(GW.get_graph());
	gw_p = &GW;
	old_flush = gw_p->set_flush(false);
}

void GraphWinInterface::update()
{
	if (_flush)
		gw_p->redraw();
}


//-------------------------------------------------------------------
//                        GraphWinAnimInterface
//-------------------------------------------------------------------

GraphWinAnimInterface::GraphWinAnimInterface (GraphWin& GW) :
//	LayoutBuffer(GW.get_graph())
	AGDLayout(GW.get_graph())
{
	gw_p = &GW;
	old_flush = GW.set_flush(false);
	init_buffer();
}


void GraphWinAnimInterface::init(GraphWin &GW)
{
	gw_p->set_flush(old_flush);
	set_graph(GW.get_graph());
	gw_p = &GW;
	old_flush = GW.set_flush(false);
	init_buffer();
}


void GraphWinAnimInterface::init_buffer()
{
	const graph &G = gw_p->get_graph();
	point p;

	node v;
	forall_nodes(v,G) {
		p = gw_p->get_position(v);
		set_position(v,DPoint(p.xcoord(),p.ycoord()));
		set_width(v,gw_p->get_radius(v)*2);
		set_height(v,gw_p->get_radius2(v)*2);
	}

	edge e;
	forall_edges(e,G) {
		leda::list<leda::point> points = gw_p->get_bends(e);
		DPolyline l;
		forall(p,points) l.append(DPoint(p.xcoord(),p.ycoord()));
		set_bend_points(e,l);
		p = gw_p->get_source_anchor(e);
		set_source_anchor(e,DPoint(p.xcoord(),p.ycoord()));
		p = gw_p->get_target_anchor(e);
		set_target_anchor(e,DPoint(p.xcoord(),p.ycoord()));
	}
}

/*
void GraphWinAnimInterface::init_buffer()
{
	const graph &G = gw_p->get_graph();
	point p;

	node v;
	forall_nodes(v,G) {
		p = gw_p->get_position(v);
		set_position(v,DPoint(p.xcoord(),p.ycoord()));
	}

	edge e;
	forall_edges(e,G) {
		leda::list<leda::point> points = gw_p->get_bends(e);
		DPolyline l;
		forall(p,points) l.append(DPoint(p.xcoord(),p.ycoord()));
		set_bend_points(e,l);
		p = gw_p->get_source_anchor(e);
		set_source_anchor(e,DPoint(p.xcoord(),p.ycoord()));
		p = gw_p->get_target_anchor(e);
		set_target_anchor(e,DPoint(p.xcoord(),p.ycoord()));
	}
}
*/

void GraphWinAnimInterface::update()
{
	const graph&G = get_graph();
	node_array<point> pos(G);
	node_array<double> rad1(G);
	node_array<double> rad2(G);
	edge_array<list<point> > bends(G);
	edge_array<point> src_anch(G);
	edge_array<point> tgt_anch(G);

	//bool old_flush = gw_p->set_flush(false);

	node v;
	forall_nodes(v,G) {
		pos[v] = point(get_x(v),get_y(v));
		rad1[v] = get_width(v)/2;
		rad2[v] = get_height(v)/2;
	}

	edge e;
	forall_edges(e,G) {
		const list<DPoint> &dpl = get_bend_points(e);
		DPoint dp;
		forall(dp,dpl) {
			bends[e].append(point(dp.xcoord(),dp.ycoord()));
		}
		DPoint anc = get_source_anchor(e);
		src_anch[e] = point(anc.xcoord(),anc.ycoord());
		anc = get_target_anchor(e);
		tgt_anch[e] = point(anc.xcoord(),anc.ycoord());
	}

	gw_p->set_flush(true);
	gw_p->set_layout(pos,rad1,rad2,bends,src_anch,tgt_anch);
	gw_p->set_flush(false);
}

/*void GraphWinAnimInterface::update()
{
	const graph&G = get_graph();
	node_array<point> pos(G);
	edge_array<list<point> > bends(G);

	//bool old_flush = gw_p->set_flush(false);

	node v;
	forall_nodes(v,G)
		pos[v] = point(get_x(v),get_y(v));

	edge e;
	forall_edges(e,G) {
		const list<DPoint> &dpl = get_bend_points(e);
		DPoint dp;
		forall(dp,dpl) {
			bends[e].append(point(dp.xcoord(),dp.ycoord()));
		}
		DPoint anc = get_source_anchor(e);
		gw_p->set_source_anchor(e,point(anc.xcoord(),anc.ycoord()));
		anc = get_target_anchor(e);
		gw_p->set_target_anchor(e,point(anc.xcoord(),anc.ycoord()));
	}

	gw_p->set_flush(true);
	gw_p->set_layout(pos,bends,false);
	gw_p->set_flush(false);
}
*/
