/*******************************************************************************
+
+  AGD 1.2
+
+
+  random_hierarchy.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



#include <AGD/AGD.h>
#include <LEDA/core/random.h>
#include <AGD/generators.h>
#include <AGD/LeveledGraph.h>

using namespace leda;

class bEdge {
public:
  int head,tail,id,pos;
  bEdge *next;
  bEdge(int t,int h,int c):head(h),tail(t),id(c),pos(-1),next(0) {};
};

int cmpId(bEdge *const& a,bEdge *const& b) {
  return (a->id<b->id?-1:(a->id>b->id?1:0));
}

int cmpTail(bEdge *const& a,bEdge *const& b) {
  return (a->tail<b->tail?-1:(a->tail>b->tail?1:cmpId(a,b)));
}

int cmpHead(bEdge *const& a,bEdge *const& b) {
  return (a->head<b->head?-1:(a->head>b->head?1:cmpId(a,b)));
}

void random_hierarchy(LeveledGraph& G,
		      int numberOfNodes,int numberOfEdges,
		      bool planar,bool singleSource,bool longEdges) {

  G.clear();

  random_source& rs = AGD::get_random_source();
  node *nnr=new node[3*numberOfNodes];
  int *vrt=new int[3*numberOfNodes];
  int *fst=new int[numberOfNodes+1];
  list<bEdge*> startEdges;
  bEdge *actEdge,*nextEdge,*toDelete;
  node v;
  int act,next,n1,n2,idc=0;
  double x1,x2,r;
  bool connected;

  /** Place nodes **/

  for(int i=0;i<numberOfNodes;i++) G.new_node(0);
  int numberOfLayers=0,totNumber=0,realCount=0;
  fst[0]=0;
  forall_nodes(v,G) {
    if(longEdges&&numberOfLayers) vrt[totNumber++]=1;
    nnr[totNumber]=v;
    vrt[totNumber++]=0;
    G.level(v,numberOfLayers);
    realCount++;
    rs>>r;
    if((totNumber==1 && singleSource) || realCount==numberOfNodes
       ||r*r*numberOfNodes<1) {
      if(longEdges&&numberOfLayers) vrt[totNumber++]=1;
      fst[++numberOfLayers]=totNumber;
    }
  }

  /** Determine allowed neighbours **/

  int *leftN=new int[totNumber];
  int *rightN=new int[totNumber];
  for(int l=1;l<numberOfLayers;l++) {
    if(planar) {
      n1=fst[l-1]; n2=fst[l];
      leftN[n2]=n1;
      while(n1<fst[l]&&n2<fst[l+1]) {
	rs>>r;
        if(n1!=fst[l]-1&&
	   (n2==fst[l+1]-1||
	    r<(double)(fst[l]-fst[l-1])/(double)(fst[l+1]-fst[l-1])))
	  n1++;
        else {
          rightN[n2]=n1;
          if(++n2<fst[l+1]) leftN[n2]=n1;
        }
      }
    }
    else for(n2=fst[l];n2<fst[l+1];n2++) {
      leftN[n2]=fst[l-1];
      rightN[n2]=fst[l]-1;
    }
  }

  /** Insert edges **/

  list<bEdge*> *edgeIn=new list<bEdge*>[totNumber];
  list<bEdge*> *edgeOut=new list<bEdge*>[totNumber];
  if(numberOfLayers) {
    x1=numberOfEdges; x2=0;
    for(n2=fst[1];n2<totNumber;n2++) if(!vrt[n2])
      x2+=rightN[n2]-leftN[n2]+1;
    for(n2=fst[1];n2<totNumber;n2++) if(!vrt[n2]) {
      connected=!singleSource;
      for(n1=leftN[n2];n1<=rightN[n2]||!connected;n1++) {
	rs>>r;
        if(r<x1/x2||n1>rightN[n2]) {
          next=(n1<=rightN[n2]?n1:rs(leftN[n2],rightN[n2]));
	  act=n2;
          nextEdge=new bEdge(next,act,idc++);
	  while(vrt[next]) {
	    act=next;
	    next=rs(leftN[act],rightN[act]);
	    edgeOut[act]+=nextEdge;
	    nextEdge=new bEdge(next,act,idc++);
	    edgeIn[act]+=nextEdge;
	  }
          startEdges+=nextEdge;
	  connected=1;
	  x1-=1;
        }
        if(n1<=rightN[n2]) x2-=1;
      }
    }
  }
  delete[] leftN;
  delete[] rightN;

  if(planar) for(act=0;act<totNumber;act++) {
    edgeIn[act].sort(&cmpTail); edgeOut[act].sort(&cmpHead);
  }
  for(act=0;act<totNumber;act++) forall(nextEdge,edgeIn[act])
    nextEdge->next=edgeOut[act].pop_front();
  delete[] edgeOut;

  forall(actEdge,startEdges) {
    nextEdge=actEdge;
    while(vrt[nextEdge->head]) nextEdge=nextEdge->next;
    G.new_edge(nnr[actEdge->tail],nnr[nextEdge->head]);
  }

  /** Clean up **/

  forall(nextEdge,startEdges) {
    toDelete=nextEdge;
    while(vrt[nextEdge->head]) {
      nextEdge=nextEdge->next;
      delete toDelete;
      toDelete=nextEdge;
    }
    delete toDelete;
  }

  delete[] edgeIn;
  delete[] fst;
  delete[] vrt;
  delete[] nnr;

}
