/*******************************************************************************
+
+  AGD 1.2
+
+
+  StPlanarLayout.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



#include <AGD/StPlanarLayout.h>
#include <AGD/LayoutInterface.h>
#include <LEDA/graph/face_array.h>


using namespace leda;

/*ostream &operator<<(ostream& os, edge e)
{
	if (e)
		os << "(" << index(source(e)) << "," << index(target(e)) << ")";
	else
		os << "nil";
	return os;
}*/


StPlanarGridLayout::StPlanarGridLayout () :
	_compaction(symmetryPreserving)
{
	add_precondition (key::st_planar);

	add_post_rule (key::reduced_acyclic, 1, key::straight_line);
	add_post_rule (key::no_crossings);
}


AGDModule *StPlanarGridLayout::clone() const
{
	StPlanarGridLayout *M = new StPlanarGridLayout;

	M->compaction(compaction());

	return M;
}

void StPlanarGridLayout::do_call(GraphCopyGridConcept &C)
{
	GraphCopy &GC = C.get_graph();

	// find s and t
	node v;
	forall_nodes(v,GC) {
		if (GC.indeg (v) == 0) s = v;
		if (GC.outdeg(v) == 0) t = v;
	}

	// find edge (s,t); if (s,t) not in GC, add it
	forall_out_edges(st,s)
		if (target(st) == t) break;

	stMissing = (st == nil);
	if (stMissing)
		st = GC.new_edge(s,t);


	// embed GC
	list<edge> R;
	GC.make_map(R);
	GC.make_planar_map();

	draw(GC,C.get_layout(),R);
}

void StPlanarGridLayout::call_plane (GraphCopy &GC, edge e_st, GridLayout &gl)
{
	st = e_st;
	s = source(st);
	t = target(st);
	stMissing = false;

	edge e;
	list<edge> R;
	forall_edges(e,GC)
		if (GC.is_dummy(e)) R.append(e);

	draw(GC,gl,R);
}

void StPlanarGridLayout::draw(GraphCopy &GC,
	GridLayout &gl, const list<edge> &R)
{
	// set up first-/lastout, first-/lastin
	firstout.init(GC);
	lastout .init(GC);
	firstin .init(GC);
	lastin  .init(GC);

	node v;
	forall_nodes(v,GC) {
		if (v == t || v == s) continue;

		edge e = GC.first_adj_edge(v);
		while (!GC.is_dummy(e)) e = GC.cyclic_adj_pred(e);
		while (GC.is_dummy(e)) e = GC.cyclic_adj_pred(e);

		firstout[v] = e;
		firstin [v] = GC.reversal(GC.cyclic_adj_succ(e));

		while (!GC.is_dummy(e)) e = GC.cyclic_adj_pred(e);
		
		lastin [v] = GC.reversal(e);
		lastout[v] = GC.cyclic_adj_succ(e);
	}

	// we draw edge (s,t) to the left!
	if (stMissing) {
		firstout[s] = GC.cyclic_adj_pred(st);
		firstin [t] = GC.reversal(GC.cyclic_adj_succ(GC.reversal(st)));
	} else
		firstout[s] = firstin[t] = st;

	lastout [s] = GC.cyclic_adj_succ(st);
	lastin  [t] = GC.reversal(GC.cyclic_adj_pred(GC.reversal(st)));
	firstin[s] = lastin[s] = firstout[t] = lastout[t] = nil;

/*	node v;
	forall_nodes(v,GC) {
		if (v == t || v == s) continue;

		edge e = GC.first_adj_edge(v);
		while (!GC.is_dummy(e)) e = GC.cyclic_adj_succ(e);
		while (GC.is_dummy(e)) e = GC.cyclic_adj_succ(e);

		firstout[v] = e;
		firstin [v] = GC.reversal(GC.cyclic_adj_pred(e));

		while (!GC.is_dummy(e)) e = GC.cyclic_adj_succ(e);
		
		lastin [v] = GC.reversal(e);
		lastout[v] = GC.cyclic_adj_pred(e);
	}

	// we draw edge (s,t) to the left!
	if (stMissing) {
		firstout[s] = GC.cyclic_adj_succ(st);
		firstin [t] = GC.reversal(GC.cyclic_adj_pred(GC.reversal(st)));
	} else
		firstout[s] = firstin[t] = st;

	lastout [s] = GC.cyclic_adj_pred(st);
	lastin  [t] = GC.reversal(GC.cyclic_adj_succ(GC.reversal(st)));
	firstin[s] = lastin[s] = firstout[t] = lastout[t] = nil;*/


	// find transitive edges
	list<edge> transitives;
	findTransitiveEdges(GC,transitives);


	// remove added edges
	GC.del_edges(R);
	if (stMissing) GC.del_edge(st);


	// split all transitive edges
	edge e;
	forall(e,transitives) {
		node w = target(e);
		edge e1 = GC.split(e);
		if (firstin[w] == e) firstin[w] = e1;
		if (lastin [w] == e) lastin [w] = e1;
		node u = source(e1);
		firstin[u] = lastin[u] = e;
		firstout[u] = lastout[u] = e1;
	}


	// compute preleminary layout
	X.init(GC); Y.init(GC);
	xNode = new node[GC.number_of_nodes()];
	yNode = new node[GC.number_of_nodes()];

	count = 0; labelX(GC, s);
	count = 0; labelY(GC, s);


	// compaction
	//GridLayout &gl = C.get_layout();
	if (compaction() != none)
		compact(GC, gl);
	else
		forall_nodes(v,GC) {
			gl.x(v) = X[v];
			gl.y(v) = Y[v];
		}


	//clean-up
	firstin .init(); lastin .init();
	firstout.init(); lastout.init();
	X.init(); Y.init();
	delete[] xNode; delete[] yNode;
}

void StPlanarGridLayout::findTransitiveEdges(const GraphCopy &GC, list<edge> &T)
{
	face_array<node> orig(GC), dest(GC);

	// set orig[f] to the (unique) source of f
	// and dest[f] to the (unique) sink of f
	face f;
	forall_faces(f,GC) {
		edge e;
		forall_face_edges(e,f) {
			edge ePred = GC.face_cycle_pred(e);
			node v = source(e);

			if (!GC.is_dummy(e) && GC.is_dummy(ePred))
				orig[f] = v;
			if (GC.is_dummy(e) && !GC.is_dummy(ePred))
				dest[f] = v;
		}
	}

	// e = (u,v) transitive <=> ex. face f: e in f and u = orig[f] and v = dest[f]
	edge e;
	forall_edges(e,GC) {
		if (GC.is_dummy(e)) continue;
		face f = GC.face_of(e), fR = GC.face_of(GC.reversal(e));
		node u = source(e), v = target(e);
		if ((u == orig[f] && v == dest[f]) || (u == orig[fR] && v == dest[fR]))
			T.append(e);
	}
}

void StPlanarGridLayout::labelX(const graph &G, node v)
{
	xNode[X[v] = count++] = v;

	if (v != t) {
		edge e1 = firstout[v];
		edge e = e1;
		do {
			node w = target(e);
			if (e == lastin[w])
				labelX(G,w);
			e = G.cyclic_adj_pred(e);
		} while (e != e1);
	}
}

void StPlanarGridLayout::labelY(const graph &G, node v)
{
	yNode[Y[v] = count++] = v;

	if (v != t) {
		edge e1 = lastout[v];
		edge e = e1;
		do {
			node w = target(e);
			if (e == firstin[w])
				labelY(G,w);
			e = G.cyclic_adj_succ(e);
		} while (e != e1);
	}
}

void StPlanarGridLayout::compact(const graph &G, GridLayout &gl)
{
	bool mL_leq_mR = true, mL_gt_mR = true;

	if (compaction() == minimumArea) {
		int mL = 0, mR = 0;
		node u;
		forall_nodes(u,G) {
			if (u == t) continue;
			edge e = firstout[u];
			if (e == lastin [target(e)]) mL++;
			e = lastout[u];
			if (e == firstin[target(e)]) mR++;
		}

		mL_leq_mR = (mL <= mR);
		mL_gt_mR  = (mL >  mR);
	}

	node_map<int> &x = gl.x();
	x[s] = 0;
	int i;
	node u = s;
	for (i = 1; i < G.number_of_nodes(); i++) {
		node v = xNode[i];
		if (Y[u] > Y[v] || (firstout[u] == lastout[u] &&
									firstin[v] == lastin[v] && mL_leq_mR))
			x[v] = x[u] + 1;
		else
			x[v] = x[u];
		u = v;
	}

	node_map<int> &y = gl.y();
	y[s] = 0;
	for (u = s, i = 1; i < G.number_of_nodes(); i++) {
		node v = yNode[i];
		if (X[u] > X[v] || (firstout[u] == lastout[u] &&
									firstin[v] == lastin[v] && mL_gt_mR))
			y[v] = y[u] + 1;
		else
			y[v] = y[u];
		u = v;
	}
}



StPlanarLayout::StPlanarLayout () :
	_compaction(StPlanarGridLayout::symmetryPreserving),
	_sep(20)
{
	add_precondition (key::st_planar);

	add_post_rule (key::reduced_acyclic, 1, key::straight_line);
	add_post_rule (key::no_crossings);
}


AGDModule *StPlanarLayout::clone() const
{
	StPlanarLayout *M = new StPlanarLayout;

	M->compaction(compaction());
	M->separation(separation());
	return M;
}

void StPlanarLayout::do_call(ConstLedaGraphConcept &C)
{
	const graph &G = C.get_graph();
	LayoutInterface &A = C.get_layout();

	GridLayout gl(G);
	StPlanarGridLayout st_planar;
	st_planar.compaction(compaction());
	st_planar.call_grid(G,gl);

	draw(G,gl,A);
}

void StPlanarLayout::draw(const graph &G, GridLayout &gl,
	LayoutInterface &A)
{
	double w = 0;
	node v;
	forall_nodes(v,G) {
		if (A.get_width (v) > w) w = A.get_width (v);
		if (A.get_height(v) > w) w = A.get_height(v);
	}

	double d = w + _sep/sqrt(2.0);

	A.make_straight_line();

	forall_nodes(v, G)
		A.set_position(v, DPoint((gl.x(v)-gl.y(v))*d, (gl.x(v)+gl.y(v))*d));

	edge e;
	forall_edges(e,G) {
		const IPolyline &ipl = gl.bends(e);
		DPolyline dpl;
		IPoint ip;
		forall(ip, ipl)
			dpl.append(DPoint((ip.xcoord()-ip.ycoord())*d,
				(ip.xcoord()+ip.ycoord())*d));
		A.set_bend_points(e, dpl);
	}
}


void StPlanarLayout::call_plane (GraphCopy &GC, edge e_st,
	LayoutInterface &A)
{
	GridLayout gl(GC);
	StPlanarGridLayout st_planar;
	st_planar.compaction(compaction());
	st_planar.call_plane(GC,e_st,gl);

	const graph &G = GC.original();
	GridLayout glG(G);

	node v;
	forall_nodes(v,G) {
		glG.x(v) = gl.x(GC.copy(v));
		glG.y(v) = gl.y(GC.copy(v));
	}

	edge e;
	forall_edges(e,G) {
		IPolyline &ipl = glG.bends(e);
		const list<edge> &L = GC.chain(e);
		edge eC;
		forall(eC,L)
			ipl.append(IPoint(gl.x(target(eC)),gl.y(target(eC))));
		ipl.pop_back();
	}

	draw(GC.original(),glG,A);
	A.update();
}
