//-------------------------------------------------------------------
//
//                     MixedModelBase.h
//
//    author: Carsten Gutwenger
//    date:   september 1998
//-------------------------------------------------------------------


#include <AGD/PlaneGraphCopy.h>
#include <AGD/GridLayout.h>
#include <AGD/AugmentationModule.h>

#include "MMOrder.h"
#include "IOPoints.h"

#include <LEDA/internal/std/fstream.h>


using namespace leda;

class MixedModelBase {

protected:

	PlaneGraphCopy &PG;
	face exterior_face;

	GridLayout &gl;

	MMOrder   mmo;
	IOPoints iops;

	array<int> dyl, dyr;
	array<list_item> left_op, right_op;
	node_array<list_item> next_left, next_right;
	node_array<int> dxla, dxra;


public:

	MixedModelBase(PlaneGraphCopy &_PG, face ext_face, GridLayout &_gl) :
		PG(_PG), exterior_face(ext_face), gl(_gl), iops(_PG) { }

	virtual ~MixedModelBase() { }

	void compute_order(AugmentationModule &augmenter, bool prefer_nodes,
		double base_ratio);

	void assign_iop_coords();

	void place_nodes();
	void compute_x_coords();
	void compute_y_coords();

	void set_bends();
	void postprocessing1();
	void postprocessing2();


	// functions for debugging output

	void printMMOrder (ostream &os);
	void printInOutPoints (ostream &os);
	void print(ostream &os, const InOutPoint &iop);
	void printNodeCoords (ostream &os);


protected:

	bool exists(edge e) {
		return !PG.is_dummy(e) || !PG.is_dummy(PG.reversal(e));
	}

	bool has_left  (int k) const;
	bool has_right (int k) const;

	face remove_deg1_nodes();

private:

	void _first_point(int &x, int &y, edge e);
	bool _is_redundant(int x1, int y1, int x2, int y2, int x3, int y3);

};
