/*******************************************************************************
+
+  AGD 1.2
+
+
+  ShapeModule.c
+
+
+  Copyright (c) 1998-2003
+  by  Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/



//#line 508 "Lweb/shape.lw"
#include <AGD/ShapeModule.h>
#include <LEDA/internal/std/math.h>

using namespace leda;

const string epsilon = "";
  
ShapeModule::ShapeModule() : TopologyModule() {
  round_ = true;
}

const int ShapeModule::in_graph = 1;
const int ShapeModule::in_layout = 2;
const int ShapeModule::out_ortho_rep = 1;

bool ShapeModule::check(const graph& G, AgdKey &p) const {
  return check_graph(in_graph, G, p);
}

OrthogonalRepresentation &ShapeModule::call(const graph &G,
                                            const LayoutInterface& L) {

  face f_0 = nil; 
  PlaneGraphCopy &PG = TopologyModule::call(G, L, f_0);
  
  OrthogonalRepresentation *H = new OrthogonalRepresentation(PG, f_0);
  
  
//#line 579 "Lweb/shape.lw"
node_array<DPoint> position(*H);
edge_array<DPolyline> bends(*H);

edge e; forall_edges (e, *H) {
  
  edge o = H->original(e);
  if (o) {
    position[source(e)] = L.get_position(source(o));
    position[target(e)] = L.get_position(target(o));
    bends[e] = L.get_bend_points(o);
  } else {
    edge r = H->original(H->reversal(e));
    if (!r) error_handler(1, "ShapeModule::call. No reversal.");
    DPolyline B = L.get_bend_points(r);
    B.reverse();
    bends[e] = B;
  }
}


//#line 533 "Lweb/shape.lw"
                                                   ;

  face f; forall_faces(f, *H) {
    edge e; forall_face_edges(e, f) {
      
      edge e_succ = H->face_cycle_succ(e);
      node u = source(e), v = target(e), w = target(e_succ);

      // i muss letzter bend point auf e, l muss erster 
      // bend point auf h sein.

      DPoint i = bends[e].empty() ? position[u] : bends[e].back();

      DPoint j = position[v];

      DPoint l = bends[e_succ].empty() ? position[w] : bends[e_succ].head();

      if (round_) H->set_angle(e, (int)rint((2 * LEDA_PI - angle(j, i, l))/LEDA_PI * 2) * 90);
      else H->set_angle(e, (int)rint((2 * LEDA_PI - angle(j, i, l))/LEDA_PI * 180));

      DPolyline b = bends[e];
      if (!b.empty()) {	// at least one bend
        b.push(position[source(e)]); b.append(position[target(e)]);
	for (list_item it = b.get_item(1); it != b.last();
	     it = b.succ(it)) {
	  DPoint q = b[it], p = b[b.pred(it)], r = b[b.succ(it)];

	  int alpha = round_ ? ((int)rint((2 * LEDA_PI - angle(q, p, r))/
					  LEDA_PI * 2) * 90) 
	    : ((int)rint((2 * LEDA_PI - angle(q, p, r))/LEDA_PI * 180));

	  if (alpha == 90) H->add_convex_bend(e);
	  if (alpha == 270) H->add_concave_bend(e);
	}
      }
    } 
  } 

  return *H;

}


//#line 603 "Lweb/shape.lw"
AGDModule *ShapeModule::clone() const {
  ShapeModule *S = new ShapeModule();
  return S;
}

