
/**********************************************************

                TricComp.h

  author:    Carsten Gutwenger
  date:      July 1999

**********************************************************/


#ifndef TRIC_COMP_H
#define TRIC_COMP_H


#include <LEDA/graph/graph.h>
#include <LEDA/core/array.h>
#include <LEDA/core/b_stack.h>
#include <LEDA/internal/std/fstream.h>

using namespace leda;


class GraphCopy;

enum { TricComp_bond=0, TricComp_polygon=1, TricComp_triconnected=2 };

typedef int CompType;


class TricComp_comp_struct {


friend class TricComp;
friend class SPQRTree;

		leda::list<leda::edge> edges;
		CompType type;

		TricComp_comp_struct &operator<<(leda::edge e) {
			edges.append(e);
			return *this;
		}

		void finishTricOrPoly(leda::edge e) {
			edges.append(e);
			type = (edges.length() >= 4) ? TricComp_triconnected : TricComp_polygon;
		}

		friend ostream& operator<<(ostream &os, const TricComp_comp_struct&) {
			return os;
		}

		friend istream& operator>>(istream& is, TricComp_comp_struct&) {
			return is;
		}
	};



class TricComp {
enum { bond=0, polygon=1, triconnected=2 };
public:

	TricComp(const leda::graph &G, bool is_map);

	~TricComp();


	GraphCopy  *gcp;
	leda::array<TricComp_comp_struct> component;
	int num_comp;

	bool check_comp();
	bool check_sep_pair(leda::edge e_virt);

private:

	void remove_reversals();
	void split_multi_edges();

	int *TSTACK_h, *TSTACK_a, *TSTACK_b;
	int top;

	void TSTACK_push (int h, int a, int b) {
		TSTACK_h[++top] = h;
		TSTACK_a[top] = a;
		TSTACK_b[top] = b;
	}

	void TSTACK_pushEOS() {
		TSTACK_a[++top] = -1;
	}

	bool TSTACK_notEOS() {
		return (TSTACK_a[top] != -1);
	}

	TricComp_comp_struct &new_comp() {
		return component[num_comp++];
	}

	TricComp_comp_struct &new_comp(CompType t) {
		TricComp_comp_struct &C = component[num_comp++];
		C.type = t;
		return C;
	}

	enum edge_type { unseen, tree, frond, removed };

	void DFS1 (const leda::graph& G, leda::node v, leda::node u);
	void build_acceptable_adj_struct (const leda::graph& G);
	void DFS2 (const leda::graph& G);
	void path_finder(const leda::graph& G, leda::node v);
	void PathSearch (const graph& G, leda::node v);
	void assemble_triconnected_components();

	void print_os(leda::edge e);
	void print_stacks();

	int high(leda::node v) {
		return (HIGHPT[v].empty()) ? 0 : HIGHPT[v].head();
	}

	void del_high(leda::edge e) {
		list_item it = IN_HIGH[e];
		if (it != nil) {
			leda::node v = target(e);
			HIGHPT[v].del_item(it);
		}
	}

	const leda::graph *gp;

	leda::node_array<int>       NUMBER, LOWPT1, LOWPT2, ND, DEGREE;
	leda::array<leda::node>      NODEAT;
	leda::node_array<leda::node> FATHER;
	leda::edge_array<edge_type> TYPE;
	leda::node_array<leda::list<leda::edge> > A;
	leda::node_array<int>  NEWNUM;
	leda::edge_array<bool> START;
	leda::node_array<leda::edge> TREE_ARC;
	leda::node_array<list<int> > HIGHPT;
	leda::edge_array<list_item> IN_ADJ, IN_HIGH;
	leda::b_stack<leda::edge> ESTACK;

	leda::node start;
	int num_count;
	bool new_path;

LEDA_MEMORY(TricComp)
};


#endif
