#include <LEDA/graphics/d3_gl_window.h>


LEDA_BEGIN_NAMESPACE 


void d3gl_redraw(gl_window* w)
{ ((d3_gl_window*)w->get_inf(0))->draw(); } 


d3_gl_window::d3_gl_window(window& P, const gl_camera& gl_cam,
						   const set<d3_gl_scene*>& S) 
: textures(compare), scenes(compare)
{ 
  second_win = (window*)&P;
  cam = (gl_camera*)&gl_cam;  

  gl_win = new gl_window(P.width(),P.height()-P.get_panel_height());       
  
  base_init();
  real_gl = false;   

  width  = P.width();
  height = P.height();
  
  gl_win->display(0,P.get_panel_height(),P);    
  gl_init();
  gl_resize();
  join_scenes(S);
}

d3_gl_window::d3_gl_window(const gl_window& glW, const gl_camera& gl_cam,
			  window* M, const set<d3_gl_scene*>& S)
: textures(compare), scenes(compare)
{
  second_win = M;
  gl_win = (gl_window*)&glW;
  cam = (gl_camera*)&gl_cam;      
  
  base_init();
  real_gl = true;  

  width  = glW.width();
  height = glW.height();  
  
  gl_init();
  gl_resize();
  join_scenes(S);
}

d3_gl_window::~d3_gl_window(){

	if(!real_gl){
		second_win->close();
		gl_win->close();
		delete gl_win;
	}
	else{
		gl_win->close();
		second_win->close();
	}

	seq_item it = textures.min_item();
	while(it){
		delete textures.key(it);
		textures.del_item(it);
		it = textures.min_item();
	}
}

void d3_gl_window::base_init() 
{ 
  draw_axis   = draw_coord_with_ln = draw_coord_with_pts = false;  
  axis_line_w = coord_line_w = pt_size  = 2.0f;
  axis_line_c = coord_line_c = pt_col   = d3_gl_colF(0.0f,0.0f,0.0f,1.0f);
  bg_color    = d3_gl_colF(1.0f,1.0f,1.0f,0.0f);

  xmin = (float) gl_win->xmin();
  xmax = (float) gl_win->xmax();
  ymin = (float) gl_win->ymin();
  ymax = (float) gl_win->ymax();
  zmin = -1.0f;
  zmax =  1.0f;
  
  x_grid_dist = y_grid_dist = z_grid_dist = 1.0;

  prae_gl_func = post_gl_func = NULL;

  rsz_func = d3_gl_resize;
  
  gl_win->set_inf(this,0);
  gl_win->set_redraw(d3gl_redraw);     
}


void d3_gl_window::add_scene(d3_gl_scene* sc)
{ 
	if(sc){
		sc->d3_gl_win = this;
		scenes.insert(sc);
		sc->update_textures();
	}	
}

void d3_gl_window::del_scene(d3_gl_scene* sc)
{ if(sc) scenes.del(sc); }

void d3_gl_window::join_scenes(const set<d3_gl_scene*>& S)
{ 
	d3_gl_scene* P;
	forall(P,S)
		add_scene(P);
}

void d3_gl_window::disjoin_scenes(const set<d3_gl_scene*>& S)
{ scenes.diff(S); }

void d3_gl_window::clear_scenes()
{ scenes.clear(); }


void d3_gl_window::set_coord(const double& x0, const double& x1, const double& y0,
		  const double& y1, const double& z0, const double& z1, const bool& update_cam)
{ 
    double x,y,z;
	
	if(update_cam){
		cam->get_eye(x,y,z);	
		x = (x-xmin)/(xmax-xmin);
		y = (y-ymin)/(ymax-ymin);
		z = (z-zmin)/(zmax-zmin);
	}
	
    xmin = x0;  xmax = x1;
    ymin = y0;  ymax = y1;
    zmin = z0;  zmax = z1;
	
	if(update_cam){
		x = x*(xmax-xmin)+xmin;
		y = y*(ymax-ymin)+ymin;
		z = z*(zmax-zmin)+zmin;
		cam->set_eye(x,y,z);
	}	
      
	gl_resize();
}


void d3_gl_window::set_coord_by_scenes(const bool& update_cam)
{
	if(scenes.empty())
		return;

	double x0, x1;
	double y0, y1;
	double z0, z1;

	double x,y,z;

	if(update_cam){
		cam->get_eye(x,y,z);	
		x = (x-xmin)/(xmax-xmin);
		y = (y-ymin)/(ymax-ymin);
		z = (z-zmin)/(zmax-zmin);
	}

	xmin = 0.0; xmax = 0.0;
	ymin = 0.0; ymax = 0.0;
	zmin = 0.0; zmax = 0.0;

	d3_gl_scene* scene;
	forall(scene,scenes)
	{
		scene->comp_optimal_coord(x0,x1,y0,y1,z0,z1);
		if(x0<xmin) xmin = x0;
		if(x1>xmax) xmax = x1;
		if(y0<ymin) ymin = y0;
		if(y1>ymax) ymax = y1;
		if(z0<zmin) zmin = x0;
		if(z1>zmax) zmax = x1;
	}
	
	float help;

	help = xmin;
	xmin = (xmin - 2.0f) - ((xmax-xmin)*0.5f);
	xmax = (xmax + 2.0f) + ((xmax-help)*0.5f);
	help = ymin;
	ymin = (ymin - 2.0f) - ((ymax-ymin)*0.5f);
	ymax = (ymax + 2.0f) + ((ymax-help)*0.5f);
	help = zmin;
	zmin = (zmin - 2.0f) - ((zmax-zmin)*0.5f);
	zmax = (zmax + 2.0f) + ((zmax-help)*0.5f);
	
	if(update_cam){
		x = x*(xmax-xmin)+xmin;
		y = y*(ymax-ymin)+ymin;
		z = z*(zmax-zmin)+zmin;
		cam->set_eye(x,y,z);
	}
	
	gl_resize();
}


void d3_gl_window::gl_resize() const		
{
	gl_win->gl_start();

	int view_width = gl_win->width();	
	int view_height;

	if(!real_gl){
		view_height = height-second_win->get_panel_height();		
	}
	else{  
		int panel_height;   
		if(second_win == NULL) panel_height = 0;
		else                   panel_height = second_win->get_panel_height(); 
		view_height = height - panel_height;		
	}

	rsz_func(view_width,view_height);								
}

void d3_gl_window::gl_init() const										
{
	gl_win->gl_start();

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL); 
	glClearDepth(1.0f);	
								
	glShadeModel(GL_SMOOTH);		
	
    glClearColor(bg_color.r(),bg_color.g(),bg_color.b(),bg_color.a());	    		
}


void d3_gl_window::draw_pointed_coord(const double& mp_x, const double& mp_y,
									  const double& mp_z) const
{
	double ypos = mp_y;

	while(ypos < ymax){
		draw_pointed_plane(mp_x,mp_y,mp_z,ypos);
		ypos += y_grid_dist;
	}	
	draw_pointed_plane(mp_x,mp_y,mp_z,ymax);

	ypos = mp_y - y_grid_dist;
	while(ypos > ymin){
		draw_pointed_plane(mp_x,mp_y,mp_z,ypos);
		ypos -= y_grid_dist;
	}
	draw_pointed_plane(mp_x,mp_y,mp_z,ymin);
}


void d3_gl_window::draw_pointed_plane(const double& mp_x, const double& mp_y,
									  const double& mp_z, const double& ypos) const
{
	double xpos = mp_x;	
	double zpos = mp_z;

	glBegin(GL_POINTS);		

	while(zpos < zmax){
	
		while(xpos < xmax){
			glVertex3d(xpos,ypos,zpos);
			xpos += x_grid_dist;
		}
		xpos = xmax;
		glVertex3d(xpos,ypos,zpos);

		xpos = mp_x-x_grid_dist;

		while(xpos > xmin){
			glVertex3d(xpos,ypos,zpos);
			xpos -= x_grid_dist;
		}
		xpos = xmin;
		glVertex3d(xpos,ypos,zpos);

		zpos += z_grid_dist;
		xpos = mp_x;
	}
	zpos = zmax;

	while(xpos < xmax){
		glVertex3d(xpos,ypos,zpos);
		xpos += x_grid_dist;
	}
	xpos = xmax;
	glVertex3d(xpos,ypos,zpos);

	xpos = mp_x-x_grid_dist;

	while(xpos > xmin){
		glVertex3d(xpos,ypos,zpos);
		xpos -= x_grid_dist;
	}
	xpos = xmin;
	glVertex3d(xpos,ypos,zpos);

	zpos = mp_z-z_grid_dist;
	xpos = mp_x;

	while(zpos > zmin){
	
		while(xpos < xmax){
			glVertex3d(xpos,ypos,zpos);
			xpos += x_grid_dist;
		}
		xpos = xmax;
		glVertex3d(xpos,ypos,zpos);

		xpos = mp_x-x_grid_dist;

		while(xpos > xmin){
			glVertex3d(xpos,ypos,zpos);
			xpos -= x_grid_dist;
		}
		xpos = xmin;
		glVertex3d(xpos,ypos,zpos);

		zpos -= z_grid_dist;
		xpos = mp_x;
	}
	zpos = zmin;
	
	while(xpos < xmax){
		glVertex3d(xpos,ypos,zpos);
		xpos += x_grid_dist;
	}
	xpos = xmax;
	glVertex3d(xpos,ypos,zpos);

	xpos = mp_x-x_grid_dist;

	while(xpos > xmin){
		glVertex3d(xpos,ypos,zpos);
		xpos -= x_grid_dist;
	}
	xpos = xmin;
	glVertex3d(xpos,ypos,zpos);
	
	glEnd();	
}

void d3_gl_window::draw_coord_with_lines(const double& mp_x, const double& mp_y,
									     const double& mp_z) const
{
	  double xpos, ypos, zpos;	  

	  // parallels to x-axis
	  ypos = mp_y;

	  glBegin(GL_LINES);
	  
	  while(ypos <= ymax){
         
        zpos = mp_z;	
		while(zpos < zmax){
			glVertex3d(xmin,ypos,zpos);
			glVertex3d(xmax,ypos,zpos);			
			zpos += z_grid_dist;
		}
		glVertex3d(xmin,ypos,zmax);
		glVertex3d(xmax,ypos,zmax);

		zpos = mp_z;
		while(zpos > zmin){
			glVertex3d(xmin,ypos,zpos);
			glVertex3d(xmax,ypos,zpos);			
			zpos -= z_grid_dist;
		}
		glVertex3d(xmin,ypos,zmin);
		glVertex3d(xmax,ypos,zmin);

		if(ypos == ymax) break;
		ypos += y_grid_dist;
		if(ypos > ymax) ypos = ymax;		
	  }

	  ypos = mp_y;	  

	  while(ypos >= ymin){
          
        zpos = mp_z; 
		while(zpos < zmax){
			glVertex3d(xmin,ypos,zpos);
			glVertex3d(xmax,ypos,zpos);			
			zpos += z_grid_dist;
		}
		glVertex3d(xmin,ypos,zmax);
		glVertex3d(xmax,ypos,zmax);

		zpos = mp_z;
		while(zpos > zmin){
			glVertex3d(xmin,ypos,zpos);
			glVertex3d(xmax,ypos,zpos);			
			zpos -= z_grid_dist;
		}
		glVertex3d(xmin,ypos,zmin);
		glVertex3d(xmax,ypos,zmin);

		if(ypos == ymin) break;
		ypos -= y_grid_dist;
		if(ypos < ymin) ypos = ymin;		
	  }		  	  	  
	  
	  //  parallels to y-axis
	  zpos = mp_z; 	  

	  while(zpos <= zmax){
		  xpos = mp_x; 	
		  while(xpos < xmax){
			  glVertex3d(xpos,ymin,zpos);
			  glVertex3d(xpos,ymax,zpos);
			  xpos += x_grid_dist;
		  }
	  	  glVertex3d(xmax,ymin,zpos);
		  glVertex3d(xmax,ymax,zpos);

	 	  xpos = mp_x; 
		  while(xpos > xmin){
			  glVertex3d(xpos,ymin,zpos);
			  glVertex3d(xpos,ymax,zpos);
			  xpos -= x_grid_dist;
		  }
		  glVertex3d(xmin,ymin,zpos);
		  glVertex3d(xmin,ymax,zpos);

		  if(zpos == zmax) break;
		  zpos += z_grid_dist;
		  if(zpos > zmax)
			  zpos = zmax;		  
	  }

	  zpos = mp_z; 	  

	  while(zpos >= zmin){
		  xpos = mp_x; 	
		  while(xpos < xmax){
			  glVertex3d(xpos,ymin,zpos);
			  glVertex3d(xpos,ymax,zpos);
			  xpos += x_grid_dist;
		  }
	  	  glVertex3d(xmax,ymin,zpos);
		  glVertex3d(xmax,ymax,zpos);

	 	  xpos = mp_x; 
		  while(xpos > xmin){
			  glVertex3d(xpos,ymin,zpos);
			  glVertex3d(xpos,ymax,zpos);
			  xpos -= x_grid_dist;
		  }
		  glVertex3d(xmin,ymin,zpos);
		  glVertex3d(xmin,ymax,zpos);

		  if(zpos == zmin) break;
		  zpos -= z_grid_dist;
		  if(zpos < zmin)
			  zpos = zmin;		  
	  }      
	  
	  // parallels to z-axis
	  ypos = mp_y; 

	  while(ypos <= ymax){
		  xpos = mp_x; 	
		  while(xpos < xmax){
			  glVertex3d(xpos,ypos,zmin);
			  glVertex3d(xpos,ypos,zmax);
			  xpos += x_grid_dist;
		  }	
		  glVertex3d(xmax,ypos,zmin);
		  glVertex3d(xmax,ypos,zmax);

	 	  xpos = mp_x; 
		  while(xpos > xmin){
			  glVertex3d(xpos,ypos,zmin);
			  glVertex3d(xpos,ypos,zmax);
			  xpos -= x_grid_dist;
		  }
		  glVertex3d(xmin,ypos,zmin);
		  glVertex3d(xmin,ypos,zmax);

		  if(ypos == ymax) break;
		  ypos += y_grid_dist;
		  if(ypos > ymax)
			  ypos = ymax;		  
	  }

	  ypos = mp_y; 

	  while(ypos >= ymin){
		  xpos = mp_x; 	
		  while(xpos < xmax){
			  glVertex3d(xpos,ypos,zmin);
			  glVertex3d(xpos,ypos,zmax);
			  xpos += x_grid_dist;
		  }	
		  glVertex3d(xmax,ypos,zmin);
		  glVertex3d(xmax,ypos,zmax);

	 	  xpos = mp_x; 
		  while(xpos > xmin){
			  glVertex3d(xpos,ypos,zmin);
			  glVertex3d(xpos,ypos,zmax);
			  xpos -= x_grid_dist;
		  }
		  glVertex3d(xmin,ypos,zmin);
		  glVertex3d(xmin,ypos,zmax);

		  if(ypos == ymin) break;
		  ypos -= y_grid_dist;
		  if(ypos < ymin)
			  ypos = ymin;		  
	  }
			
	  glEnd();
}

void d3_gl_window::draw_coord_arrow(vector* v) const
{                                               
	glBegin(GL_TRIANGLES);	    
		gl_vertex(v[0]); gl_vertex(v[1]); gl_vertex(v[2]);
		gl_vertex(v[0]); gl_vertex(v[2]); gl_vertex(v[3]);
		gl_vertex(v[0]); gl_vertex(v[3]); gl_vertex(v[4]);
		gl_vertex(v[0]); gl_vertex(v[4]); gl_vertex(v[1]);
	glEnd();

	glBegin(GL_QUADS);
		gl_vertex(v[1]); gl_vertex(v[2]); gl_vertex(v[3]); gl_vertex(v[4]);
	glEnd();
}


void d3_gl_window::draw_scenes() 
{ 
  gl_win->gl_start();     
 
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  
	
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  (*cam)();    
  
  if(prae_gl_func)
	prae_gl_func(this,gl_win);     
  
  d3_gl_scene* sc_ptr;  
  forall(sc_ptr,scenes){
	  if(sc_ptr->get_visible())		  
		  sc_ptr->draw(); 	  
  }
  
  double x_mp = xmin + (xmax-xmin)/2;
  double y_mp = ymin + (ymax-ymin)/2;
  double z_mp = zmin + (zmax-zmin)/2;
  
  if(draw_coord_with_ln){
	  glLineWidth(coord_line_w);
	  coord_line_c();	  	  
	  draw_coord_with_lines(x_mp,y_mp,z_mp);
  }

  if(draw_coord_with_pts){
	  glPointSize(pt_size);
	  pt_col();	  	  
	  draw_pointed_coord(x_mp,y_mp,z_mp);
  }

  if(draw_axis){
	  glLineWidth(axis_line_w);
	  axis_line_c();	  

	  glBegin(GL_LINES);
		glVertex3d(xmin,y_mp,z_mp);
		glVertex3d(xmax,y_mp,z_mp);

		glVertex3d(x_mp,ymin,z_mp);
		glVertex3d(x_mp,ymax,z_mp);

		glVertex3d(x_mp,y_mp,zmin);
		glVertex3d(x_mp,y_mp,zmax);
	  glEnd();
	  
	  // left coord arrow
	  vector v[5]; 
	  v[0] = vector(xmin,y_mp,z_mp);
	  v[1] = vector(xmin+(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			        -(xmax-xmin)/ARROW_QUAD_QUOTIENT,
				     (xmax-xmin)/ARROW_QUAD_QUOTIENT);
	  v[2] = vector(xmin+(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			         (xmax-xmin)/ARROW_QUAD_QUOTIENT,
				     (xmax-xmin)/ARROW_QUAD_QUOTIENT);
	  v[3] = vector(xmin+(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			         (xmax-ymin)/ARROW_QUAD_QUOTIENT,
				    -(xmax-xmin)/ARROW_QUAD_QUOTIENT);
	  v[4] = vector(xmin+(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			        -(xmax-xmin)/ARROW_QUAD_QUOTIENT,
				    -(xmax-xmin)/ARROW_QUAD_QUOTIENT);

	  draw_coord_arrow(v);

	  // right coord arrow
	  v[0] = vector(xmax,y_mp,z_mp);
	  v[1] = vector(xmax-(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			        -(xmax-xmin)/ARROW_QUAD_QUOTIENT,
				     (xmax-xmin)/ARROW_QUAD_QUOTIENT);
	  v[2] = vector(xmax-(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			         (xmax-xmin)/ARROW_QUAD_QUOTIENT,
				     (xmax-xmin)/ARROW_QUAD_QUOTIENT);
	  v[3] = vector(xmax-(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			         (xmax-xmin)/ARROW_QUAD_QUOTIENT,
				     -(xmax-xmin)/ARROW_QUAD_QUOTIENT);
	  v[4] = vector(xmax-(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			         -(xmax-xmin)/ARROW_QUAD_QUOTIENT,
				     -(xmax-xmin)/ARROW_QUAD_QUOTIENT);

	  draw_coord_arrow(v);

	  // front coord arrow
	  v[0] = vector(x_mp,y_mp,zmax);
	  v[1] = vector( (zmax-zmin)/ARROW_QUAD_QUOTIENT,
			        -(zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmax-(zmax-zmin)/ARROW_DEPTH_QUOTIENT);
	  v[2] = vector( (zmax-zmin)/ARROW_QUAD_QUOTIENT,
			         (zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmax-(zmax-zmin)/ARROW_DEPTH_QUOTIENT);
	  v[3] = vector(-(zmax-zmin)/ARROW_QUAD_QUOTIENT,
			         (zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmax-(zmax-zmin)/ARROW_DEPTH_QUOTIENT);
	  v[4] = vector(-(zmax-zmin)/ARROW_QUAD_QUOTIENT,
			        -(zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmax-(zmax-zmin)/ARROW_DEPTH_QUOTIENT);

	  draw_coord_arrow(v);

	  // back coord arrow
	  v[0] = vector(x_mp,y_mp,zmin);
	  v[1] = vector( (zmax-zmin)/ARROW_QUAD_QUOTIENT,
			        -(zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmin+(zmax-zmin)/ARROW_DEPTH_QUOTIENT);
	  v[2] = vector( (zmax-zmin)/ARROW_QUAD_QUOTIENT,
			         (zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmin+(zmax-zmin)/ARROW_DEPTH_QUOTIENT);
	  v[3] = vector(-(zmax-zmin)/ARROW_QUAD_QUOTIENT,
			         (zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmin+(zmax-zmin)/ARROW_DEPTH_QUOTIENT);
	  v[4] = vector(-(zmax-zmin)/ARROW_QUAD_QUOTIENT,
			        -(zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmin+(zmax-zmin)/ARROW_DEPTH_QUOTIENT);

	  draw_coord_arrow(v);
	  
	  // bottom coord arrow
	  v[0] = vector(x_mp,ymin,z_mp);
	  v[1] = vector(-(ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymin+(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					 (ymax-ymin)/ARROW_QUAD_QUOTIENT);
	  v[2] = vector(-(ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymin+(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					-(ymax-ymin)/ARROW_QUAD_QUOTIENT);	  
	  v[3] = vector( (ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymin+(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					-(ymax-ymin)/ARROW_QUAD_QUOTIENT);	  
	  v[4] = vector( (ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymin+(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					 (ymax-ymin)/ARROW_QUAD_QUOTIENT);
	  
	  draw_coord_arrow(v);

	  // top coord arrow
	  v[0] = vector(x_mp,ymax,z_mp);
	  v[1] = vector(-(ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymax-(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					 (ymax-ymin)/ARROW_QUAD_QUOTIENT);
	  v[2] = vector(-(ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymax-(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					-(ymax-ymin)/ARROW_QUAD_QUOTIENT);	  
	  v[3] = vector( (ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymax-(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					-(ymax-ymin)/ARROW_QUAD_QUOTIENT);	  
	  v[4] = vector( (ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymax-(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					 (ymax-ymin)/ARROW_QUAD_QUOTIENT);
	  
	  draw_coord_arrow(v);
	  
  }
  
  if(post_gl_func)
	post_gl_func(this,gl_win); 
   
  glFinish();

  gl_win->gl_finish();    
}


void d3_gl_window::draw() 
{       
  // case 1 : gl_window is sub-window -> resize it if necessary  
  if(!real_gl){    
	if(second_win->width() != width || second_win->height() != height){		
		width  = second_win->width();
		height = second_win->height();

		gl_win->resize(0,second_win->get_panel_height(),second_win->width(),
			second_win->height() - second_win->get_panel_height() );

		BASE_WINDOW* SW = second_win->get_status_window();
		
		if(SW && !SW->is_closed()){
			close_status_window();
			open_status_window(SW->get_bg_color(),SW->height());			
		}				

		gl_resize();		
	}
  }
  // case 2 : gl_window is main-window -> resize panel-win if it exists and if it is necessary
  else{ 	  
	if(gl_win->width() != width || gl_win->height() != height){				
		width  = gl_win->width();
		height = gl_win->height();	     				
		if(second_win)
			second_win->resize(0,0,width,second_win->get_panel_height());

		BASE_WINDOW* SW = gl_win->get_status_window();
		
		if(SW && !SW->is_closed()){
			close_status_window();
			open_status_window(SW->get_bg_color(),SW->height());			
		}
		
		gl_resize();		
	}	
  }  
  
  d3_gl_window::draw_scenes(); 
}


GLuint* d3_gl_window::reg_tex(char* t_data, int wi, int he,const GLint& filter)
{
	gl_win->gl_start();

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	
	GLuint* t_num = new GLuint(0);
	
	glGenTextures(1, t_num);	

	glBindTexture(GL_TEXTURE_2D, *t_num);   	
        glTexImage2D(GL_TEXTURE_2D, 0, 3, wi, he, 0, GL_RGB, GL_UNSIGNED_BYTE,
	                                                 (GLubyte*)t_data);
	
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter);  
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter);	
	  
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP); 
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP); 
     
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

    textures.insert(t_num,tex_params(t_data,wi,he,filter));		

	gl_win->gl_finish();
	
	return t_num;
}

void d3_gl_window::reg_scene_tex(GLuint* t_num, char* t_data, int wi, int he,
                                                          const GLint& filter)
{
	gl_win->gl_start();

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);	
	
	glGenTextures(1, t_num);	

	glBindTexture(GL_TEXTURE_2D, *t_num);   	
    glTexImage2D(GL_TEXTURE_2D, 0, 3, wi, he, 0, GL_RGB, GL_UNSIGNED_BYTE,
	                                                  (GLubyte*)t_data);
	
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter);  
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter);	
	  
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP); 
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP); 
     
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);    		

	gl_win->gl_finish();
}

void d3_gl_window::print_tex_nums()
{
	seq_item it = textures.min_item();
	while(it){
		cout<<*(textures.key(it))<<" ";
		it = textures.succ(it);
	}	
}

void d3_gl_window::unreg_tex(GLuint* t_num)
{
	gl_win->gl_start();	

	seq_item it = textures.lookup(t_num);
	if(it){
		glDeleteTextures(1,t_num);
		delete textures.key(it);
		textures.del_item(it);				
	}	

	gl_win->gl_finish();
}

void d3_gl_window::update_textures()
{
	gl_win->gl_start();
			
	seq_item it = textures.min_item();
	while(it){
		GLuint* t_num = textures.key(it);
		tex_params& T = textures[it];

		glGenTextures(1, t_num);	

		glBindTexture(GL_TEXTURE_2D, *t_num);   	
		glTexImage2D(GL_TEXTURE_2D, 0, 3, T.second(),
		T.third(), 0, GL_RGB, GL_UNSIGNED_BYTE,
		(GLubyte*)T.first());
	
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, T.fourth());  
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, T.fourth());	
	  
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP); 
		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP); 
     
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);		
	}	

	glFinish();
	gl_win->gl_finish();
}


void d3_gl_window::set_tex_filter(GLuint* t_num, const GLint& filter)
{	
	seq_item it = textures.lookup(t_num);    

	if(it == nil) return;

	gl_win->gl_start();
	
	glBindTexture(GL_TEXTURE_2D,*t_num);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter);  
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter);

	gl_win->gl_finish();
}

GLint d3_gl_window::get_tex_filter(GLuint* t_num)
{
	seq_item it = textures.lookup(t_num);

	if(it)
		return textures[it].fourth();
	else
		return -1;
}



// event-functions 

#define BUFSIZE 512

d3_gl_scene* d3_gl_window::pick_object(const double& x, const double& y) const
{
	unsigned int selectBuf[BUFSIZE];
	int hits;	
	int viewport[4]; // view port
	
	double x2 = gl_win->real_to_pix(x);
	double y2 = gl_win->real_to_pix(y);
	
	gl_win->gl_start();
	glGetIntegerv(GL_VIEWPORT, viewport);

	glSelectBuffer(BUFSIZE, selectBuf);
	glRenderMode(GL_SELECT);

	glInitNames();
	glPushName(0);	
	
	if(viewport[3] == 0) viewport[3] = 1;

	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();  
	gluPickMatrix(x2, y2, 5.0, 5.0, viewport);
    gluPerspective(50.0f, ((GLfloat)viewport[2])/((GLfloat)viewport[3]), 1.0f, 10000000000.0f);	

	glMatrixMode(GL_MODELVIEW);							
	glLoadIdentity();
	(*cam)();
  
	d3_gl_scene* sc_ptr;
    unsigned int index = 1;
    forall(sc_ptr,scenes){
	    if(sc_ptr->get_visible()){
	    	  glLoadName(index);
		      ++index;
		      sc_ptr->draw(); 
		}
	}	
	
	hits = glRenderMode(GL_RENDER);

	gl_win->gl_finish();
	gl_resize();

	// process hits
	if(hits){ // always take first hit
		unsigned int* ptr = selectBuf;
		++ptr; ++ptr; ++ptr;
		index = 1;		

		forall(sc_ptr,scenes){
			if(!sc_ptr->get_visible()) continue;
			            
			if(index == *ptr){	
				return sc_ptr;				
			}
			else
				++index;
		}		
	}

	return NULL;
}

int d3_gl_window::get_event(int& val, double& x, double& y) const
{    
  window* sub_win;
  BASE_WINDOW* status_win;

  if(real_gl)
  {
	  if(second_win==NULL)
		  sub_win=gl_win;
	  else
		  sub_win=second_win;

	  status_win = gl_win->get_status_window();
  }
  else{
	  sub_win = second_win;
	  status_win = second_win->get_status_window();	  
  }

  BASE_WINDOW* w;
       
  int e = BASE_WINDOW::get_event(w,val,x,y);  

  if(w){
	if(w == gl_win || w == status_win || w == sub_win)	
		return e;		
	else
		return no_event; 
  }

  return no_event;
}

int d3_gl_window::get_mouse(double& x, double& y) const
{
  int val;
  if(get_event(val,x,y)==button_press_event)
 	  return val;
  else
	  return NO_BUTTON;
}

int d3_gl_window::read_mouse(double& x, double& y) const
{
  int but;  
  do but = d3_gl_window::get_mouse(x,y); while (but == NO_BUTTON);  
  return but;
}

int d3_gl_window::read_event(int& val, double& x, double& y) const
{
  int event;  
  do event = d3_gl_window::get_event(val,x,y); while (event == no_event);  
  return event;
}

BASE_WINDOW* d3_gl_window::create_status_window(color bc, int h) const
{
	if(!real_gl){ 		
		if(second_win->get_status_window() && !second_win->get_status_window()->is_closed()){			
			gl_win->resize(0,second_win->get_panel_height(),second_win->width(),
			second_win->height() - second_win->get_panel_height() - h);			
		}
		
		gl_resize();			

		return second_win->create_status_window(bc,h);
	}
	else{
		return gl_win->create_status_window(bc,h);
	}
}

BASE_WINDOW* d3_gl_window::open_status_window(color bc, int h) const
{
	if(!real_gl){ 	
		return second_win->open_status_window(bc,h);
	}
	else{
		return gl_win->open_status_window(bc,h);		
	}
}

void d3_gl_window::close_status_window() const
{
	if(!real_gl){
		second_win->close_status_window();				
	}
	else gl_win->close_status_window();
}

void d3_gl_window::destroy_status_window() const
{
	close_status_window();

	if(!real_gl){		
		second_win->destroy_status_window();
	}
	else gl_win->destroy_status_window();
}

void d3_gl_window::set_status_string(const char* str) const
{ if(!real_gl) second_win->set_status_string(str);
  else gl_win->set_status_string(str);       
 }

const char* d3_gl_window::get_status_string() const
{ if(!real_gl) return second_win->get_status_string();
  else return gl_win->get_status_string();
}

BASE_WINDOW* d3_gl_window::get_status_window() const
{ if(!real_gl) return second_win->get_status_window();
  else return gl_win->get_status_window();
}


gl_window* d3_gl_window::set_panel_win(window* M)
{
	if(M == NULL) return NULL;

	if(!real_gl){ 
		gl_window* old_gl_win = gl_win;		
		
		gl_win = new gl_window(M->width(),M->height()-M->get_panel_height()); 
		gl_win->set_inf(this,0);
		gl_win->set_redraw(d3gl_redraw);
		
		M->display();
		M->resize(second_win->xpos(),second_win->ypos(),second_win->width(),
			second_win->height());			
		gl_win->display(0,M->get_panel_height(),*M);
		gl_win->resize(0,M->get_panel_height(),M->width(),M->height()-M->get_panel_height());				
		
        second_win->close();  
		second_win = M;

		delete old_gl_win;
		old_gl_win = NULL;		
		
		gl_init();
		gl_resize();		

		update_textures();

		d3_gl_scene* sc;
		forall(sc,scenes)
			sc->update_textures();
		
		return gl_win;
	}
	else{		
		M->display(*gl_win,0,0);
		M->resize(0,0,gl_win->width(),M->get_panel_height());

		if(second_win)
			second_win->close();
		
		second_win = M;
		
		gl_resize();

		return NULL;
	}		
}

void d3_gl_window::get_coord(double& x0, double& x1, double& y0, double& y1,
							 double& z0, double& z1) const
{
	x0 = xmin; x1 = xmax;
	y0 = ymin; y1 = ymax;
	z0 = zmin; z1 = zmax;
}

void d3_gl_window::set_bg_color(const d3_gl_colF& c)
{ 
	bg_color = c;
	gl_win->gl_start();
	glClearColor(bg_color.r(),bg_color.g(),bg_color.b(),bg_color.a());
	glFinish();
	gl_win->gl_finish();
}

int d3_gl_window::get_panel_height() const
{
	if(second_win)
		return second_win->get_panel_height();
	else
		return -1;	
}

void d3_gl_window::center_camera() const
{
	double x_mp = xmin + (xmax-xmin)/2;
	double y_mp = ymin + (ymax-ymin)/2;
	double z_mp = zmin + (zmax-zmin)/2;
	cam->set(x_mp, y_mp, z_mp, x_mp, y_mp, z_mp-5.0, 0.0, 1.0, 0.0);
}


void d3_gl_resize(int w, int h)
{
	glViewport(0,0,w,h);
	
	if(h == 0) h = 1; // prevent a division by zero			

	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();  
    gluPerspective(50.0f, ((GLfloat)w)/((GLfloat)h), 1.0f, 10000000000.0f);		

	glMatrixMode(GL_MODELVIEW);							
	glLoadIdentity();
}


int compare(const gl_uint_ptr& p1, const gl_uint_ptr& p2)
{ 	if(p1 == p2)
		return 0;
	else
		return -1;
}


LEDA_END_NAMESPACE
