#include <LEDA/d3_gl_col.h>

LEDA_BEGIN_NAMESPACE

d3_gl_col::d3_gl_col()
{
	c.first()  = 1;
	c.second() = 1;
	c.third()  = 1;
	c.fourth() = 1;
}

d3_gl_col::d3_gl_col(const GLfloat& r, const GLfloat& g, const GLfloat& b, const GLfloat& a)
{
	c.first()  = r;
	c.second() = g;
	c.third()  = b;
	c.fourth() = a;
	correct_rgba();
}

d3_gl_col::d3_gl_col(const GLfloat array[4])
{
	c.first()  = array[0];
	c.second() = array[1];
	c.third()  = array[2];
	c.fourth() = array[3];
	correct_rgba();
}

d3_gl_col::d3_gl_col(const color& c2, const GLfloat& a)
{
	int r,g,b;
	c2.get_rgb(r,g,b);
	c.first()  = ((GLfloat)r)/255.0f;
	c.second() = ((GLfloat)g)/255.0f;
	c.third()  = ((GLfloat)b)/255.0f;
	c.fourth() = a;
	correct_rgba();
}

d3_gl_col::d3_gl_col(const d3_gl_col& c2)
{	c = c2.c;	}

void d3_gl_col::correct_rgba()
{
	if(c.first() < 0) c.first() = 0;
	if(c.first() > 1) c.first() = 1;
	if(c.second() < 0) c.second() = 0;
	if(c.second() > 1) c.second() = 1;
	if(c.third() < 0) c.third() = 0;
	if(c.third() > 1) c.third() = 1;
	if(c.fourth() < 0) c.fourth() = 0;
	if(c.fourth() > 1) c.fourth() = 1;
}

void d3_gl_col::get_rgba(GLfloat& r, GLfloat& g, GLfloat& b, GLfloat& a) const
{
	r = c.first();
	g = c.second();
	b = c.third();
	a = c.fourth();
}

void d3_gl_col::get_rgba(GLfloat array[4]) const
{
	array[0] = c.first();
	array[1] = c.second();
	array[2] = c.third();
	array[3] = c.fourth();
}

void d3_gl_col::set_rgba(const GLfloat& r, const GLfloat& g,
						 const GLfloat& b, const GLfloat& a)
{
	c.first()  = r;
	c.second() = g;
	c.third()  = b;
	c.fourth() = a;
	correct_rgba();
}

void d3_gl_col::set_rgba(const GLfloat array[4])
{
	c.first()  = array[0];
	c.second() = array[1];
	c.third()  = array[2];
	c.fourth() = array[3];
	correct_rgba();
}

color d3_gl_col::get_color() const
{
	return color( (int)(255.0*c.first()), (int)(255.0*c.second()), (int)(255.0*c.third()) );
}

void d3_gl_col::set_color(const color& c2)
{
	int r,g,b;
	c2.get_rgb(r,g,b);
	c.first()  = ((GLfloat)r) / 255.0f;
	c.second() = ((GLfloat)g) / 255.0f;
	c.third()  = ((GLfloat)b) / 255.0f;
	c.fourth() = 1;
}

const d3_gl_col& d3_gl_col::operator=(const d3_gl_col& c2)
{
	c = c2.c;
	return *this;
}

bool d3_gl_col::operator==(const d3_gl_col& c2) const
{	return c==c2.c;   }

bool d3_gl_col::operator!=(const d3_gl_col& c2) const
{	return c!=c2.c;	  }


void d3_gl_col::operator()() const
{	glColor4f(c.first(),c.second(),c.third(),c.fourth());   }



ostream& operator<<(ostream& os, const d3_gl_col& c)
{	
	os << c.c;
	return os;
}

istream& operator>>(istream& in, d3_gl_col& d3_c)
{
	in >> d3_c.c;
	d3_c.correct_rgba();
	return in;
}


LEDA_END_NAMESPACE
