#include <LEDA/d3_gl_window.h>


LEDA_BEGIN_NAMESPACE 


void d3gl_redraw(gl_window* w)
{ ((d3_gl_window*)w->get_inf(0))->draw(); } 

d3_gl_window::~d3_gl_window(){

	if(!real_gl){
		second_win->close();
		gl_win->close();
		delete gl_win;
	}
	else{
		gl_win->close();
		second_win->close();
	}
}

void d3_gl_window::base_init() 
{ 
  draw_axis   = draw_coord   = draw_pts = false;  
  axis_line_w = coord_line_w = pt_size  = 2.0f;
  axis_line_c = coord_line_c = pt_col   = d3_gl_col(0.0f,0.0f,0.0f,1.0f);
  bg_color    = d3_gl_col(1.0f,1.0f,1.0f,0.0f);

  xmin = (float) gl_win->xmin();
  xmax = (float) gl_win->xmax();
  ymin = (float) gl_win->ymin();
  ymax = (float) gl_win->ymax();
  zmin = -1.0f;
  zmax =  1.0f;
  
  x_grid_dist = y_grid_dist = z_grid_dist = 1.0;
  
  gl_win->set_inf(this,0);
  gl_win->set_redraw(d3gl_redraw);     
}


void d3_gl_window::init(window& W, gl_camera& gl_cam) 
{   
  second_win = &W;
  cam = &gl_cam;  

  gl_win = new gl_window(W.width(),W.height()-W.get_panel_height());       
  
  base_init();
  real_gl = false;   

  width  = W.width();
  height = W.height();
  
  gl_win->display(0,W.get_panel_height(),W);  
  gl_init();
  gl_resize();
}

void d3_gl_window::init(gl_window& glW, gl_camera& gl_cam, window* panel_ptr) 
{   
  second_win = panel_ptr;
  gl_win = &glW;
  cam = &gl_cam;      
  
  base_init();
  real_gl = true;  

  width  = glW.width();
  height = glW.height();  
  
  gl_init();
  gl_resize();
}


void d3_gl_window::init(list<d3_gl_scene*>&  d3glsc)
{
	scenes.clear();

	d3_gl_scene* scene;
	forall(scene,d3glsc){
		scene->d3_gl_win = this;
		scenes.append(scene);
		scene->re_reg_all_textures();
	}
} 


void d3_gl_window::clear_scenes()
{ scenes.clear(); }

void d3_gl_window::append_scene(d3_gl_scene* sc)
{   if(sc){
		sc->d3_gl_win = this;
		scenes.append(sc);
		sc->re_reg_all_textures();
	}
}


void d3_gl_window::remove_scene(d3_gl_scene* scene)
{ scenes.remove(scene); }


void d3_gl_window::remove_scene(const int& i)
{ 
  if(i > -1 && i < scenes.length()){
	  scenes.remove(scenes[scenes[i]]);
  }
  else
	  LEDA_EXCEPTION(1,"d3_gl_window: index out of range");
}


void d3_gl_window::clear_draws()
{ draws.clear(); }

void d3_gl_window::append_draw(additional_draw d)
{ if(d) draws.append(d); }

void d3_gl_window::remove_draw(additional_draw d)
{ draws.remove(d); }

void d3_gl_window::remove_draw(const int& i)
{ 
  if(i > -1 && i < draws.length()){
	  draws.remove(draws[draws[i]]);
  }
  else
	  LEDA_EXCEPTION(1,"d3_gl_window: index out of range");
}


void d3_gl_window::set_coord_by_points()
{
	if(scenes.empty())
		return;

	double x0, x1;
	double y0, y1;
	double z0, z1;

	xmin = 0.0; xmax = 0.0;
	ymin = 0.0; ymax = 0.0;
	zmin = 0.0; zmax = 0.0;

	d3_gl_scene* scene;
	forall(scene,scenes)
	{
		scene->comp_optimal_coord(x0,x1,y0,y1,z0,z1);
		if(x0<xmin) xmin = x0;
		if(x1>xmax) xmax = x1;
		if(y0<ymin) ymin = y0;
		if(y1>ymax) ymax = y1;
		if(z0<zmin) zmin = x0;
		if(z1>zmax) zmax = x1;
	}
	
	float help;

	help = xmin;
	xmin = (xmin - 2.0f) - ((xmax-xmin)*0.5f);
	xmax = (xmax + 2.0f) + ((xmax-help)*0.5f);
	help = ymin;
	ymin = (ymin - 2.0f) - ((ymax-ymin)*0.5f);
	ymax = (ymax + 2.0f) + ((ymax-help)*0.5f);
	help = zmin;
	zmin = (zmin - 2.0f) - ((zmax-zmin)*0.5f);
	zmax = (zmax + 2.0f) + ((zmax-help)*0.5f);	
	
	gl_resize();
}

void d3_gl_window::set_coord(const double& x0, const double& x1, const double& y0,
		  const double& y1, const double& z0, const double& z1, const bool& update_cam)
{ 
    double x,y,z;
	
	if(update_cam){
		cam->get_eye(x,y,z);	
		x = (x-xmin)/(xmax-xmin);
		y = (y-ymin)/(ymax-ymin);
		z = (z-zmin)/(zmax-zmin);
	}
	
    xmin = x0;  xmax = x1;
    ymin = y0;  ymax = y1;
    zmin = z0;  zmax = z1;
	
	if(update_cam){
		x = x*(xmax-xmin)+xmin;
		y = y*(ymax-ymin)+ymin;
		z = z*(zmax-zmin)+zmin;
		cam->set_eye(x,y,z);
	}	
      
	gl_resize();
}


void d3_gl_window::gl_resize()		
{
	gl_win->gl_start();

	int view_width = gl_win->width();	
	int view_height;

	if(!real_gl){
		view_height = height-second_win->get_panel_height();		
	}
	else{  
		int panel_height;   
		if(second_win == NULL) panel_height = 0;
		else                   panel_height = second_win->get_panel_height(); 
		view_height = height - panel_height;		
	}
	glViewport(0,0,view_width,view_height);
	
	if(view_height == 0) // prevent a division by zero
		view_height = 1;	

	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();  
    gluPerspective(50.0f, ((GLfloat)view_width)/((GLfloat)view_height), 1.0f, 10000000000.0f);		

	glMatrixMode(GL_MODELVIEW);							
	glLoadIdentity();							
}

void d3_gl_window::gl_init()										
{
	gl_win->gl_start();

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL); 
	glClearDepth(1.0f);	
								
	glShadeModel(GL_SMOOTH);		
	
    glClearColor(bg_color.r(),bg_color.g(),bg_color.b(),bg_color.a());	    		
}


inline void d3_gl_window::draw_coords()
{
	  double xpos, ypos, zpos;	  

	  // parallels to x-axis
	  ypos = 0.0;
	  
	  while(ypos <= ymax){
         
        zpos = 0.0;	
		while(zpos < zmax){
			glVertex3d(xmin,ypos,zpos);
			glVertex3d(xmax,ypos,zpos);			
			zpos += z_grid_dist;
		}
		glVertex3d(xmin,ypos,zmax);
		glVertex3d(xmax,ypos,zmax);

		zpos = 0.0;
		while(zpos > zmin){
			glVertex3d(xmin,ypos,zpos);
			glVertex3d(xmax,ypos,zpos);			
			zpos -= z_grid_dist;
		}
		glVertex3d(xmin,ypos,zmin);
		glVertex3d(xmax,ypos,zmin);

		if(ypos == ymax) break;
		ypos += y_grid_dist;
		if(ypos > ymax) ypos = ymax;		
	  }

	  ypos = 0.0;	  

	  while(ypos >= ymin){
          
        zpos = 0.0; 
		while(zpos < zmax){
			glVertex3d(xmin,ypos,zpos);
			glVertex3d(xmax,ypos,zpos);			
			zpos += z_grid_dist;
		}
		glVertex3d(xmin,ypos,zmax);
		glVertex3d(xmax,ypos,zmax);

		zpos = 0.0;
		while(zpos > zmin){
			glVertex3d(xmin,ypos,zpos);
			glVertex3d(xmax,ypos,zpos);			
			zpos -= z_grid_dist;
		}
		glVertex3d(xmin,ypos,zmin);
		glVertex3d(xmax,ypos,zmin);

		if(ypos == ymin) break;
		ypos -= y_grid_dist;
		if(ypos < ymin) ypos = ymin;		
	  }		  	  	  
	  
	  //  parallels to y-axis
	  zpos = 0.0; 	  

	  while(zpos <= zmax){
		  xpos = 0.0; 	
		  while(xpos < xmax){
			  glVertex3d(xpos,ymin,zpos);
			  glVertex3d(xpos,ymax,zpos);
			  xpos += x_grid_dist;
		  }
	  	  glVertex3d(xmax,ymin,zpos);
		  glVertex3d(xmax,ymax,zpos);

	 	  xpos = 0.0; 
		  while(xpos > xmin){
			  glVertex3d(xpos,ymin,zpos);
			  glVertex3d(xpos,ymax,zpos);
			  xpos -= x_grid_dist;
		  }
		  glVertex3d(xmin,ymin,zpos);
		  glVertex3d(xmin,ymax,zpos);

		  if(zpos == zmax) break;
		  zpos += z_grid_dist;
		  if(zpos > zmax)
			  zpos = zmax;		  
	  }

	  zpos = 0.0; 	  

	  while(zpos >= zmin){
		  xpos = 0.0; 	
		  while(xpos < xmax){
			  glVertex3d(xpos,ymin,zpos);
			  glVertex3d(xpos,ymax,zpos);
			  xpos += x_grid_dist;
		  }
	  	  glVertex3d(xmax,ymin,zpos);
		  glVertex3d(xmax,ymax,zpos);

	 	  xpos = 0.0; 
		  while(xpos > xmin){
			  glVertex3d(xpos,ymin,zpos);
			  glVertex3d(xpos,ymax,zpos);
			  xpos -= x_grid_dist;
		  }
		  glVertex3d(xmin,ymin,zpos);
		  glVertex3d(xmin,ymax,zpos);

		  if(zpos == zmin) break;
		  zpos -= z_grid_dist;
		  if(zpos < zmin)
			  zpos = zmin;		  
	  }      
	  
	  // parallels to z-axis
	  ypos = 0.0; 

	  while(ypos <= ymax){
		  xpos = 0.0; 	
		  while(xpos < xmax){
			  glVertex3d(xpos,ypos,zmin);
			  glVertex3d(xpos,ypos,zmax);
			  xpos += x_grid_dist;
		  }	
		  glVertex3d(xmax,ypos,zmin);
		  glVertex3d(xmax,ypos,zmax);

	 	  xpos = 0.0; 
		  while(xpos > xmin){
			  glVertex3d(xpos,ypos,zmin);
			  glVertex3d(xpos,ypos,zmax);
			  xpos -= x_grid_dist;
		  }
		  glVertex3d(xmin,ypos,zmin);
		  glVertex3d(xmin,ypos,zmax);

		  if(ypos == ymax) break;
		  ypos += y_grid_dist;
		  if(ypos > ymax)
			  ypos = ymax;		  
	  }

	  ypos = 0.0; ;

	  while(ypos >= ymin){
		  xpos = 0.0; 	
		  while(xpos < xmax){
			  glVertex3d(xpos,ypos,zmin);
			  glVertex3d(xpos,ypos,zmax);
			  xpos += x_grid_dist;
		  }	
		  glVertex3d(xmax,ypos,zmin);
		  glVertex3d(xmax,ypos,zmax);

	 	  xpos = 0.0; 
		  while(xpos > xmin){
			  glVertex3d(xpos,ypos,zmin);
			  glVertex3d(xpos,ypos,zmax);
			  xpos -= x_grid_dist;
		  }
		  glVertex3d(xmin,ypos,zmin);
		  glVertex3d(xmin,ypos,zmax);

		  if(ypos == ymin) break;
		  ypos -= y_grid_dist;
		  if(ypos < ymin)
			  ypos = ymin;		  
	  }
			
	  glEnd();
}

inline void d3_gl_window::draw_coord_arrow(vector* v) 
{                                               
	glBegin(GL_TRIANGLES);	    
		gl_vertex(v[0]); gl_vertex(v[1]); gl_vertex(v[2]);
		gl_vertex(v[0]); gl_vertex(v[2]); gl_vertex(v[3]);
		gl_vertex(v[0]); gl_vertex(v[3]); gl_vertex(v[4]);
		gl_vertex(v[0]); gl_vertex(v[4]); gl_vertex(v[1]);
	glEnd();

	glBegin(GL_QUADS);
		gl_vertex(v[1]); gl_vertex(v[2]); gl_vertex(v[3]); gl_vertex(v[4]);
	glEnd();
}


void d3_gl_window::draw_scenes() 
{ 
  gl_win->gl_start();     
 
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  
	
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  (*cam)();    
  
  additional_draw d;
  forall(d,draws)
  	  d(this,gl_win);    
  
  d3_gl_scene* sc_ptr;  
  forall(sc_ptr,scenes){
	  if(sc_ptr->get_visible())		  
		  sc_ptr->draw(); 	  
  }       
  
  if(draw_coord){
	  glLineWidth(coord_line_w);
	  coord_line_c();	  
	  glBegin(GL_LINES);
	  draw_coords();
  }

  if(draw_pts){
	  glPointSize(pt_size);
	  pt_col();	  
	  glBegin(GL_POINTS);
	  draw_coords();
  }

  if(draw_axis){
	  glLineWidth(axis_line_w);
	  axis_line_c();	  

	  glBegin(GL_LINES);
		glVertex3d(xmin,0.0,0.0);
		glVertex3d(xmax,0.0,0.0);

		glVertex3d(0.0,ymin,0.0);
		glVertex3d(0.0,ymax,0.0);

		glVertex3d(0.0,0.0,zmin);
		glVertex3d(0.0,0.0,zmax);
	  glEnd();
	  
	  // left coord arrow
	  vector v[5]; 
	  v[0] = vector(xmin,0.0,0.0);
	  v[1] = vector(xmin+(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			        -(xmax-xmin)/ARROW_QUAD_QUOTIENT,
				     (xmax-xmin)/ARROW_QUAD_QUOTIENT);
	  v[2] = vector(xmin+(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			         (xmax-xmin)/ARROW_QUAD_QUOTIENT,
				     (xmax-xmin)/ARROW_QUAD_QUOTIENT);
	  v[3] = vector(xmin+(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			         (xmax-ymin)/ARROW_QUAD_QUOTIENT,
				    -(xmax-xmin)/ARROW_QUAD_QUOTIENT);
	  v[4] = vector(xmin+(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			        -(xmax-xmin)/ARROW_QUAD_QUOTIENT,
				    -(xmax-xmin)/ARROW_QUAD_QUOTIENT);

	  draw_coord_arrow(v);

	  // right coord arrow
	  v[0] = vector(xmax,0.0,0.0);
	  v[1] = vector(xmax-(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			        -(xmax-xmin)/ARROW_QUAD_QUOTIENT,
				     (xmax-xmin)/ARROW_QUAD_QUOTIENT);
	  v[2] = vector(xmax-(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			         (xmax-xmin)/ARROW_QUAD_QUOTIENT,
				     (xmax-xmin)/ARROW_QUAD_QUOTIENT);
	  v[3] = vector(xmax-(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			         (xmax-xmin)/ARROW_QUAD_QUOTIENT,
				     -(xmax-xmin)/ARROW_QUAD_QUOTIENT);
	  v[4] = vector(xmax-(xmax-xmin)/ARROW_DEPTH_QUOTIENT,
			         -(xmax-xmin)/ARROW_QUAD_QUOTIENT,
				     -(xmax-xmin)/ARROW_QUAD_QUOTIENT);

	  draw_coord_arrow(v);

	  // front coord arrow
	  v[0] = vector(0.0,0.0,zmax);
	  v[1] = vector( (zmax-zmin)/ARROW_QUAD_QUOTIENT,
			        -(zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmax-(zmax-zmin)/ARROW_DEPTH_QUOTIENT);
	  v[2] = vector( (zmax-zmin)/ARROW_QUAD_QUOTIENT,
			         (zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmax-(zmax-zmin)/ARROW_DEPTH_QUOTIENT);
	  v[3] = vector(-(zmax-zmin)/ARROW_QUAD_QUOTIENT,
			         (zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmax-(zmax-zmin)/ARROW_DEPTH_QUOTIENT);
	  v[4] = vector(-(zmax-zmin)/ARROW_QUAD_QUOTIENT,
			        -(zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmax-(zmax-zmin)/ARROW_DEPTH_QUOTIENT);

	  draw_coord_arrow(v);

	  // back coord arrow
	  v[0] = vector(0.0,0.0,zmin);
	  v[1] = vector( (zmax-zmin)/ARROW_QUAD_QUOTIENT,
			        -(zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmin+(zmax-zmin)/ARROW_DEPTH_QUOTIENT);
	  v[2] = vector( (zmax-zmin)/ARROW_QUAD_QUOTIENT,
			         (zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmin+(zmax-zmin)/ARROW_DEPTH_QUOTIENT);
	  v[3] = vector(-(zmax-zmin)/ARROW_QUAD_QUOTIENT,
			         (zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmin+(zmax-zmin)/ARROW_DEPTH_QUOTIENT);
	  v[4] = vector(-(zmax-zmin)/ARROW_QUAD_QUOTIENT,
			        -(zmax-zmin)/ARROW_QUAD_QUOTIENT,
				     zmin+(zmax-zmin)/ARROW_DEPTH_QUOTIENT);

	  draw_coord_arrow(v);
	  
	  // bottom coord arrow
	  v[0] = vector(0.0,ymin,0.0);
	  v[1] = vector(-(ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymin+(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					 (ymax-ymin)/ARROW_QUAD_QUOTIENT);
	  v[2] = vector(-(ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymin+(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					-(ymax-ymin)/ARROW_QUAD_QUOTIENT);	  
	  v[3] = vector( (ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymin+(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					-(ymax-ymin)/ARROW_QUAD_QUOTIENT);	  
	  v[4] = vector( (ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymin+(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					 (ymax-ymin)/ARROW_QUAD_QUOTIENT);
	  
	  draw_coord_arrow(v);

	  // top coord arrow
	  v[0] = vector(0.0,ymax,0.0);
	  v[1] = vector(-(ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymax-(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					 (ymax-ymin)/ARROW_QUAD_QUOTIENT);
	  v[2] = vector(-(ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymax-(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					-(ymax-ymin)/ARROW_QUAD_QUOTIENT);	  
	  v[3] = vector( (ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymax-(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					-(ymax-ymin)/ARROW_QUAD_QUOTIENT);	  
	  v[4] = vector( (ymax-ymin)/ARROW_QUAD_QUOTIENT,
		              ymax-(ymax-ymin)/ARROW_DEPTH_QUOTIENT,
					 (ymax-ymin)/ARROW_QUAD_QUOTIENT);
	  
	  draw_coord_arrow(v);
	  
  }    
   
  glFinish();

  gl_win->gl_finish();    
}


void d3_gl_window::draw() 
{       
  // case 1 : gl_window is sub-window -> resize it if necessary  
  if(!real_gl){    
	if(second_win->width() != width || second_win->height() != height){		
		width  = second_win->width();
		height = second_win->height();

		gl_win->resize(0,second_win->get_panel_height(),second_win->width(),
			second_win->height() - second_win->get_panel_height() );

		BASE_WINDOW* SW = second_win->get_status_window();
		
		if(SW && !SW->is_closed()){
			close_status_window();
			open_status_window(SW->get_bg_color(),SW->height());			
		}				

		gl_resize();		
	}
  }
  // case 2 : gl_window is main-window -> resize panel-win if it exists and if it is necessary
  else{ 	  
	if(gl_win->width() != width || gl_win->height() != height){				
		width  = gl_win->width();
		height = gl_win->height();	     				
		if(second_win)
			second_win->resize(0,0,width,second_win->get_panel_height());

		BASE_WINDOW* SW = gl_win->get_status_window();
		
		if(SW && !SW->is_closed()){
			close_status_window();
			open_status_window(SW->get_bg_color(),SW->height());			
		}
		
		gl_resize();		
	}	
  }  
  
  d3_gl_window::draw_scenes(); 
}


void d3_gl_window::reg_tex(GLuint* t_num, char* t_data, int width, int height,
						   const GLint& filter)
{
	gl_win->gl_start();

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);	
	
	glGenTextures(1, t_num);	

	glBindTexture(GL_TEXTURE_2D, *t_num);   	
    glTexImage2D(GL_TEXTURE_2D, 0, 3, width,
	height, 0, GL_RGB, GL_UNSIGNED_BYTE,
	(GLubyte*)t_data);
	
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter);  
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter);	
	  
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP); 
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP); 
     
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

	tex_info_list.push(tex_info(t_data,width,height,filter));	
	tex_num_list.push(t_num);	

	gl_win->gl_finish();	
}

void d3_gl_window::print_tex_nums()
{
	GLuint* t_num;
	forall(t_num,tex_num_list)
		cout<<*t_num<<"  ";
}

void d3_gl_window::unreg_tex(GLuint* t_num)
{
	gl_win->gl_start();

	list_item num  = tex_num_list.first();
	list_item info = tex_info_list.first();
	
	while(num != nil){
		if(tex_num_list[num] == t_num){			
			tex_num_list.del(num);
			tex_info_list.del(info);

			gl_win->gl_finish();
			return;
		}

		num  = tex_num_list.succ(num);		
		info = tex_info_list.succ(info);
	}

	gl_win->gl_finish();
}

void d3_gl_window::re_reg_all_textures()
{
	gl_win->gl_start();

	list_item num  = tex_num_list.first();
	list_item info = tex_info_list.first();		

	while(num != nil){
		GLuint* t_num  = tex_num_list[num];

		tex_info t_info = tex_info_list[info];
		char*   t_data = t_info.first();
		int     t_w    = t_info.second();
		int     t_h    = t_info.third();		
		GLint   t_filter = t_info.fourth();

		num  = tex_num_list.succ(num);
		info = tex_info_list.succ(info);				
		
		unreg_tex(t_num);
		reg_tex(t_num,t_data,t_w,t_h,t_filter);		
	}

	gl_win->gl_finish();
}


void d3_gl_window::set_tex_filter(GLuint* t_num, const GLint& filter)
{
	list_item num  = tex_num_list.first();
	list_item info = tex_info_list.first();		

	while(num != nil){
		if(*tex_num_list[num] == *t_num)
			break;

		num  = tex_num_list.succ(num);
		info = tex_info_list.succ(info);
	}

	if(num == nil) return;

	gl_win->gl_start();
	
	glBindTexture(GL_TEXTURE_2D,*t_num);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, filter);  
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, filter);

	gl_win->gl_finish();
}

GLint d3_gl_window::get_tex_filter(GLuint* t_num)
{
	list_item num  = tex_num_list.first();
	list_item info = tex_info_list.first();		

	while(num != nil){
		if(*tex_num_list[num] == *t_num)
			return tex_info_list[info].fourth();

		num  = tex_num_list.succ(num);
		info = tex_info_list.succ(info);
	}

	return -1;
}



// event-functions 

#define BUFSIZE 512

d3_gl_scene* d3_gl_window::pick_object(const double& x, const double& y)
{
	unsigned int selectBuf[BUFSIZE];
	int hits;	
	int viewport[4]; // view port
	
	double x2 = gl_win->real_to_pix(x);
	double y2 = gl_win->real_to_pix(y);
	
	gl_win->gl_start();
	glGetIntegerv(GL_VIEWPORT, viewport);

	glSelectBuffer(BUFSIZE, selectBuf);
	glRenderMode(GL_SELECT);

	glInitNames();
	glPushName(0);	
	
	if(viewport[3] == 0) viewport[3] = 1;

	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();  
	gluPickMatrix(x2, y2, 5.0, 5.0, viewport);
    gluPerspective(50.0f, ((GLfloat)viewport[2])/((GLfloat)viewport[3]), 1.0f, 10000000000.0f);	

	glMatrixMode(GL_MODELVIEW);							
	glLoadIdentity();
	(*cam)();
  
	d3_gl_scene* sc_ptr;
    unsigned int index = 1;
    forall(sc_ptr,scenes){
	    if(sc_ptr->get_visible()){
	    	  glLoadName(index);
		      ++index;
		      sc_ptr->draw(); 
		}
	}	
	
	hits = glRenderMode(GL_RENDER);

	gl_win->gl_finish();
	gl_resize();

	// process hits
	if(hits){ // always take first hit
		unsigned int* ptr = selectBuf;
		++ptr; ++ptr; ++ptr;
		index = 1;		

		forall(sc_ptr,scenes){
			if(!sc_ptr->get_visible()) continue;
			            
			if(index == *ptr){	
				return sc_ptr;				
			}
			else
				++index;
		}		
	}

	return NULL;
}

int d3_gl_window::move(int& val, double& x, double& y) 
{    
  window* sub_win;
  BASE_WINDOW* status_win;

  if(real_gl)
  {
	  if(second_win==NULL)
		  sub_win=gl_win;
	  else
		  sub_win=second_win;

	  status_win = gl_win->get_status_window();
  }
  else{
	  sub_win = second_win;
	  status_win = second_win->get_status_window();	  
  }

  BASE_WINDOW* w;
       
  int e = BASE_WINDOW::get_event(w,val,x,y);  

  if(w){
	if(w == gl_win || w == status_win || w == sub_win)	
		return e;		
	else
		return no_event; 
  }

  return no_event;
}

int d3_gl_window::get_mouse(double& x, double& y)
{
  int val;
  if(move(val,x,y)==button_press_event)
 	  return val;
  else
	  return NO_BUTTON;
}

int d3_gl_window::read_mouse(double& x, double& y)
{
  int but;  
  do but = d3_gl_window::get_mouse(x,y); while (but == NO_BUTTON);  
  return but;
}

int d3_gl_window::get_event(int& val, double& x, double& y)
{ return move(val,x,y); }

int d3_gl_window::read_event(int& val, double& x, double& y)
{
  int event;  
  do event = d3_gl_window::get_event(val,x,y); while (event == no_event);  
  return event;
}

BASE_WINDOW* d3_gl_window::create_status_window(color bc, int h)
{
	if(!real_gl){ 		
		if(second_win->get_status_window() && !second_win->get_status_window()->is_closed()){			
			gl_win->resize(0,second_win->get_panel_height(),second_win->width(),
			second_win->height() - second_win->get_panel_height() - h);			
		}
		
		gl_resize();			

		return second_win->create_status_window(bc,h);
	}
	else{
		return gl_win->create_status_window(bc,h);
	}
}

BASE_WINDOW* d3_gl_window::open_status_window(color bc, int h)
{
	if(!real_gl){ 	
		return second_win->open_status_window(bc,h);
	}
	else{
		return gl_win->open_status_window(bc,h);		
	}
}

void d3_gl_window::close_status_window()
{
	if(!real_gl){
		second_win->close_status_window();				
	}
	else gl_win->close_status_window();
}

void d3_gl_window::destroy_status_window()
{
	close_status_window();

	if(!real_gl){		
		second_win->destroy_status_window();
	}
	else gl_win->destroy_status_window();
}

void d3_gl_window::set_status_string(const char* str)
{ if(!real_gl) second_win->set_status_string(str);
  else gl_win->set_status_string(str);       
 }

const char* d3_gl_window::get_status_string()
{ if(!real_gl) return second_win->get_status_string();
  else return gl_win->get_status_string();
}

BASE_WINDOW* d3_gl_window::get_status_window()
{ if(!real_gl) return second_win->get_status_window();
  else return gl_win->get_status_window();
}


gl_window* d3_gl_window::set_menu_win(window* M)
{
	if(M == NULL) return NULL;

	if(!real_gl){ 
		gl_window* old_gl_win = gl_win;		
		
		gl_win = new gl_window(M->width(),M->height()-M->get_panel_height()); 
		gl_win->set_inf(this,0);
		gl_win->set_redraw(d3gl_redraw);
		
		M->display();
		M->resize(second_win->xpos(),second_win->ypos(),second_win->width(),
			second_win->height());			
		gl_win->display(0,M->get_panel_height(),*M);
		gl_win->resize(0,M->get_panel_height(),M->width(),M->height()-M->get_panel_height());				
		
        second_win->close();  
		second_win = M;

		delete old_gl_win;
		old_gl_win = NULL;		
		
		gl_init();
		gl_resize();		

		re_reg_all_textures();

		d3_gl_scene* sc;
		forall(sc,scenes)
			sc->re_reg_all_textures();
		
		return gl_win;
	}
	else{		
		M->display(*gl_win,0,0);
		M->resize(0,0,gl_win->width(),M->get_panel_height());

		second_win->close();
		
		second_win = M;
		
		gl_resize();

		return NULL;
	}		
}



int compare(const additional_draw& d1, const additional_draw& d2)
{ 	if(d1 == d2)
		return 0;
	else
		return -1;
}


int compare(const gl_uint_ptr& p1, const gl_uint_ptr& p2)
{ 	if(p1 == p2)
		return 0;
	else
		return -1;
}


LEDA_END_NAMESPACE
