// compression module includes
#include <LEDA/coding/MTF.h>

LEDA_BEGIN_NAMESPACE

/// MTFCoder ///////////////////////////////////////////////////////////////////////////////

MTFCoder::MTFCoder(streambuf* src_stream, streambuf* tgt_stream, bool own_streams)
: base(src_stream, tgt_stream, own_streams)
{}

MTFCoder::MTFCoder(const char* src_file_name, const char* tgt_file_name)
: base(src_file_name, tgt_file_name)
{}

#ifdef LEDA_SIMPLE_MTF
void
MTFCoder::init_table()
{
	byte i = 0;
	do {
		Table[i] = i;
	} while (++i);
}
#else
void 
MTFCoder::init_table()
{
	byte i = 0;
	do {
		CyclicSucc[i] = i+1;
	} while (++i);
	Tail = 0xFF;
}
#endif

void 
MTFCoder::init_encoding()
{
	base::init_encoding();
	
	init_table();
}

void 
MTFCoder::init_decoding()
{
	base::init_decoding();

	init_table();
}

#ifdef LEDA_SIMPLE_MTF
void 
MTFCoder::encode_stream()
{
	int c;
	while ((c = read_byte()) >= 0 && c != EOF) {
		byte symbol = (byte) c;

		byte rank = 0;

		while (Table[rank] != symbol) ++rank;

		write_byte(rank);

		while (rank) {
			Table[rank] = Table[rank-1];
			--rank;
		}
		Table[0] = symbol;
	}

	if (c == EOF) {
		encoding_finished();
	}
}

void 
MTFCoder::decode_stream()
{
	int c;
	while ((c = read_byte()) >= 0 && c != EOF) {
		byte rank = (byte) c;

		byte symbol = Table[rank];

		while (rank) {
			Table[rank] = Table[rank-1];
			--rank;
		}
		Table[0] = symbol;

		if (write_byte(symbol) < 0) return;
	}

	decoding_finished();
}
#else
void 
MTFCoder::encode_stream()
{
	byte head = CyclicSucc[Tail];

	int c;
	while ((c = read_byte()) >= 0 && c != EOF) {
		byte symbol = (byte) c;

		byte rank = 0;
		if (symbol != head) {
			byte pred_cur_sym;
			byte cur_sym = head;
			do {
				++rank;
				cur_sym = CyclicSucc[pred_cur_sym = cur_sym];
			} while (cur_sym != symbol);

			if (Tail == cur_sym) {
				Tail = pred_cur_sym;
			}
			else {
				CyclicSucc[pred_cur_sym] = CyclicSucc[cur_sym];
				CyclicSucc[cur_sym] = head;
				CyclicSucc[Tail] = cur_sym;
			}
			head = cur_sym;
		}
		write_byte(rank);
	}

	if (c == EOF) {
		encoding_finished();
	}
}

void 
MTFCoder::decode_stream()
{
	byte head = CyclicSucc[Tail];

	int c;
	while ((c = read_byte()) != EOF) {
		byte rank = (byte) c;

		byte cur_sym;
		if (rank == 0) cur_sym = head;
		else {
			byte pred_cur_sym;
			cur_sym = head;
			do {
				cur_sym = CyclicSucc[pred_cur_sym = cur_sym];
			} while (--rank);

			if (Tail == cur_sym) {
				Tail = pred_cur_sym;
			}
			else {
				CyclicSucc[pred_cur_sym] = CyclicSucc[cur_sym];
				CyclicSucc[cur_sym] = head;
				CyclicSucc[Tail] = cur_sym;
			}
			head = cur_sym;
		}

		if (write_byte(cur_sym) < 0) return;
	}

	decoding_finished();
}
#endif

LEDA_END_NAMESPACE
