
#include <LEDA/coding/suffix_array.h>
#include <LEDA/system/error.h>
#include "../compress/plain_c/fm_common.h"
#include "../compress/plain_c/fm_ds_ssort.h"

#include <LEDA/internal/PREAMBLE.h>

#include <string.h>

LEDA_BEGIN_NAMESPACE

void suffix_array::construct_sa(const char* t, int tlen)
{

	len = tlen;

	// int suffix sorting algorithm
	int overshoot = init_ds_ssort(500, 2000);
	if (overshoot == 0) { leda_assert(false,"suffix_array::construct_sa : overshoot is empty"); return; }
	// allocate space for text and for overshoot
	text       = new unsigned char[tlen + overshoot];
	
	if(text==0) // this should be done by try - catch!
		{leda_assert(false, "suffix_array::consruct_sa : memory allocation failed"); return;}

	// allocate space for suffix array
	sa  = new int[tlen];
	if(sa==0) // this should be done by try - catch!
		{leda_assert(false, "suffix_array::consruct_sa : memory allocation failed"); return;}


	// copy t
	memcpy((void*) text,(const void*) t, tlen);
	// constrcuct suffix array
	ds_ssort(text, sa, tlen);
	sa_computed = true;
}

suffix_array::suffix_array()
: text(0),sa(0),len(0),sa_computed(false), lcp_computed(false)
{}

suffix_array::suffix_array(istream& in)
 : text(0), sa(0), LCP(0),sa_computed(false), lcp_computed(false)
{
	string tmp;
	tmp.read_file(in);
	construct_sa(tmp,tmp.length());
}

suffix_array::suffix_array(const suffix_array& S)
{
/*
  sn: this is a (copy) constructor
      sa_computed and lcp_computed are still undefined here.
*/

/*
	if(sa_computed) {delete[] sa; delete[] text;}
	if(lcp_computed) delete[] LCP;
*/

	len = S.len;

	if(S.sa_computed)
	{	
		sa_computed = true;
		sa   = new int[len];
		text = new unsigned char[strlen((const char*)S.text)];
		memcpy((void*)sa,(const void*)S.sa,len*sizeof(int));
		memcpy((void*)text,(const void*) S.text, strlen((const char*)S.text));
	}
	else
	{
		sa_computed = false;
		sa = 0;
	}
	if(S.lcp_computed)
	{
		lcp_computed = true;
		LCP = new int[len];
		memcpy((void*)LCP,(const void*)S.LCP,S.len*sizeof(int));
	}
	else
	{
		lcp_computed = false;
		LCP=0;
	}
}

suffix_array& 
suffix_array::operator =(const suffix_array& S)
{
	if(this == &S) return *this;
	else
	{
		if(sa_computed) {delete[] sa; delete[] text;}
		if(lcp_computed) delete[] LCP;

		len = S.len;

		if(S.sa_computed)
		{	
			sa_computed = true;
			sa   = new int[len];
			text = new unsigned char[strlen((const char*)S.text)];
			memcpy((void*)sa,(const void*)S.sa,len*sizeof(int));
			memcpy((void*)text,(const void*) S.text, strlen((const char*) S.text));
		}
		else
		{
			sa_computed = false;
			sa = 0;
		}
		if(S.lcp_computed)
		{
			lcp_computed = true;
			LCP = new int[len];
			memcpy((void*)LCP,(const void*)S.LCP,S.len*sizeof(int));
		}
		else
		{
			lcp_computed = false;
			LCP=0;
		}
		return *this;
	}
}

suffix_array::suffix_array(const char* t)
: text(0), sa(0), LCP(0),sa_computed(false), lcp_computed(false)
{
	int tlen = (int)strlen( t );
	construct_sa(t,tlen);
	
}

suffix_array::suffix_array(string t)
: text(0), sa(0), LCP(0),sa_computed(false), lcp_computed(false)
{
	construct_sa(t,t.length());
}

suffix_array::~suffix_array()
{
	if(text) delete[] text;
	if(sa_computed) delete[] sa;
	if(lcp_computed) delete[] LCP;
}

void
suffix_array::clear()
{
	if(sa_computed) {delete[] sa; delete[]text; sa_computed=false; sa=0;text=0;}
	if(lcp_computed){delete[] LCP; LCP=0; lcp_computed=false;}
	len=0;
}

int 
suffix_array::operator [](int i) const
{
	
	leda_assert(i>=0 && i<len,"suffix_array::operator[](int i):index out of bound");

	return sa[i];
	
}

unsigned char
suffix_array::symbol(int i) const
{
	leda_assert(i>=0 && i<len,"suffix_array::operator[](int i):index out of bound");

	return text[i];
}

void suffix_array::compute_lcp()
{
	if(!lcp_computed)
	{
		LCP = new int[len];

		lcp9();
		lcp_computed = true;
	}
}

void suffix_array::lcp13()
{
	int *rank = new int[len];
	
	int i;
	for(i=0;i<len;i++) rank[sa[i]]=i;
	int h=0;

	for(i=0;i<len;i++)
	{
		int k = rank[i];
		if(k==0) LCP[k]=-1;
		else
		{
			int j = sa[k-1];
			while(i+h <len && j+h < len && text[i+h]==text[j+h]) h++;
			LCP[k] = h;
		}
		if(h>0) h--;
	}


	delete[] rank;

	
}

void suffix_array::lcp9()
{
	// remove this later!!!
	if(LCP != 0) delete[] LCP;
	LCP = new int[len];

	int count[256];
	int i;
	for(i=0;i<256;i++) count[i]=0;
	//compute alphabet count for text
	for(i=0;i<len;i++) count[text[i]]++;
	// prefix sum
	
	int sum;
	for (i = 0, sum = 0;  i <256;  i++) 
	{
		int tmp = count[i];  
		count[i] = sum;  
		sum += tmp; 
  }

	int eos_pos=-1;
	// compute SA2RANKNEXT
	int j = count[text[len-1]]++;
	LCP[j] = 0;
	for(i=0;i<len;i++)
	{
		if(sa[i]==0) eos_pos = i;
		else
		{
			int c = text[sa[i]-1];
			j = count[c]++;
			LCP[j] = i;
		}
	}

	int h=0;
	int k = eos_pos;
	for(i=0;i<len;i++)
	{
		int nextk = LCP[k];
		if(k==0) LCP[k]=-1;
		else
		{
			int j = sa[k-1];
			while(i+h < len && j+h < len && text[i+h] == text[j+h]) h++;
			LCP[k] = h;
		}
		if(h>0) h--;
		k = nextk;
	}

	
}

int suffix_array::lcp(int i) const
{
	leda_assert(LCP !=0, "suffix_array::lcp(int i): lcp array was not computed");
	leda_assert(i>=0 && i<len,"suffix_array::lcp(int i):index out of bound");
	return LCP[i];
}

int suffix_array::lcp(int i, int j) const
{
	leda_assert(i>=0 && i<len,"suffix_array::lcp(int i):index out of bound");
	leda_assert(j>=0 && i<len,"suffix_array::lcp(int j):index out of bound");
	int h=0;

	while(i+h <len && j+h < len && text[i+h]==text[j+h]) h++;
	return h;
}
	
int suffix_array::search(const string& p, smode mode) const
{
  int l = 0;
  int r = len-1;

  int len_patt = p.length();
  
	
  while (l <= r)
  { 
    int m = (l+r)/2;
		
    int pos_suf = sa[m];
	  int h=0;
    while ((pos_suf+h < len) && (h < len_patt) && (text[pos_suf+h] ==  (unsigned char) p[h])) h++; 

    if (h == len_patt)
		{ 
			if(mode == larger) {l = m+1;}
			else{ r = m-1; }
		}
    else 
		{
			
			if ((((unsigned char) p[h]) - text[pos_suf+h]) > 0) { l = m+1; } else { r = m-1; } 
		}
  }
  return l;  
}

int suffix_array::num_occurrences(const string& p) const
{
	return (search(p,larger) - search(p,smaller));
}

list<int>
suffix_array::occurrences(const string &p) const
{
	int l = search(p,smaller);
	int r = search(p,larger);

	list<int> L;
	for(int i=l; i<r;i++) L.append(sa[i]);
	return L;
}

string
suffix_array::suffix(int i) const
{
	leda_assert(i>=0 && i<len,"suffix_array::suffix(int i):index out of bound");
	
	char* tmp= new char[len-i+1];
	tmp[len-i]='\0';
	for(int j=0; j<len-i;j++) tmp[j]= text[i+j];
	string l(tmp);
	delete[] tmp;
	return l;
}

void
suffix_array::build(const string& t)
{
	clear();
	construct_sa(t,t.length());
}
	


void suffix_array::print() const
{
	for(int i=0;i<len;i++)
	{
		cout << sa[i]<< " ";
	}
	cout << endl;
}

void suffix_array::print_lcp() const
{
	if(lcp_computed)
	{
		for(int i=0;i<len;i++)
		{
			cout << LCP[i]<< " ";
		}
		cout << endl;
	}
}
LEDA_END_NAMESPACE


