/*******************************************************************************
+
+  LEDA 5.0  
+
+
+  simple_coder.h
+
+
+  Copyright (c) 1995-2004
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/

#ifndef _SIMPLE_CODER_H
#define _SIMPLE_CODER_H

/*
	This example shows how users can construct their own coders and 
	how to integrate them into LEDA such that they work together seamlessly
	with the AutoDecoder from LEDA.
	(More documentation can be found in the c-file.)
 */

#include <LEDA/coding/coder_base.h>

#if defined(LEDA_STD_IO_HEADERS) && !defined(streampos)
using std::streampos;
using std::streamoff;
#endif

class simple_coder : public leda::coder_base {
public:
	typedef leda::coder_base base;

	// every coder has its own unique ID (so that it can be identified by the AutoDecoder)
	static const id_type ID = 0xFF004711; // 0x00xxxxxx is reserved for LEDA

	enum { DefaultByteToAdd = 1 };

public:
	// every user-defined coder should provide the following constructors
	// (it must provide a standard constructor)
	simple_coder(std::streambuf* src_stream = 0, std::streambuf* tgt_stream = 0);
	simple_coder(const char* src_file_name, const char* tgt_file_name);

	// this is just demo:
	uint32 get_byte_count() const { return ByteCount; }

	// accessors for the "header" data
	// (these are coding paramters which remain constant during coding, 
	// e.g. block size)
	char get_char_to_add() const { return ByteToAdd; }
	void set_char_to_add(char c) { ByteToAdd = c; }

protected:
	// the following functions should be implemented by every user-defined coder:
	virtual const char* get_class_name() const { return "simple_coder"; }
	virtual id_type get_id() const { return ID; }

	virtual void init_encoding();
	virtual void encode_stream();

	virtual void init_decoding();
	virtual void decode_stream();

	// only for a user-defined coder with "header" data:
	virtual void do_reset(reset_type t);
	virtual void write_header_to_tgt_stream();
	virtual void read_header_from_src_stream();

	// implement this to support fast seeks
	// (of course, not every user-defined can do this)
	virtual streampos decode_seek(streampos current_pos, streampos new_pos);

public:
	// make the framework aware that we have fast seeks
	virtual bool is_optimizing_seeks() const { return true; }

private:
	uint32 ByteCount; // for the demo
	byte   ByteToAdd; // this will be the "header" data
};

