#include <LEDA/graphics/window.h>
#include <LEDA/system/file.h>
using std::cin;
using std::cout;
using std::cerr;
using std::endl;
using std::flush;

#if !defined(__unix__)

int main() { 
  cerr << endl;
  cerr << "This program works only for Unix." << endl;
  cerr << endl;
  return 0; 
}

#else

#include <unistd.h>
#include <signal.h>


using namespace leda;


static window* wp = 0;
static string text;
static string cursor;
static string prompt;

static double line_height = 0;
static double text_height = 0;
static double scroll_top = 0;

int child_pid = 0;

void start_child_process(string prog, int fd_out, int fd_in)
{
  // start program in backround and redirect its std output to fd_out 
  // and its std input from fd_in (using bash command line syntax)

  string f_pid("%d.child",getpid());

  char cmd[256];
  sprintf(cmd,"%s >&%d 2>&1 <&%d & echo $! > %s", ~prog, fd_out,fd_in,~f_pid);

  cout << endl;
  cout << cmd << endl;

  system(cmd);

  if (is_file(f_pid))
  { ifstream ifstr(f_pid);
    ifstr >> child_pid;
    ifstr.close();
    delete_file(f_pid);
  
    cout << endl;
    cout << "child_pid = " << child_pid << endl;;
    setenv("CHILD_PID",string("%d",child_pid),1);
  
  }
}


void terminate_child_process()
{ 
  if (child_pid == 0) return; 

  cout << "terminate child process and sub-processes" << endl;

/*
  cout << "kill: pid = " << child_pid << endl;
  kill(child_pid,SIGKILL);
*/

  string cmd;

  // kill sub-processes of process with pid = child_pid
  cmd = string("/bin/pkill -9 -P %d",child_pid);
  cout << cmd << endl;
  system(cmd);

  // kill process with pid = child_pid
  cmd = string("/bin/kill -9 %d",child_pid);
  cout << cmd << endl;
  system(cmd);

  exit(0);
}


void sig_handler(int s) {
  cout << endl;
  cout << string("SIGNAL (%d): %s",s,sys_siglist[s]) << endl;
  terminate_child_process();
  exit(0);
}


static void print_text(bool update_sb)
{ window& W = *wp;

  // line_height must be updated - why ?
  line_height = W.text_height("H");

  if (update_sb) 
  { 
    text_height = 0;

    string s;
    forall_lines(s,text) text_height += W.text_height(s);

    double display_height = W.ymax() - W.ymin() - line_height;

    if (scroll_top < text_height - display_height) {
      scroll_top = text_height - display_height;
    }

    double sb_pos = scroll_top/text_height;
    double sb_size = display_height/text_height;
    if (sb_size > 1.0) sb_size = 1.0;
    W.set_scrollbar_pos(sb_pos,sb_size);
  }


  double x = W.xmin() + W.pix_to_real(5);
  double y = W.ymax() + scroll_top - W.pix_to_real(5);

  string txt = text + cursor;

  W.start_buffering();
  W.clear();

  string s;
  forall_lines(s,txt) 
  { if (y <= W.ymax() && y >= W.ymin()) W.draw_text(x,y,s.expand(8));
    y -= line_height;
  }

  W.flush_buffer();
  W.stop_buffering();
}
     


void scroll_up(int i)
{ //if (scroll_top < 0.001) return;
  if (scroll_top < 0.1) return;
  scroll_top -= line_height;
  wp->set_scrollbar_pos(scroll_top/text_height);
  print_text(false);
}

void scroll_down(int i)
{ if (scroll_top > (text_height - line_height - 0.001)) return;
  scroll_top += line_height;
  wp->set_scrollbar_pos(scroll_top/text_height);
  print_text(false);
}

void scroll_drag(int i)
{ if (i < 0) return;
  scroll_top = (i/1000.0)*(text_height - line_height);
  if (scroll_top < 0) scroll_top = 0;
  print_text(false);
}


void redraw(window* wp) {
   print_text(true);
}

void delete_handler(window* wp) {
  terminate_child_process();
}


void handle_controls(string& s)
{
/*
  string clear_ctrl1 = "\e[3J\e[H\e[2J";
  string clear_ctrl2 = "\e[H\e[2J";
  string label_ctrl  = "\e]2;";
*/

  string clear_ctrl1 = "\033[3J\033[H\033[2J";
  string clear_ctrl2 = "\033[H\033[2J";
  string label_ctrl  = "\033]2;";

  if (s.index(clear_ctrl1) != -1) 
  { s = s.replace_all(clear_ctrl1,"");
    s = "";
    text = "";
    text_height = 0;
    scroll_top = text_height;
   }

  if (s.index(clear_ctrl2) != -1) 
  { s = s.replace_all(clear_ctrl2,"");
    s = "";
    text = "";
    text_height = 0;
    scroll_top = text_height;
   }

  int p = s.index(label_ctrl); 
  if (p != -1) {
    int q = s.index("\a",p); // ^G
    if (q != -1) {
      string label = s(p+label_ctrl.length(),q-1);
      s = s.del(p,q);
      wp->set_frame_label(label);
    }
  }

}



bool fd_poll(int fd, int msec)
{ timeval polltime;
  polltime.tv_sec  = msec / 1000;
  polltime.tv_usec = 1000 * (msec % 1000);

  fd_set rdset;
  FD_ZERO(&rdset);
  FD_SET(fd,&rdset);
  return select(fd+1,&rdset,0,0,&polltime) > 0;
}


int fd_select(int fd1, int fd2)
{ 
/*
  timeval polltime;
  polltime.tv_sec  = msec / 1000;
  polltime.tv_usec = 1000 * (msec % 1000);
*/

  fd_set rdset;
  FD_ZERO(&rdset);
  FD_SET(fd1,&rdset);
  FD_SET(fd2,&rdset);
  int fd = fd1;
  if (fd2 > fd) fd = fd2;
  select(fd+1,&rdset,0,0,0);

  if (FD_ISSET(fd1,&rdset)) return fd1;
  if (FD_ISSET(fd2,&rdset)) return fd2;
  return -1;
}




int main(int argc, char** argv) 
{
  signal(SIGINT,sig_handler);
  signal(SIGTERM,sig_handler);
  signal(SIGQUIT,sig_handler);
  signal(SIGKILL,sig_handler);

  int dpi = window::screen_dpi();

  string title = "LEDA Terminal";

  string prog;

  if (argc < 2){ 
  //prog = "bash -i";
  //prog = "csh -i";
  //prog = "su -c 'csh -i' naeher";
  //prog = "su naeher";
    prog = "su -c 'tcsh -i' naeher";
    text = "";
  }
  else
  { prog = argv[1];
    text = "\n" + prog + "\n";
/*
    title += "   ";
    title += prog.replace("terminal/","");
*/
   }

/*
  if (prog == "bash") prompt = "bash: ";
  if (prog == "csh")  prompt = "csh: ";
*/


  
  window W(title);
  wp = &W;

/*
  menu M;
  M.button("Button 1");
  M.button("Button 2");
  M.button("Settings");
  M.separator();
  M.button("Exit",999);

  W.button("Menu", M);
  W.button("done",999);
  W.make_menu_bar(1);
*/

  //W.set_fixed_font();

  int font_sz = dpi/5;

  cout << window::screen_width()/font_sz << endl;

  if (50*font_sz > window::screen_width()) { 
     font_sz = window::screen_width()/50;
  }

  cout << "font_sz = " << font_sz << endl;


  W.set_font(string("F%d",font_sz));

  W.set_window_delete_handler(delete_handler);
  W.set_redraw(redraw);

  W.display();

  W.open_scrollbar(scroll_up, scroll_down, scroll_drag,0,0);

  int fd_disp = W.display_fd();

  cout << "display: fd = " << fd_disp << endl;

  int width = W.real_to_pix(80.75 * W.text_width("H"));
  int height = W.real_to_pix(32.75 * W.text_height("H"));

  width += W.scrollbar_width();

  cout << "width  = " << width << endl;
  cout << "height = " << height << endl;

  int xpos = (window::screen_width() - width)/2;
  int ypos = (window::screen_height()- height)/2;

  W.resize(xpos,ypos,width,height);

  line_height = W.text_height("H");

  int output_pipe[2];

  if (pipe(output_pipe) < 0) {
    cerr << "Could not create output pipe." << endl;
    return 1;
  }

  int input_pipe[2];

  if (pipe(input_pipe) < 0) {
    cerr << "Could not create input pipe." << endl;
    return 1;
  }

  // start program in backround and redirect its output from stdout 
  // to one end (1) of the output pipe and its input from stdin to 
  // one end (0) of the input pipe  (using bash command line syntax)

  start_child_process(prog,output_pipe[1],input_pipe[0]);


  string line = "";

  for(;;) {

   char buf[1024];

   int fd_read = output_pipe[0]; 
   int fd_write = input_pipe[1]; 

   int fd = fd_select(fd_read,fd_disp);
 //cout << "fd = " << fd << endl;

   if (fd == -1) {
     cout << "select: fd = " << fd << endl;
     continue;
   }


   if (fd == fd_read) 
   { // read output pipe of program
     int sz = read(fd_read, buf, 1024);
     buf[sz] = '\0';
     string s = buf;
     handle_controls(s);
     text += s.replace_all("\e","^[");
     cursor = "_";
     print_text(true);
     continue;
    }


   // handle window events

   int val = 0;
   double x,y;
   unsigned long t;
   window* wp;

 //int e = W.read_event(val,x,y,t);

   int e = read_event(wp,val,x,y,t,0);
   
   //cout << event_name[e] << endl;

   if (e == display_event && val == 0) break; 

   if (wp != &W) continue;

   if (e == button_press_event) 
   { //cout << "button pressed: " << val << endl;

     if (val == CLOSE_BUTTON) break;

     if (val == 999) {
       //if (W.confirm("Exit LEDA Terminal")) break;
       break;
     }

     if (val == MOUSE_BUTTON(4)) scroll_up(0);
     if (val == MOUSE_BUTTON(5)) scroll_down(0);

     continue;
   }


   if (e == key_press_event) 
   { //cout << "key pressed: " << val << endl;

     if (val == KEY_ESCAPE) break;

     if (val == KEY_UP) {
        scroll_up(0);
        continue;
     }

     if (val == KEY_DOWN) {
       scroll_down(0);
       continue;
     }

     if (isprint(val))
     { line += char(val);
       if (text != "Password: ") text += char(val);
      }
     else
     { if (val == KEY_BACKSPACE && line != "")
       { line = line.head(-1);
         if (text != "Password: ") text = text.head(-1);
        }
      }
  
     print_text(true);
  
     if (val == KEY_RETURN) 
     { cursor = "";
       line += "\n";
       text += "\n";

       print_text(true);
    
       // write line to input pipe of prog
       write(fd_write,line,line.length());
    
       if (line == "exit\n") {
          leda_wait(1);
          break;
       }
    
       line = "";
     }
  
     continue;
   }

  }

  cout << "TERMINAL FINISHED" << endl;
  terminate_child_process();

  return 0;
}

#endif
