#define _WIN32_WINNT 0x0400

#include <windows.h>
#include <shlobj.h>
#include <stdio.h>


HRESULT CreateLink(const char* obj_path,  const char* link_path, 
                                          const char* wdir_path)
{ 
  const char* descript = "LEDA Shortcut";

  HRESULT hres; 
  void*   ptr; 

  // Get a pointer to the IShellLink interface. 
  hres = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, 
                                                 IID_IShellLink, &ptr); 

  IShellLink* psl = (IShellLink*)ptr;

  if (SUCCEEDED(hres)) 
  { 
    // Set path, working directory, and description 
    psl->SetPath(obj_path); 
    psl->SetWorkingDirectory(wdir_path); 
    psl->SetDescription(descript); 

    // Query IShellLink for the IPersistFile interface for saving the 
    // shortcut in persistent storage. 
    hres = psl->QueryInterface(IID_IPersistFile,&ptr); 

    IPersistFile* ppf = (IPersistFile*)ptr;

    if (SUCCEEDED(hres))
    { //WORD wsz[MAX_PATH]; 
      wchar_t wsz[MAX_PATH]; 

      // Ensure that the string is ANSI. 
      MultiByteToWideChar(CP_ACP, 0, link_path, -1, wsz, MAX_PATH); 

      // Save the link by calling IPersistFile::Save. 
      hres = ppf->Save(wsz, TRUE); 
      ppf->Release(); 
     } 

    psl->Release(); 
  } 

  return hres; 
}



void clean_dir(const char* dirname)
{
  DWORD att = GetFileAttributes(dirname);
  if (att == 0xFFFFFFFF || !(att & FILE_ATTRIBUTE_DIRECTORY)) return;
  SetCurrentDirectory(dirname);

  WIN32_FIND_DATA fd;
  HANDLE ha = FindFirstFile("*",&fd);
  if (ha && ha != (HANDLE)0xffffffff)
  { do { DeleteFile(fd.cFileName);
    } while (FindNextFile(ha,&fd));
    FindClose(ha);
   }
}




const int argc = 3;
char* argv[] = { "xlman", "graphwin", "logo" };


DWORD WINAPI CopyProgress(LARGE_INTEGER total_sz,
                          LARGE_INTEGER total_bytes_transferred,
                          LARGE_INTEGER stream_sz,
                          LARGE_INTEGER stream_bytes_transferred,
                          DWORD stream_num,
                          DWORD callback_reason,
                          HANDLE source,
                          HANDLE destination,
                          LPVOID data)
{

  printf("bytes = %d\n",total_bytes_transferred);
  return 0;
}


main()
{
  const char* leda_path = "D:\\LEDA\\BETA\\win32\\bcc";
  const char* xlman_path = "D:\\LEDA\\BETA\\win32\\bcc\\demo\\xlman";


  // copy leda.dll to system directory

  char dll_path[256];
  GetSystemDirectory(dll_path,256);
  sprintf(dll_path+strlen(dll_path),"\\LEDA.DLL");
  SetCurrentDirectory(leda_path);

  BOOL cancel = FALSE;

  CopyFileEx("LEDA.DLL",dll_path,CopyProgress,NULL,&cancel,
                                                       COPY_FILE_RESTARTABLE);


  unsigned long sz = 256;
  char prog_path[256];

  HKEY hKey;
  RegOpenKeyEx(HKEY_CURRENT_USER,
                     "Software",       NULL, KEY_QUERY_VALUE, &hKey);
  RegOpenKeyEx(hKey, "Microsoft",      NULL, KEY_QUERY_VALUE, &hKey);
  RegOpenKeyEx(hKey, "Windows",        NULL, KEY_QUERY_VALUE, &hKey);
  RegOpenKeyEx(hKey, "CurrentVersion", NULL, KEY_QUERY_VALUE, &hKey);
  RegOpenKeyEx(hKey, "Explorer",       NULL, KEY_QUERY_VALUE, &hKey);
  RegOpenKeyEx(hKey, "Shell Folders",  NULL, KEY_QUERY_VALUE, &hKey);
  RegQueryValueEx(hKey,"Programs", NULL,NULL, (unsigned char*)prog_path,&sz);

  sprintf(prog_path+strlen(prog_path),"\\LEDA");

  clean_dir(prog_path);

  CreateDirectory(prog_path,NULL);

  char buf1[256];
  char buf2[256];

  CoInitialize(NULL);
  HRESULT hres;

  for(int i=0; i<argc; i++)
  { sprintf(buf1,"%s\\%s.exe",xlman_path,argv[i]);
    sprintf(buf2,"%s\\%s.lnk",prog_path,argv[i]);
    HRESULT hres = CreateLink(buf1,buf2,leda_path);
    if (!SUCCEEDED(hres)) printf("hres = %d\n",hres);
   }

  return 0;
}

