#include <LEDA/graph.h>
#include <LEDA/point.h>
#include <LEDA/segment.h>
#include <LEDA/plane_alg.h>
#include <LEDA/subdivision.h>

#if defined(LEDA_STD_IO_HEADERS)
using namespace std;
#endif


int main()
{
  GRAPH<point,segment> G;
  node n0, n1, n2, n3, n4, n5, n6, n7, n8, n9, n10, n11;
//edge e;
  face f;
  list<edge> R;

  // graph:

  //    9--------------------7-------------------10
  //    |                    |                    |
  //    |                    |                    |
  //    |                    |                    |
  //    |        F3          |         F5         |          F4
  //    |                    5                    |
  //    |                   / \                   |
  //    |                  /   \                  |
  //    |                 /     \                 |
  //    6----------------1   F2  3----------------4
  //    |                 \     /                 |
  //    |                  \   /                  |
  //    |                   \ /                   |
  //    |                    0                    |
  //    |        F0          |         F1         |
  //    |                    |                    |
  //    |                    |                    |
  //    |                    |                    |
  //    8--------------------2-------------------11


  //  Das Programm wurde mit LEDA 3.5.2 und g++2.7.2.2 erzeugt und liefert 
  //  folgende falsche Ausgabe:

  // F[ 0](): [ 1][ 6][ 8][ 2][ 0]
  // F[ 1](): [ 2][11][ 4][ 3][ 0]
  // F[ 2](): [ 3][ 5][ 1][ 0]
  // F[ 3](): [ 5][ 7][ 9][ 6][ 1]
  // F[ 4](): [ 8][ 6][ 9][ 7][10][ 4][11][ 2]
  // F[ 5](): [ 4][10][ 7][ 5][ 3]
  // ++++++++++++++++++++++++++++++++++++++++++++


  // [0]((0,-1)) : [0]---->[1][0]---->[2][0]---->[3]
  // [1]((-1,0)) : [1]---->[6][1]---->[0][1]---->[5]
  // [2]((0,-11)) : [2]---->[11][2]---->[0][2]---->[8]
  // [3]((1,0)) : [3]---->[0][3]---->[4][3]---->[5]
  // [4]((11,0)) : [4]---->[10][4]---->[3][4]---->[11]
  // [5]((0,1)) : [5]---->[3][5]---->[7][5]---->[1]
  // [6]((-11,0)) : [6]---->[8][6]---->[1][6]---->[9]
  // [7]((0,11)) : [7]---->[9][7]---->[5][7]---->[10]
  // [8]((-11,-11)) : [8]---->[2][8]---->[6]
  // [9]((-11,11)) : [9]---->[6][9]---->[7]
  // [10]((11,11)) : [10]---->[7][10]---->[4]
  // [11]((11,-11)) : [11]---->[4][11]---->[2]
  

  // ++++++++++++++++++++++++++++++++++++++++++++
  // x: 0
  // y: 0
  // point: (0,0)
  // face: F[ 3](): [ 5][ 7][ 9][ 6][ 1]
  
  // x: 3
  // y: 3
  // point: (3,3)
  // face: F[ 4](): [ 8][ 6][ 9][ 7][10][ 4][11][ 2]
  
  // x: 100
  // y: 100
  // point: (100,100)
  // face: F[ 0](): [ 1][ 6][ 8][ 2][ 0]
  
  // x: -3
  // y: -3
  // point: (-3,-3)
  // face: F[ 3](): [ 5][ 7][ 9][ 6][ 1]
  
  // x: -3
  // y: 5
  // point: (-3,5)
  // face: F[ 4](): [ 8][ 6][ 9][ 7][10][ 4][11][ 2]
  
  // x: 5
  // y: -5
  // point: (5,-5)
  // face: F[ 5](): [ 4][10][ 7][ 5][ 3]
 


  point p0(0,-1); 
  point p1(-1,0); 
  point p2(0,-11);
  point p3(1,0);
  point p4(11,0);
  point p5(0,1);
  point p6(-11,0);
  point p7(0,11);
  point p8(-11,-11);
  point p9(-11,11);
  point p10(11,11);
  point p11(11,-11);

  n0 = G.new_node(p0);
  n1 = G.new_node(p1);
  n2 = G.new_node(p2);
  n3 = G.new_node(p3);
  n4 = G.new_node(p4);
  n5 = G.new_node(p5);
  n6 = G.new_node(p6);
  n7 = G.new_node(p7);
  n8 = G.new_node(p8);
  n9 = G.new_node(p9);
  n10 = G.new_node(p10);
  n11 = G.new_node(p11);

  G.new_edge( n0, n1, segment(p0 ,p1 ) );
  G.new_edge( n0, n2, segment(p0 ,p2 ) );
  G.new_edge( n0, n3, segment(p0 ,p3 ) );
  G.new_edge( n1, n6, segment(p1 ,p6 ) );
  G.new_edge( n1, n0, segment(p1 ,p0 ) );
  G.new_edge( n1, n5, segment(p1 ,p5 ) );
  G.new_edge( n2, n11, segment(p2 ,p11 ) );
  G.new_edge( n2, n0, segment(p2 ,p0 ) );
  G.new_edge( n2, n8, segment(p2 ,p8 ) );
  G.new_edge( n3, n0, segment(p3 ,p0 ) );
  G.new_edge( n3, n4, segment(p3 ,p4 ) );
  G.new_edge( n3, n5, segment(p3 ,p5 ) );
  G.new_edge( n4, n10, segment(p4 ,p10 ) );
  G.new_edge( n4, n3, segment(p4 ,p3 ) );
  G.new_edge( n4, n11, segment(p4 ,p11 ) );
  G.new_edge( n5, n3, segment(p5 ,p3 ) );
  G.new_edge( n5, n7, segment(p5 ,p7 ) );
  G.new_edge( n5, n1, segment(p5 ,p1 ) );
  G.new_edge( n6, n8, segment(p6 ,p8 ) );
  G.new_edge( n6, n1, segment(p6 ,p1 ) );
  G.new_edge( n6, n9, segment(p6 ,p9 ) );
  G.new_edge( n7, n9, segment(p7 ,p9 ) );
  G.new_edge( n7, n5, segment(p7 ,p5 ) );
  G.new_edge( n7, n10, segment(p7 ,p10 ) );
  G.new_edge( n8, n2, segment(p8 ,p2 ) );
  G.new_edge( n8, n6, segment(p8 ,p6 ) );
  G.new_edge( n9, n6, segment(p9 ,p6 ) );
  G.new_edge( n9, n7, segment(p9 ,p7 ) );
  G.new_edge( n10, n7, segment(p10 ,p7 ) );
  G.new_edge( n10, n4, segment(p10 ,p4 ) );
  G.new_edge( n11, n4, segment(p11 ,p4 ) );
  G.new_edge( n11, n2, segment(p11 ,p2 ) );


/*
  G.make_map();
  G.compute_faces();
*/

  subdivision<segment> S( G );

  forall_faces( f, S )
  { S.print_face(f); 
    cout << endl;
   }

  cout << "++++++++++++++++++++++++++++++++++++++++++++\n\n";
  S.print();
  cout << "\n++++++++++++++++++++++++++++++++++++++++++++\n";

  S.print_stripes();
  cout << endl;


  point p;

  while (cin >> p)
  { f = S.locate_point( p );
    cout << "Face: "; 
    S.print_face(f);
    cout << endl;
    cout << endl;
  }

  return 0;
}


