#include <math.h>
#include <LEDA/graph.h>
#include <LEDA/point.h>
#include <LEDA/segment.h>
#include <LEDA/rat_point.h>
#include <LEDA/rat_segment.h>
#include <LEDA/plane_alg.h>


using namespace leda;
using std::cout;
using std::flush;
using std::endl;




void sweep(const list<rat_segment>& L, GRAPH<rat_point,rat_segment>& G)
{
   edge e;

   G.clear();

   if (L.size() == 0) return;

   if (L.size() == 1) 
   { rat_segment s = L.head();
     node v = G.new_node(s.source());
     node w = G.new_node(s.target());
     G.new_edge(v,w,s);
     return;
   }


   list<rat_segment> L1;
   list<rat_segment> L2;

   int i = 0;

   rat_segment s;
   forall(s,L)
   { if (i%2 == 0) 
       L1.append(s);
     else
       L2.append(s);
     i++;
    }

   GRAPH<rat_point,rat_segment> G1;
   sweep(L1,G1);
   L1.clear();
   forall_edges(e,G1) L1.append(G1[e]);

   GRAPH<rat_point,rat_segment> G2;
   sweep(L2,G2);
   L2.clear();
   forall_edges(e,G2) L2.append(G2[e]);

   G.clear();

   //SWEEP_SEGMENTS_RB(L1,L2,G);
   L1.conc(L2);
   SWEEP_SEGMENTS(L1,G);
}




int main()
{ 
  int N  = read_int("N  = ");
  int k  = read_int("k  = ");

  rand_int.set_seed(1234567*N);

  for (; k <= 300; k+=10)
  {
    integer size = (1<<k)/2;

    list<rat_segment> seglist;

    GRAPH<rat_point,rat_segment> G;

    for(int i=0; i < N; i++)
    { integer x1 = integer::random(k)-size;
      integer y1 = integer::random(k)-size;
      integer w1 = 1;
      integer x2 = integer::random(k)-size;
      integer y2 = integer::random(k)-size;
      integer w2 = 1;
      rat_segment s(rat_point(x1,y1,w1),rat_point(x2,y2,w2));
      seglist.append(s);
     }
  
    cout << string("k = %2d  ",k) << flush;
    float T = used_time();
    sweep(seglist,G);
    float t = used_time(T);
    cout << string("|V|= %4d ",G.number_of_nodes());
    cout << string("|E|= %4d ",G.number_of_edges());
    cout << string("time = %6.2f sec  ",t);
    cout << endl;

    G.clear();
  
    cout << string("k = %2d  ",k) << flush;
    T = used_time();
    SWEEP_SEGMENTS(seglist,G);
    t = used_time(T);
    cout << string("|V|= %4d ",G.number_of_nodes());
    cout << string("|E|= %4d ",G.number_of_edges());
    cout << string("time = %6.2f sec  ",t);
    cout << endl;

    cout << endl;

   }
  
  return 0;
}

