#define LEDA_CHECKING_OFF


#include <LEDA/graph.h>
#include <LEDA/s_graph.h>
#include <LEDA/dimacs_graph.h>
#include <LEDA/assert.h>

#include <LEDA/node_array1.h>
#include <LEDA/edge_array1.h>

using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::ifstream;
using std::cout;
using std::cerr;
using std::flush;
using std::endl;
#endif

#define LEDA_GRAPH_ARRAY1
#include <LEDA/templates/max_flow_s1.t>



template<class graph_t>
void mfs_test(string fname, graph_t& G)
{
  ifstream in(fname);

  typename graph_t::node s=0;
  typename graph_t::node t=0;

  edge_array1<int,0,graph_t>  cap;
  //assert(cap.use_edge_data(G));

  edge_array1<int,1,graph_t>  flow;
  //assert(flow.use_edge_data(G));

  read_dimacs_mf(in,G,s,t,cap);

  cout << "MAX_FLOW    " << flush;

  max_flow<int,graph_t>  mf;

  mf(G,s,t,cap,flow);

  cout << string("time: %6.2f sec  f = %d", mf.cpu_time(), mf.flow()) << endl;

  mf.check();
}




int main(int argc, char** argv)
{
  if (argc < 2)
  { cerr << "usage: mf_test file" << endl;
    exit(1);
   }

/*
  graph G0(3,2);
  mfs_test(argv[1],G0);
*/

  s_graph<in_edges_graph,3,2> G1;
  mfs_test(argv[1],G1);

  s_graph<opposite_graph,3,2> G2;
  mfs_test(argv[1],G2);


  return 0;
}
