#define LEDA_CHECKING_OFF


#include <LEDA/graph.h>
#include <LEDA/s0_graph.h>
#include <LEDA/dimacs_graph.h>

#include <LEDA/node_slot.h>
#include <LEDA/edge_slot.h>

#define LEDA_GRAPH_ARRAY1
#include <LEDA/templates/max_flow_s5.t>

#include <LEDA/assert.h>


using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::cin;
using std::flush;
using std::endl;
#endif

int main(int argc, char** argv)
{
  typedef s_graph<opposite_graph,data_slots<3>,data_slots<2> > static_graph;

  static_graph G;
  static_graph::node s=0;
  static_graph::node t=0;

  edge_slot<int,0,static_graph>  cap;
  //assert(cap.use_edge_data(G));

  edge_slot<int,1,static_graph>  flow;
  //assert(flow.use_edge_data(G));


  cout << endl;
  cout << "reading DIMACS maxflow problem from std input ... ";
  cout << flush;

  read_dimacs_mf(cin,G,s,t,cap);
  cout << endl;

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  cout << string("%6d nodes",n) << endl;
  cout << string("%6d edges",m) << endl;
  cout << endl;


  cout << "MAX_FLOW    " << flush;

  max_flow<int,static_graph>  mf;

  if (argc >= 2)  
     mf.set_heuristic(atof(argv[1]));

  mf(G,s,t,cap,flow);

  cout << endl;
  cout << "     ";
  cout << string("time: %6.2f sec  f = %d", mf.cpu_time(), mf.flow()) << endl;

  mf.statistics(cout);
  mf.check();

  //mf.print();

  return 0;
}




