#define LEDA_CHECKING_OFF

#include <LEDA/graph.h>
#include <LEDA/dimacs_graph.h>
#include <LEDA/node_slot.h>
#include <LEDA/edge_slot.h>

#include <LEDA/fd_graph.h>

#include <LEDA/templates/dijkstra_s.t>

using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::cin;
using std::flush;
using std::endl;
#endif

int main(int argc, char** argv)
{
  typedef fd_graph< fixed_degree::basic_graph_traits<2,3,1> >  static_graph;
  
  typedef static_graph::node node;
  typedef static_graph::edge edge;

  static_graph G;
  node s = 0;

  node_slot<int, 1,static_graph>  dist;
  node_slot<edge,2,static_graph>  pred;
  edge_slot<int, 0,static_graph>  cost;
  
  cout << endl;

  cout << "fd_graph< basic_graph_traits<";
  cout << static_graph::out_degree << ",";
  cout << static_graph::node_type::data_slots << ",";
  cout << static_graph::edge_type::data_slots << "> >" << endl;
  cout << "sizeof(node_type) " << sizeof(static_graph::node_type)  << endl;
  cout << "sizeof(edge_type) " << sizeof(static_graph::edge_type)  << endl;
  cout << endl;
  
  cout << "reading DIMACS shortest path problem from std input ... ";
  cout << flush;
  
  read_dimacs_sp(cin,G,s,cost);
  
  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  cout << endl;

  node v;
  forall_nodes(v,G)
  {
    edge e;
    forall_adj_edges(e,v)
      if (G.target(e) == G.stop_node())
      {
        cout << "insert edge (" << G.index(v) << "," << G.index(v) << ") " << endl;
        edge f = G.new_edge(v,v);
        cost[f] = 0;
      }
  }
  
  pred.init(G);
  
  cout << endl;
  cout << string("%6d nodes",n) << endl;
  cout << string("%6d edges",m) << endl;
  cout << endl;

  forall_nodes(v,G) pred[v] = nil;
 
  cout << "DIJKSTRA    " << flush;
  float T = used_time();
  
  dijkstra<int,static_graph> D;
  D.run(G,s,cost,dist,pred);
  
  double d = 0;
  forall_nodes(v,G) d += dist[v];
  
  cout << endl;
  cout << "     ";
  cout << string("time: %6.2f sec %f |DIST|",used_time(T),d) << endl;
  cout << endl;

  return 0;
}
