#define LEDA_CHECKING_OFF

#include <fstream>

#include <LEDA/graph.h>
#include <LEDA/dimacs_graph.h>
#include <LEDA/node_slot.h>
#include <LEDA/edge_slot.h>

#include <LEDA/s_graph.h>

#include <LEDA/templates/dijkstra_s.t>

using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::cin;
using std::flush;
using std::endl;
#endif

int main(int argc, char** argv)
{
  typedef s_graph< basic_graph_traits<3,1> >  static_graph;
  
  typedef static_graph::node node;
  typedef static_graph::edge edge;

  static_graph G;
  node s = 0;

  edge_slot<int,0,static_graph>  cost;
  node_slot<int,1,static_graph>  dist;
  node_slot<edge,2,static_graph> pred;
  
  cout << endl;
  cout << "s_graph< basic_graph_traits<3,1> >" << endl;
  cout << "reading DIMACS shortest path problem from std input ... ";
  cout << flush;
  
  read_dimacs_sp(cin,G,s,cost);
  
  cout << endl;
  
  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  cout << endl;
  cout << string("%6d nodes",n) << endl;
  cout << string("%6d edges",m) << endl;
  cout << endl;

  node v;
  forall_nodes(v,G) pred[v] = nil;
 
  cout << "DIJKSTRA    " << flush;
  float T = used_time();
  
  dijkstra<int,static_graph> D;
  D.run(G,s,cost,dist,pred);
  
  double d = 0;
  
  forall_nodes(v,G) d += dist[v];
  
  cout << endl;
  cout << "     ";
  cout << string("time: %6.2f sec %f |DIST|",used_time(T),d) << endl;
  cout << endl;

  return 0;
}




