#if !defined(LEDA_ROOT_INCL_ID)
#define LEDA_ROOT_INCL_ID 430258
#include <LEDA/PREAMBLE.h>
#endif


#include <LEDA/graph.h>
#include <LEDA/node_pq.h>
#include <LEDA/graph_alg.h>

LEDA_BEGIN_NAMESPACE


template <class NT>
__temp_func_inline
void DIJKSTRA_T(const graph& G, node s, 
                   const edge_array<NT>& cost,
                   node_array<NT>& dist, 
                   node_array<edge>& pred)
{ 
  node_pq<NT>  PQ(G); 

  dist[s] = 0;
  PQ.insert(s,0);

  node v;
  forall_nodes(v,G) pred[v] = nil; 

  while (!PQ.empty())
  { node u = PQ.del_min();  // add u to S
    NT du = dist[u];
    edge e;
    forall_adj_edges(e,u)                                                    
    { v = G.opposite(u,e);  // makes it work for ugraphs
      NT c = du + cost[e]; 
      if (pred[v] == nil && v != s ) 
        PQ.insert(v,c);     // first message to v   
      else if (c < dist[v]) PQ.decrease_p(v,c); // better path 
          else continue;
      dist[v] = c; 
      pred[v] = e;                           
    }                                                                  
  } 
}


template <class NT>
__temp_func_inline
void DIJKSTRA_T(const graph& G, node s, 
                   const edge_array<NT>& cost,
                   node_array<NT>& dist)
{ node_array<edge> pred(G);
  DIJKSTRA_T(G,s,cost,dist,pred);
  
}

#if LEDA_ROOT_INCL_ID == 430258
#undef LEDA_ROOT_INCL_ID
#include <LEDA/POSTAMBLE.h>
#endif

LEDA_END_NAMESPACE

