#if !defined(LEDA_ROOT_INCL_ID)
#define LEDA_ROOT_INCL_ID 430259
#include <LEDA/PREAMBLE.h>
#endif

#include <LEDA/node_slot.h>
#include <LEDA/edge_slot.h>
#include <LEDA/node_pq_slot.h>

LEDA_BEGIN_NAMESPACE


template<class NT, class graph_t> 
class dijkstra {

  typedef typename graph_t::node node;
  typedef typename graph_t::edge edge;

public:

template<class cost_array, class dist_array, class pred_array>
void run(const graph_t& G, node s, 
         const cost_array& cost, dist_array& dist, pred_array& pred)
{ 
  node_pq_slot<NT,0,graph_t>  PQ(G); 

  dist[s] = 0;
  PQ.insert(s,0);

  node v;
  forall_nodes(v,G) pred[v] = nil; 

  while (!PQ.empty())
  { node u = PQ.del_min();  // add u to S
    NT du = dist[u];
    edge e;
    forall_adj_edges(e,u)                                                    
    { //v = G.opposite(u,e);  // makes it work for ugraphs
      v = G.target(e);
      NT c = du + cost[e]; 
      if (pred[v] == nil && v != s ) 
        PQ.insert(v,c);     // first message to v   
      else if (c < dist[v]) PQ.decrease_p(v,c); // better path 
          else continue;
      dist[v] = c; 
      pred[v] = e;                           
    }                                                                  
  } 
}

template<class cost_array, class dist_array, class pred_array>
void operator()(const graph_t& G, node s, 
                const cost_array& cost, dist_array& dist, pred_array& pred)
{ run(G,s,cost,dist,pred); }



template<class cost_array, class dist_array>
void run(const graph_t& G, node s, 
              const cost_array& cost, dist_array& dist)
{ 
  node_array<NT> pred(G);
  DIJKSTRA(G,s,cost,dist,pred);
}

template<class cost_array, class dist_array>
void operator()(const graph_t& G, node s, 
                const cost_array& cost, dist_array& dist)
{ run(G,s,cost,dist); }

};



#if LEDA_ROOT_INCL_ID == 430259
#undef LEDA_ROOT_INCL_ID
#include <LEDA/POSTAMBLE.h>
#endif

LEDA_END_NAMESPACE

