#include <LEDA/integer.h>
#include <LEDA/std/string.h>
#include <assert.h>

#define LOW_WORD(x)   (x & 0x0000ffff)
#define HIGH_WORD(x)  (x  >> 16)

//typedef long  digit;

typedef unsigned short digit2;
typedef unsigned int   sz_t;

const double pow53 = ldexp(1.0,53);
const double pow52 = ldexp(1.0,52);
const double pow32 = ldexp(1.0,32);
const double pow16 = ldexp(1.0,16);

inline double high32(double x) { double i; modf(x/pow32,&i); return i;  }
inline double high16(double x) { double i; modf(x/pow16,&i); return i;  }

inline double low32(double x)  { return x - high32(x)*pow32; }
inline double low16(double x)  { return x - high16(x)*pow16; }


void mult(digit* prod, digit*a, sz_t a_used, digit* b, sz_t b_used)
{ 
  digit*  a_stop = a + a_used;
  digit*  b_stop = b + b_used;

  sz_t sz = a_used + b_used + 1;

  assert(2*sz < 512);

  double  DD[1024];  
  double* DD_stop = DD + 2*sz;

  double* dp;

  for(dp = DD; dp < DD_stop; dp++) *dp = 0;

  double* D = DD; 

  for(digit* bp = b; bp < b_stop; bp++) 
  { digit B = *bp;
    digit b0 = LOW_WORD(B);
    digit b1 = HIGH_WORD(B);
    D += 2;
    dp = D;
    for(digit* ap = a; ap < a_stop; ap++) 
    { double A = *ap;
      *dp++ += (A * b0);
      *dp++ += (A * b1);
     }
  }

  // sum up

  for(dp = DD+2; dp < DD_stop; dp += 2)
  { double d = dp[0];
    d += high32(dp[-2]);
    d += high16(dp[-1]);
    d += low16(dp[1])*pow16;
//assert(d < pow53);
    dp[0] = d;
    *prod++ = digit(low32(d));
   }
}



void mult1(digit* prod, digit*a, sz_t a_used, digit* b, sz_t b_used)
{ 
  digit*  a_stop = a + a_used;
  digit*  b_stop = b + b_used;

  sz_t sz = a_used + b_used;

  assert(2*sz < 512);

  double  DD[1024];  
  double* DD_stop = DD + 2*sz;

  double* dp;

  for(dp = DD; dp < DD_stop; dp++) *dp = 0;

  double* D = DD; 

  for(digit* bp = b; bp < b_stop; bp++) 
  { digit B = *bp;
    digit b0 = LOW_WORD(B);
    digit b1 = HIGH_WORD(B);
    dp = D;
    for(digit* ap = a; ap < a_stop; ap++) 
    { double A = *ap;
      *dp++ += (A * b0);
      *dp++ += (A * b1);
     }
     D += 2;
  }

  // sum up

  double x = 0;
  double y = 0;

  for(dp = DD; dp < DD_stop; dp += 2)
  { double d = dp[0];
    d += x;
    d += y;
    d += modf(dp[1]/pow16,&x)*pow32;
//assert(d < pow53);
    *prod++ = digit(modf(d/pow32,&y)*pow32);
   }
}



void mult2(digit* prod, digit*a, sz_t a_used, digit* b, sz_t b_used)
{ 
  digit*  b_stop = b + b_used;

  sz_t sz = a_used + b_used;

  assert(2*sz  < 512);

  double  DD[1024];  
  double* DD_stop = DD + 2*sz;
  double* dp;
  for(dp = DD; dp < DD_stop; dp++) *dp = 0;

  double  AD[1024];  
  double* AD_stop = AD + a_used;
  double* ap;
  for(ap = AD; ap < AD_stop; ap++) *ap = *a++;

  double* D = DD; 

  for(digit* bp = b; bp < b_stop; bp++) 
  { digit B = *bp;
    double b0 = LOW_WORD(B);
    double b1 = HIGH_WORD(B);
    double A;
    dp = D;
    ap = AD;
    while (ap < AD_stop)
    { A = *ap++; *dp++ += (A * b0); *dp++ += (A * b1); }
    D += 2;
  }

  // sum up

  double x = 0;
  double y = 0;

  for(dp = DD; dp < DD_stop; dp += 2)
  { double d = dp[0];
    d += x;
    d += y;
    d += modf(dp[1]/pow16,&x)*pow32;
//assert(d < pow53);
    *prod++ = digit(modf(d/pow32,&y)*pow32);
   }
}


inline unsigned long split32(double x, double& high)
{ unsigned long* p = ((unsigned long*)&x);
  if (x < pow52)
  { x += pow52;
    high = (*p & 0x000fffff);
   }
  else high = (*p & 0x001fffff);
  return p[1]; // low
}


inline double split16(double x, double& high)
{ unsigned long* p = ((unsigned long*)&x);
  if (x < pow52)
  { x += pow52;
    high = (*p & 0x000fffff);
   }
  else high = (*p & 0x001fffff);
  unsigned long l = p[1];
  high *= pow16;
  high += (l >> 16);
  return (l << 16); // low
}


sz_t mult3(digit* prod, digit*a, sz_t a_used, digit* b, sz_t b_used)
{ 
  digit*  b_stop = b + b_used;

  sz_t sz = a_used + b_used;

  //assert(2*sz  < 512);

  //double  DD[1024];  
  double* DD = new double[2*sz];
  double* DD_stop = DD + 2*sz;
  double* dp;
  for(dp = DD; dp < DD_stop; dp++) *dp = 0;

  //double  AD[1024];  
  double* AD = new double[a_used];
  double* AD_stop = AD + a_used;
  double* ap;
  for(ap = AD; ap < AD_stop; ap++) *ap = *a++;

  double* D = DD; 

  for(digit* bp = b; bp < b_stop; bp++) 
  { digit B = *bp;
    double b0 = LOW_WORD(B);
    double b1 = HIGH_WORD(B);
    double A;
    dp = D;
    ap = AD;
    while (ap < AD_stop)
    { A = *ap++; *dp++ += (A * b0); *dp++ += (A * b1); }
    D += 2;
  }

  // sum up

  double x = 0;
  double y = 0;

  for(dp = DD; dp < DD_stop; dp += 2)
  { double d = dp[0];
    d += x;
    d += y;
    d += split16(dp[1],x);
    *prod++ = split32(d,y);
   }

  delete[] DD;
  delete[] AD;

  if (prod[-1] == 0) sz--;
  return sz;
}


inline unsigned long split32a(double x, double& high)
{ unsigned long* p = ((unsigned long*)&x);
  //assert(x >= pow52);
  high = (*p & 0x000fffff);
  return p[1]; // low
}


inline double split16a(double x, double& high)
{ unsigned long* p = ((unsigned long*)&x);
  //assert(x >= pow52);
  high = (*p & 0x000fffff);
  unsigned long l = p[1];
  high *= pow16;
  high += (l >> 16);
  return (l << 16); // low
}



sz_t mult4(digit* prod, digit*a, sz_t a_used, digit* b, sz_t b_used)
{ 
  digit*  b_stop = b + b_used;

  sz_t sz = a_used + b_used;

  assert(a_used <= 128);
  assert(b_used <= 128);

  double  DD[512];
  double* DD_stop = DD + 2*sz;
  double* dp;
  for(dp = DD; dp < DD_stop; dp++) *dp = pow52;

  double  AD[128];  
  double* AD_stop = AD + a_used;
  double* ap;
  for(ap = AD; ap < AD_stop; ap++) *ap = *a++;

  double* D = DD; 

  digit* bp = b;
  while (bp < b_stop)
  { digit B = *bp++;
    double b0 = LOW_WORD(B);
    double b1 = HIGH_WORD(B);
    double A;
    dp = D;
    ap = AD;
    while (ap < AD_stop)
    { A = *ap++; *dp++ += (A * b0); *dp++ += (A * b1); }
    D += 2;
  }

  // sum up

  double x = 0;
  double y = 0;

  for(dp = DD; dp < DD_stop; dp += 2)
  { double d = dp[0];
    d += x;
    d += y;
    d += split16a(dp[1],x);
    *prod++ = split32a(d,y);
   }

  if (prod[-1] == 0) sz--;
  return sz;
}



sz_t mult44(digit* prod, digit*a, sz_t a_used, digit* b, sz_t b_used)
{ 
  digit*  b_stop = b + b_used;

  sz_t sz = a_used + b_used;

  assert(a_used <= 32);
  assert(b_used <= 32);

  double  DD[128];
  double* DD_stop = DD + 2*sz;
  double* dp;
  //for(dp = DD; dp < DD_stop; dp++) *dp = pow52;
  for(dp = DD; dp < DD_stop; dp++) *dp = 0;

  double  AD[32];  
  double* AD_stop = AD + a_used;
  double* ap;
  for(ap = AD; ap < AD_stop; ap++) *ap = *a++;

  double* D = DD; 

  double x = 0;
  double y = 0;

  digit* bp = b;
  while (bp < b_stop)
  { digit B = *bp++;
    double b0 = LOW_WORD(B);
    double b1 = HIGH_WORD(B);
    double A;
    dp = D;
    ap = AD;
    while (ap < AD_stop)
    { A = *ap++; 
      *dp++ += (A * b0); 
      *dp++ += (A * b1); 
     }
    double d = D[0];
    d += x;
    d += y;
    d += split16(D[1],x);
    *prod++ = split32(d,y);
    D += 2;
  }

  // sum up

  while (D < DD_stop)
  { double d = D[0];
    d += x;
    d += y;
    d += split16(D[1],x);
    *prod++ = split32(d,y);
    D += 2;
   }

  if (prod[-1] == 0) sz--;
  return sz;
}





sz_t mult5(digit* prod, digit*a, sz_t a_used, digit* b, sz_t b_used)
{ 
  digit*  b_stop = b + b_used;

  sz_t sz = a_used + b_used;

  double* DD = new double[2*sz];
  double* DD_stop = DD + 2*sz;
  double* dp;
  //for(dp = DD; dp < DD_stop; dp++) *dp = pow52;
  for(dp = DD; dp < DD_stop; dp++) *dp = 0;

  double* AD = new double[a_used];  
  double* AD_stop = AD + a_used;
  double* ap;
  for(ap = AD; ap < AD_stop; ap++) *ap = *a++;

  int limit = 16;
  int count = limit;

  double* Dsum = DD;
  double* D = DD; 
  digit* bp = b;
  while (bp < b_stop)
  { digit B = *bp++;
    double b0 = LOW_WORD(B);
    double b1 = HIGH_WORD(B);
    dp = D;
    D += 2;

    for(ap = AD; ap < AD_stop; ap++)
    { double A = *ap; 
      *dp++ += (A * b0); 
      *dp++ += (A * b1); 
     }

    if (--count == 0 || bp == b_stop)
    { // sum up
      double x = 0;
      double y = 0;
      double* Dsum_stop = Dsum + 2*(limit + a_used);
      if (bp == b_stop) Dsum_stop = DD_stop;
      assert(Dsum_stop <= DD_stop);
      for (dp = Dsum; dp < Dsum_stop; dp += 2)
      { double  d = dp[0];
        d += x;
        d += y;
/*
        d += split16a(dp[1],x);
        dp[0] = split32a(d,y) + pow52;
        dp[1] = pow52;
*/
        d += split16(dp[1],x);
        dp[0] = split32(d,y);
        dp[1] = 0;
       }
      count = limit;
      Dsum += 2*limit;
     }
  }

  for(dp = DD; dp < DD_stop; dp+=2) 
  { double x = 0;
    *prod++ = split32(*dp,x);
   }

  delete[] DD;
  delete[] AD;

  if (prod[-1] == 0) sz--;
  return sz;
}


sz_t mult6(digit* prod, digit*a, sz_t a_used, digit* b, sz_t b_used)
{ 
  digit*  b_stop = b + b_used;

  sz_t sz = a_used + b_used;

  double* DD = new double[2*sz];
  double* DD_stop = DD + 2*sz;
  double* dp;
  //for(dp = DD; dp < DD_stop; dp++) *dp = pow52;
  for(dp = DD; dp < DD_stop; dp++) *dp = 0;

  double* AD = new double[a_used];  
  double* AD_stop = AD + a_used;
  double* ap;
  for(ap = AD; ap < AD_stop; ap++) *ap = *a++;

  int limit = 16;
  int count = limit;

  double* D = DD; 
  digit* bp = b;
  while (bp < b_stop)
  { digit B = *bp++;
    double b0 = LOW_WORD(B);
    double b1 = HIGH_WORD(B);

    dp = D;
    for(ap = AD; ap < AD_stop; ap++)
    { double A = *ap; 
      *dp++ += (A * b0); 
      *dp++ += (A * b1); 
     }

    if (--count == 0)
    { // sum up
      double x = 0;
      double y = 0;
      double* stop = D + 2*a_used;
      for (dp = D; dp < stop; dp += 2)
      { double  d = dp[0];
        d += x;
        d += y;
        d += split16(dp[1],x);
        dp[0] = split32(d,y);
        dp[1] = 0;
/*
        d += split16a(dp[1],x);
        dp[0] = split32a(d,y) + pow52;
        dp[1] = pow52;
*/
       }
      dp[0] = x+y;
      count = limit;
     }

    D += 2;
  }

  double x = 0;
  double y = 0;
  for(dp = DD; dp < DD_stop; dp += 2)
  { double d = dp[0];
    d += x;
    d += y;
    d += split16(dp[1],x);
    *prod++ = split32(d,y);
/*
    d += split16a(dp[1],x);
    *prod++ = split32a(d,y);
*/
   }



  delete[] DD;
  delete[] AD;

  if (prod[-1] == 0) sz--;
  return sz;
}




void multx(digit* prod, digit*a, sz_t a_used, digit* b, sz_t b_used)
{ 
  digit*  b_stop = b + b_used;

  sz_t sz = a_used + b_used;

  assert(2*sz  < 512);
  assert(a_used < 512);

  double  DD[1024];  
  double* DD_stop = DD + 2*sz;
  double* dp;
  for(dp = DD; dp < DD_stop; dp++) *dp = 0;

  double  AD[1024];  
  double* AD_stop = AD + a_used;
  double* ap;
  for(ap = AD; ap < AD_stop; ap++) *ap = *a++;

  double* D = DD; 

  for(digit* bp = b; bp < b_stop; bp++) 
  { digit B = *bp;
    double b0 = LOW_WORD(B);
    double b1 = HIGH_WORD(B);
    double A;
    dp = D;
    ap = AD;
    while (ap < AD_stop)
    { A = *ap++; *dp++ += (A * b0); *dp++ += (A * b1); }
    D += 2;
  }

  // sum up

/*
  double x = 0;
  double y = 0;

  for(dp = DD; dp < DD_stop; dp += 2)
  { double d = dp[0];
    d += x;
    d += y;
    d += modf(dp[1]/pow16,&x)*pow32;
    *prod++ = digit(modf(d/pow32,&y)*pow32);
   }
*/
}



extern "C" digit _vecMulLoop_v8(digit*,digit*,digit*,digit);


sz_t school_mult(digit* prod, digit* a, sz_t a_used, digit* b, sz_t b_used)
{ sz_t p_used = a_used + b_used;
  digit* a_stop = a + a_used;
  digit* b_last = b + b_used - 1;
  //vecInit(prod,a_used);
  memset(prod,0,4*a_used);
  while (b < b_last) _vecMulLoop_v8(prod++,a,a_stop,*b++);
  if (_vecMulLoop_v8(prod,a,a_stop,*b) == 0) p_used--;
  return p_used;
}





int main()
{
   int n = read_int("|a| = ");
   int m = read_int("|b| = ");

   int N = read_int("N = ");

   rand_int.set_seed(n*m);

   integer a;
   integer b;

   a = integer::random(n);
   b = integer::random(m);

   // while (n--) a = (a << 32) | 0xffffffff;
   // while (m--) b = (b << 32) | 0xffffffff;

   integer ab = a*b;

/*
   cout << a << endl;
   cout << b << endl;
   cout << endl;
   cout << ab << endl;
   cout << endl;
*/

   int a_used = a.used_words();
   int b_used = b.used_words();

   digit* a_digits = new digit[a_used];
   digit* b_digits = new digit[b_used];

   int i;
   for(i=0; i<a_used; i++) a_digits[i] = a[i];
   for(i=0; i<b_used; i++) b_digits[i] = b[i];

   int sz = a_used + b_used;

   float T = used_time();

   for(i=0; i<N; i++)
   { integer p = a*b;
     assert(ab == p);
   }
   cout << string("operator*: %.2f sec",used_time(T)) << endl;


   for(i=0; i<N; i++)
   { digit* prod = new digit[sz];
     school_mult(prod,a_digits,a_used,b_digits,b_used);
     integer p(sz,prod);
     delete[] prod;
     assert(ab == p);
   }
   cout << string("school: %.2f sec",used_time(T)) << endl;


   for(i=0; i<N; i++)
   { digit* prod = new digit[sz];
     mult5(prod,a_digits,a_used,b_digits,b_used);
     integer p(sz,prod);
     delete[] prod;
     assert(ab == p);
   }
   cout << string("mult5:  %.2f sec",used_time(T)) << endl;
   


   for(i=0; i<N; i++)
   { digit* prod = new digit[sz];
     mult6(prod,a_digits,a_used,b_digits,b_used);
     integer p(sz,prod);
     delete[] prod;
     assert(ab == p);
   }
   cout << string("mult6:  %.2f sec",used_time(T)) << endl;
   

   for(i=0; i<N; i++)
   { digit* prod = new digit[sz];
     mult(prod,a_digits,a_used,b_digits,b_used);
     integer p(sz,prod);
     delete[] prod;
     assert(ab == p);
   }
   cout << string("mult0:  %.2f sec",used_time(T)) << endl;

   for(i=0; i<N; i++)
   { digit* prod = new digit[sz];
     mult1(prod,a_digits,a_used,b_digits,b_used);
     integer p(sz,prod);
     delete[] prod;
     assert(ab == p);
   }
   cout << string("mult1:  %.2f sec",used_time(T)) << endl;

   for(i=0; i<N; i++)
   { digit* prod = new digit[sz];
     mult2(prod,a_digits,a_used,b_digits,b_used);
     integer p(sz,prod);
     delete[] prod;
     assert(ab == p);
   }
   cout << string("mult2:  %.2f sec",used_time(T)) << endl;


   for(i=0; i<N; i++)
   { digit* prod = new digit[sz];
     mult3(prod,a_digits,a_used,b_digits,b_used);
     integer p(sz,prod);
     delete[] prod;
     assert(ab == p);
   }
   cout << string("mult3:  %.2f sec",used_time(T)) << endl;
   

   for(i=0; i<N; i++)
   { digit* prod = new digit[sz];
     mult4(prod,a_digits,a_used,b_digits,b_used);
     integer p(sz,prod);
     delete[] prod;
     //assert(ab == p);
   }
   cout << string("mult4:  %.2f sec",used_time(T)) << endl;

   for(i=0; i<N; i++)
   { digit* prod = new digit[sz];
     mult44(prod,a_digits,a_used,b_digits,b_used);
     integer p(sz,prod);
     delete[] prod;
     assert(ab == p);
   }
   cout << string("mult44:  %.2f sec",used_time(T)) << endl;
   

   for(i=0; i<N; i++)
   { digit* prod = new digit[sz];
     multx(prod,a_digits,a_used,b_digits,b_used);
     integer p(sz,prod);
     delete[] prod;
     //assert(ab == p);
   }
   cout << string("multx:  %.2f sec",used_time(T)) << endl;

   return 0;
}
