#include <LEDA/window.h>
#include <LEDA/pixmaps/tintin.xpm>

#include <iostream.h>
#include <fstream.h>
#include <fcntl.h>
#include<sys/types.h>
#include<sys/ioctl.h>

#include<linux/soundcard.h>

#define VIDIOCSFREQ	_IOW('v',15, unsigned long)

class volume {

  struct vol_struct
  { unsigned char left;
    unsigned char right;
   };

  vol_struct vol;
  int mixer_fd;

public:

volume()
{ mixer_fd=open("/dev/mixer",O_RDWR,0);
  if ( mixer_fd < 0 )
  { cerr << "Can't open mixer device." << endl;
    exit(1);
   }
 }


bool get_vol(int& left, int& right)
{ if (ioctl(mixer_fd, MIXER_READ(SOUND_MIXER_VOLUME),&vol) < 0) return false;
  left  = vol.left;
  right = vol.right;
  return true;
}

bool set_vol(int left, int right)
{ vol.left  = (unsigned char) left;
  vol.right = (unsigned char) right;
  return ioctl(mixer_fd, MIXER_WRITE(SOUND_MIXER_VOLUME),&vol) >= 0;
}

};


class radio : public volume {

  int radio_fd;

public:

radio()
{ radio_fd = open("/dev/bttv-fm", O_RDONLY);
  if (radio_fd < 0)
  { cerr << "Cannot open radio device." <<endl;
    exit(1);
   }
 }

void set_freq(float f)
{ int freq = int(16*f);
  ioctl(radio_fd,VIDIOCSFREQ,&freq);
 }

};



static string freq_str = "91.7";
static panel* pan_ptr;
static radio* radio_ptr;

static int vol;
static int bal;

void set_volume(int)
{ float f = vol/100.0;
  int left  = int(f*(100-bal));
  int right = int(f*(bal+100));
  if (left > 100) left = 100;
  if (right> 100) right = 100;
  radio_ptr->set_vol(left,right);
 }

void set_station(char* str)
{ float f = atof(str);
  if (f >= 87.5 && f <= 108.0)
  { radio_ptr->set_freq(f);
    pan_ptr->set_icon_label(string("%.1f MHz",f));
   }
 }


int main()
{
  radio R;
  radio_ptr = &R;

  R.set_freq(91.7);

  int left,right;
  R.get_vol(left,right);

  vol = (left+right)/2;
  bal =  0;

  set_volume(0);

  list<string> M;

  string radio_rc = getenv("HOME");
  radio_rc += "/.xradiorc";

  ifstream freqfile(radio_rc);

  M.read(freqfile);


  panel P("Radio Setup Panel");

  if (M.length() > 0)
    P.string_item("MHz",freq_str,M,8,set_station);
  else
    P.string_item("MHz",freq_str,set_station);

  P.int_item("vol",vol,0,100,set_volume);
  P.int_item("bal",bal,-100,100,set_volume);
  P.button("close",0);
  P.button("quit",1);

  pan_ptr = &P;
  set_station("91.7");

  P.set_icon_pixrect(P.create_pixrect(tintin_xpm));

  P.display(window::center,window::center);

  do P.iconify(); 
  while (P.read() == 0);

  return 0;
}


