#define LEDA_CHECKING_OFF

#define SWAP_ARRAYS
#define SORT_SMALLER

#include <LEDA/graph/graph.h>
//#include <LEDA/graph/static_sgraph.h>
#include <LEDA/graph/static_fgraph.h>
#include <LEDA/graph/dimacs_graph.h>
#include <LEDA/graph/node_slot.h>
#include <LEDA/graph/edge_slot.h>
#include <LEDA/graph/node_slot1.h>
#include <LEDA/graph/templates/mcf_cost_scaling.h>
#include <assert.h>


using namespace leda;

using std::cin;
using std::cout;
using std::cerr;
using std::flush;
using std::endl;



bool MCF_COST_SCALING_X(const graph& _G, const edge_array<int>& _lcap,
                                       const edge_array<int>& _ucap,
                                       const edge_array<int>& _cost,
                                       const node_array<int>& _supply,
                                       edge_array<int>& _flow,
                                       double f, int a, int b, 
                                       double& c, double& af, int& restarts)
{

  typedef static_fgraph<opposite_graph,data_slots<7>,data_slots<4> > st_graph;
  typedef st_graph::node st_node;
  typedef st_graph::edge st_edge;

  // node slot 0 used by st_graph
  typedef node_slot1<double,st_graph,1>  pot_array; // 2 slots !
  typedef node_slot<int,st_graph,3>      excess_array;
  typedef node_slot<int,st_graph,4>      dist_array;
  typedef node_slot<st_node,st_graph,5>  succ_array; 
  typedef node_slot<st_node,st_graph,6>  pred_array; 

  st_graph G;

  edge_slot<int,st_graph,0> flow(G);
  edge_slot<int,st_graph,1> cap(G);
  edge_slot<int,st_graph,2> cost(G);
  edge_slot<edge,st_graph,3> e_orig(G);

  excess_array excess(G);

  int n = _G.number_of_nodes();
  int m = _G.number_of_edges();

  G.start_construction(n,m);

  node_array<st_node> V(_G);

  node vv;
  forall_nodes(vv,_G) 
  { st_node v = G.new_node();
    excess[v] = _supply[vv];
    V[vv] = v;
   }

  forall_nodes(vv,_G) 
  { st_node v = V[vv];
    edge x = _G.first_out_edge(vv);
    while (x)
    { st_node w = V[_G.target(x)];
      st_edge e = G.new_edge(v,w);
      e_orig[e] = x;
      x = _G.next_out_edge(x);
     }
   }

  G.finish_construction();

  st_node v;
  forall_nodes(v,G)
  { st_edge e;
    forall_out_edges(e,v)
    { edge x = e_orig[e];
      cost[e] = _cost[x];
      cap[e]  = _ucap[x];
      int lc = _lcap[x];
      if (lc > 0)
      { st_node w = G.opposite(e,v);
        cap[e] -= lc;
        excess[v] -= lc;
        excess[w] += lc;
       }
     }
   }



  mcf_cost_scaling<int,st_graph,pot_array,excess_array,
                                dist_array,succ_array,pred_array> mcf;

  
  bool feasible = mcf.run0(G,cap,cost,excess,flow,f,a,b) ;

  if (feasible)
  { st_node v;
    forall_nodes(v,G)
    { st_edge e;
      forall_out_edges(e,v) 
      { edge x = e_orig[e];
        _flow[x] = flow[e] + _lcap[x];
       }
     }
   }

  c = 0;
  edge x;
  forall_edges(x,_G) c += _flow[x] * double(_cost[x]);

  af = mcf.af();
  restarts = mcf.restarts();

  return feasible;
}



int main(int argc, char** argv)
{

  graph G;

  node_array<int,graph> supply(G);
  edge_array<int,graph> lcap(G);

  edge_array<int,graph> flow(G);
  edge_array<int,graph> ucap(G);
  edge_array<int,graph> cost(G);

  typedef node_array<double,graph> pot_array;
  typedef node_array<int,graph>    excess_array;
  typedef node_array<int,graph>    dist_array;
  typedef node_array<node,graph>   succ_array; 
  typedef node_array<node,graph>   pred_array; 


  dimacs_mcf D(cin);
  D.translate(G,supply,lcap,ucap,cost);

  flow.init(G,0);


  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  cout << string("%6d nodes",n) << endl;
  cout << string("%6d edges",m) << endl;
  cout << endl;


  float f = 0.1;
 
  // a = log(n^2/m) - 1

  int a = 0;
  for(int d = int(n*n/m); d > 1; d /= 2) a++;  
  if (a < 3) a = 12;

  int   b = a;

  bool do_check = false;
  bool dimacs_statistics = false;


  int i = 1;
  while (i < argc)
  { string s = argv[i];
    if (s[0] != '-') break;
    //if (s == "-l") mcf.set_write_log(true);
    if (s == "-c") do_check = true;
    if (s == "-s") dimacs_statistics = true;
    i++;
   }

  if (i < argc) a = b = string(argv[i++]).atoi();
  if (i < argc) f = (float)string(argv[i++]).atof();
  if (i < argc) b = string(argv[i++]).atoi();


  if (dimacs_statistics)
  { D.print_statistics(cout);
    cout << endl;
   }

  int restarts = 0;
  double af = 0;
  double c = 0;

  float T = used_time();

  bool feasible = MCF_COST_SCALING_X(G,lcap,ucap,cost,supply,flow,f,a,b,
                                   c,af, restarts);


  if (feasible)
  { cout << string("time: %6.2f cost: %16.0f  a = %d  af = %.3f (%.2f) rs = %d",
                                            used_time(T), c,a,f,
                                            af/(1<<restarts),restarts) << endl;
    return 0;
    }


/*
  mcf_cost_scaling<int,graph,pot_array,excess_array,
                                dist_array,succ_array,pred_array> mcf;

  
  bool feasible = mcf.run(G,lcap,ucap,cost,supply,flow,f,a,b) ;
     
  double c = mcf.total_cost(G,cost,flow);
     
  if (feasible)
  { cout << string("time: %6.2f cost: %16.0f  a = %d  af = %.3f (%.2f) rs = %d",
                                            mcf.cpu_time(), c,a,f,
                                            (mcf.af()/(1<<mcf.restarts())),
                                            mcf.restarts()) << endl;
     mcf.statistics(cout);

     if (do_check)
     { float T = used_time();
       string msg;
       bool b = mcf.check(G,lcap,ucap,cost,supply,flow,msg);
       cout << string("checking time: %.2f sec",used_time(T)) << endl;
       if (!b) 
       { cerr << "mcf::check failed: " + msg << endl;
         return 2;
        }
      }
     return 0;
   }
*/

  cout << endl;
  cout << "Infeasible MCF-Problem." << endl;
  cout << endl;
  return 1;
}

