#define LEDA_CHECKING_OFF

#include <LEDA/graph/graph.h>
#include <LEDA/graph/static_graph.h>
#include <LEDA/graph/dimacs_graph.h>

#include <LEDA/graph/node_slot.h>
#include <LEDA/graph/edge_slot.h>

#include "mcf_cap_scaling.t"


using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cin;
using std::cout;
using std::flush;
using std::endl;
#endif


int main(int argc, char** argv)
{

  //typedef st_graph<bidirectional_graph,data_slots<5>,data_slots<3> > st_graph;
  typedef static_graph<opposite_graph,data_slots<5>,data_slots<3> > st_graph;
  typedef st_graph::node node;
  typedef st_graph::edge edge;


  st_graph G;

  node_array<int,st_graph>  supply(G);
  edge_array<int,st_graph>  lcap(G);
  edge_array<int,st_graph>  ucap(G);

  edge_slot<int,st_graph,0>  cost(G);
  edge_slot<int,st_graph,1>  flow(G);
  typedef edge_slot<int,st_graph,2> cap_array;

  typedef node_slot<int,st_graph,0>  pot_array;
  typedef node_slot<int,st_graph,1>  excess_array;
  typedef node_slot<int,st_graph,2>  dist_array;
  typedef node_slot<edge,st_graph,3> pred_array;
  typedef node_slot<int,st_graph,4> mark_array;


/*
  cout << endl;
  cout << "reading DIMACS min-cost-flow problem from std input ... ";
  cout << flush;
*/

  dimacs_mcf D(cin);
  D.translate(G,supply,lcap,ucap,cost);

/*
  cout << endl;

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  cout << string("%6d nodes",n) << endl;
  cout << string("%6d edges",m) << endl;
  cout << endl;
*/

  mcf_cap_scaling<int,st_graph, pot_array,excess_array,
                                    dist_array,pred_array,mark_array,cap_array> mcf;



  bool f = mcf.run(G,lcap,ucap,cost,supply,flow,4) ;

  edge e;
  double c = 0;
  forall_edges(e,G) c += cost[e]*flow[e];

  cout << string("time\t %.2f", mcf.cpu_time()) << endl;

  if (f)
    cout << string("result\t %.2f", c) << endl;
  else
    cout << string("result\t INFEASIBLE", c) << endl;

}

