/*******************************************************************************
+
+  LEDA 7.0  
+
+
+  mcfs.c
+
+
+  Copyright (c) 1995-2023
+  by Algorithmic Solutions Software GmbH
+  All rights reserved.
+ 
*******************************************************************************/


#define LEDA_CHECKING_OFF

#include <LEDA/graph/graph.h>
#include <LEDA/graph/dimacs_graph.h>

#include "incl/mcf_cap_scaling5.h"


using namespace leda;

using std::cin;
using std::cout;
using std::flush;
using std::endl;


int main(int argc, char** argv)
{
  bool do_check = false;
  bool dimacs_statistics = false;

  typedef graph::incident_edge incident_edge;

  graph G;

  node_array<int,graph>  supply(G);
  edge_array<int,graph>  lcap(G);

  edge_array<int,graph>  ucap(G);
  edge_array<int,graph>  cost(G);

  typedef node_array<int,graph>  pot_array;
  typedef node_array<int,graph>  excess_array;
  typedef node_array<int,graph>  dist_array;
  typedef node_array<incident_edge,graph> pred_array;
  typedef node_array<int,graph>  mark_array;

  cout << endl;
  cout << "reading DIMACS min-cost-flow problem from std input ... ";
  cout << flush;

  dimacs_mcf D(cin);
  D.translate(G,supply,lcap,ucap,cost);
  cout << endl;

  edge_array<int,graph>  flow(G);

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  cout << string("%6d nodes",n) << endl;
  cout << string("%6d edges",m) << endl;
  cout << endl;

  mcf_cap_scaling<int,graph, pot_array,excess_array, dist_array,
                                       pred_array,mark_array> mcf;


  int i = 1;
  while (i < argc)
  { string s = argv[i];
    if (s[0] != '-') break;
    if (s == "-l") mcf.set_write_log(true);
    if (s == "-c") do_check = true;
    if (s == "-s") dimacs_statistics = true;
    i++;
  }

  if (dimacs_statistics)
  { D.print_statistics(cout);
    cout << endl;
   }



//for(int i = 1; i<=256; i*=2)
{
  int i = 1;

  bool f = mcf.run(G,lcap,ucap,cost,supply,flow,i) ;


  double c = 0;
  edge e;
  forall_edges(e,G) c += cost[e]*flow[e];

  if (f)
   cout << string("time: %6.2f sec c = %f",mcf.cpu_time(),c) << endl;
  else
   cout << string("time: %6.2f sec. Not feasible.",mcf.cpu_time()) << endl;

  string msg;
  if (!mcf.check(G,lcap,ucap,cost,supply,flow,msg))
    error_handler(1,msg);
}

}

