#define LEDA_CHECKING_OFF

#define SWAP_ARRAYS
#define SORT_SMALLER

/*
#define NODE_SIZE_LOG 5
*/

#include <LEDA/graph/graph.h>
#include <LEDA/graph/static_fgraph.h>
#include <LEDA/graph/dimacs_graph.h>


#include <LEDA/graph/node_slot.h>
#include <LEDA/graph/edge_slot.h>

#include <LEDA/graph/node_slot1.h>

#define MCF_STATIC_GRAPH
#include "incl/mcf_cost_scaling.h"

using namespace leda;

using std::cin;
using std::cout;
using std::cerr;
using std::flush;
using std::endl;

int main(int argc, char** argv)
{

  typedef static_fgraph<opposite_graph,data_slots<7>,data_slots<3> > st_graph;
  typedef st_graph::node node;
  //typedef st_graph::edge edge;

  st_graph G;

  node_array<int,st_graph>  supply(G);
  edge_array<int,st_graph>  lcap(G);

  edge_slot<int,st_graph,0> flow(G);
  edge_slot<int,st_graph,1> ucap(G);
  edge_slot<int,st_graph,2> cost(G);


  // node slot 0 used by st_graph

  typedef node_slot1<int,   st_graph,0> excess_array;
  typedef node_slot1<double,st_graph,1> pot_array; // 2 slots !
  typedef node_slot1<int,   st_graph,3> dist_array;
  typedef node_slot1<node,  st_graph,4> succ_array; 
  typedef node_slot1<node,  st_graph,5> pred_array; 


  dimacs_mcf D(cin);
  D.translate(G,supply,lcap,ucap,cost);

/*
{ int max_d[16];
  int i;
  for(int i=0; i<16; i++) max_d[i] = 0;
  node v;
  forall_nodes(v,G)
  { edge x = 0;
    edge e;
    forall_in_edges(e,v)
    { if (x != 0)
      { int d = x-e;
        if (d < 0) d = -d;
        for(i=0; i<16; i++) 
           if (d > max_d[i]) break;
        if (i <16)
        { for(int j=15; j>i; j--) max_d[j] = max_d[j-1];
          max_d[i] = d;
         }
       }
      x = e;
     }
   }

for(i=0;i<16;i++)
  cout << string("max_d[%d] = %6d",i,max_d[i]) << endl;
}
*/

  


  mcf_cost_scaling<int,st_graph,pot_array,excess_array,
                                dist_array,succ_array,pred_array> mcf;


  double n = G.number_of_nodes();
  double m = G.number_of_edges();


  float f = float(0.1);
 
  // a = log(n^2/m) - 1

  int a = 0;
  for(int d = int(n*n/m); d > 1; d /= 2) a++;  
  if (a < 3) a = 12;

  int   b = a;

  bool do_check = false;
  bool dimacs_statistics = false;


  int i = 1;
  while (i < argc)
  { string s = argv[i];
    if (s[0] != '-') break;
    if (s == "-l") mcf.set_write_log(true);
    if (s == "-c") do_check = true;
    if (s == "-s") dimacs_statistics = true;
    i++;
   }

  if (i < argc) a = b = string(argv[i++]).atoi();
  if (i < argc) f = (float)string(argv[i++]).atof();
  if (i < argc) b = string(argv[i++]).atoi();


  if (dimacs_statistics)
  { D.print_statistics(cout);
    cout << endl;
   }

  
  bool feasible = mcf.run(G,lcap,ucap,cost,supply,flow,f,a,b) ;
     
  double c = mcf.total_cost(G,cost,flow);
     
  if (feasible)
  { cout << string("time: %6.2f cost: %14.0f  a = %d  af = %.3f (%.2f) rs = %d",
                                            mcf.cpu_time(), c,a,f,
                                            (mcf.af()/(1<<mcf.restarts())),
                                            mcf.restarts()) << endl;
     mcf.statistics(cout);

     if (do_check)
     { float T = used_time();
       string msg;
       bool b = mcf.check(G,lcap,ucap,cost,supply,flow,msg);
       cout << string("checking time: %.2f sec",used_time(T)) << endl;
       if (!b) 
       { cerr << "mcf::check failed: " + msg << endl;
         return 2;
        }
      }
     return 0;
   }

  cout << endl;
  cout << "Infeasible MCF-Problem." << endl;
  cout << endl;
  return 1;
}

