#define LEDA_CHECKING_OFF

#include <LEDA/graph/graph.h>
#include <LEDA/graph/static_fgraph.h>
#include <LEDA/graph/node_slot1.h>
#include <LEDA/graph/edge_slot1.h>
#include <LEDA/graph/node_pq.h>
#include <LEDA/graph/min_cut.h>
#include <LEDA/system/assert.h>

using namespace leda;
using std::cout;
using std::cin;
using std::endl;
using std::flush;
using std::ostream;


#define MF_UNDIRECTED
#include <LEDA/graph/templates/max_flow.h>


typedef max_flow<int>::node_data nd_type;
typedef max_flow<int>::edge_data ed_type;
  
const int ns = sizeof(nd_type)/3;
const int es = sizeof(ed_type)/4;
  
typedef static_fgraph<opposite_graph,data_slots<ns+1>,data_slots<es+2> > 
                                                                     st_graph;
typedef st_graph::node st_node;
typedef st_graph::edge st_edge;
  
typedef node_slot1<nd_type,st_graph,0>  node_data_array;
typedef node_slot1<node,st_graph,ns>    orig_array;

typedef edge_slot1<ed_type,st_graph,0> edge_data_array;
typedef edge_slot<int,st_graph,es>     cap_array;
typedef edge_slot<int,st_graph,es+1>   flow_array;

typedef  max_flow<int, st_graph, node_data_array, edge_data_array> MaxFlow;




node min_weight_node(const graph& G, const edge_array<int>& weight, int& val)
{
  node_array<int> W(G,0);

  node v;
  forall_nodes(v,G)
  { int sum = 0;
    edge e;
    forall_inout_edges(e,v) sum += weight[e];
    W[v] = sum;
   }

  node u = G.first_node();
  forall_nodes(v,G) 
      if (W[v] < W[u]) u = v;

  val = W[u];

  return u;
}



void contract(graph& G, node a, node b, edge_array<int>& weight, 
                                        node_array<edge>& s_edge,
                                        list<edge>& L, int best_val)
{ 
  if (G.degree(b) == 0) return;

  edge e;
  forall_inout_edges(e,a) s_edge[G.opposite(a,e)] =  e; 

  list<edge> del;
  list<edge> mov;

  forall_inout_edges(e,b)
  { node v = G.opposite(b,e);
    if ( v == a) { del.append(e); continue; }
    edge x = s_edge[v];
    if (x == nil)
      mov.append(e);
    else 
      { weight[x] += weight[e]; 
        if (weight[x] >= best_val) L.append(x);
        del.append(e);
       }
  }

  forall_inout_edges(e,a)  s_edge[G.opposite(a,e)] =  nil; 
 

  forall(e,mov)
  { node v = G.opposite(b,e);
    G.move_edge(e,a,v);
   }

  forall(e,del) G.hide_edge(e);

  assert(G.degree(b) == 0);
}



void construct_contract_graph(GRAPH<node,int>& G, const graph& H, 
                                                  const edge_array<int>& weight)
{
  node_array<node> v_in_G(H); 
  node v;
  forall_nodes(v,H) v_in_G[v] = G.new_node(v);
  edge e;
  forall_edges(e,H)
  { node x = v_in_G[source(e)];
    node y = v_in_G[target(e)];
    edge ee = G.new_edge(x,y,0);
    G[ee] = weight[e];
   }
}



st_node construct_flow_graph(st_graph& G, orig_array& orig, 
                                          cap_array& cap, 
                                          const graph& H, 
                                          node s,
                                          const edge_array<int>& weight,
                                          node_array<st_node>& V)
{
  V.init(H,0);

  int n = H.number_of_nodes();
  int m = H.number_of_edges();

  G.start_construction(n+1,2*m+1);

  st_node s0 = G.new_node();
  orig[s0] = 0;

  node v;
  forall_nodes(v,H) 
  { st_node u =  G.new_node();
    V[v] = u;
    orig[u] = v;
   }

  st_edge x = G.new_edge(s0,V[s]);
  cap[x] = 0;
  
  forall_nodes(v,H)
  { edge e;
    //forall_inout_edges(e,v)
    forall_out_edges(e,v)
    { node w = opposite(v,e);
      st_edge x = G.new_edge(V[v],V[w]);
      cap[x] = weight[e];
     }
   }

  G.finish_construction();

  return s0;
}




int MIN_CUT_MF(const graph& G_orig, edge_array<int>& weight_orig, list<node>& C)
{
  bool use_heuristic = true;
  double frac = 0.5;

  GRAPH<node,int> G;
  construct_contract_graph(G,G_orig,weight_orig);
  edge_array<int>& weight = G.edge_data();

  st_graph F;
  orig_array V_orig(F);
  cap_array  cap(F);
  flow_array flow(F);

  node_array<st_node> V(G,0);
  node_array<edge> s_edge(G,0);

  int best_value = MAXINT;
  node s = min_weight_node(G,weight,best_value);
  node s_orig = G[s];

  best_value++;

  st_node s0 = construct_flow_graph(F,V_orig,cap,G,s,weight,V);
  st_edge e0 = F.first_out_edge(s0);
  int m0 = G.number_of_edges();

  MaxFlow mf;
  node t_best = 0;
  int mf_count = 0;

  node t;
  forall_nodes(t,G)
  { 
    if (t == s || G.degree(t) == 0) continue;

    cap[e0] = best_value;
    //int f = mf.mincut_value(F,s0,V[t],cap,flow,best_value);
    int f = mf.mincut_value(F,s0,V[t],cap,flow);
    mf_count++;

    bool new_best_value = false;
    if (f < best_value) 
    { best_value = f;
      new_best_value = true;
      t_best = G[t];
     }

    if (use_heuristic)
    { list<edge> L;
      contract(G,s,t,weight,s_edge,L,best_value);

      while (!L.empty())
      { edge e = L.pop();
        node x = G.source(e);
        node y = G.target(e);
        if (y == s) swap(x,y);
        contract(G,x,y,weight,s_edge,L,best_value);
       }
  
      if (new_best_value) 
      { list<edge> L;
        edge e;
        forall_edges(e,G)
          if (weight[e] >= best_value) L.append(e);
  
        while (!L.empty())
        { edge e = L.pop();
          node x = G.source(e);
          node y = G.target(e);
          if (y == s) swap(x,y);
          contract(G,x,y,weight,s_edge,L,best_value);
         }
       }
     }

     if (G.number_of_edges() == 0) break;

     if (G.number_of_edges() < frac*m0)
     { //cout << string("rebuild: mf = %d   f = %d   |E| = %d   |F| = %d", 
       //                mf_count, best_value, G.number_of_edges(), 
       //                F.number_of_edges()) << endl;
       s0 = construct_flow_graph(F,V_orig,cap,G,s,weight,V);
       e0 = F.first_out_edge(s0);
       m0 = G.number_of_edges();
      }

  }

  assert(t_best != 0);

  construct_flow_graph(F,V_orig,cap,G_orig,s_orig,weight_orig,V);
  int f = mf.run(F,V[s_orig],V[t_best],cap,flow);
  assert(f == best_value);
  
  list<st_node> cut;
  mf.compute_min_st_cut(F,V[s_orig],V[t_best],cap,flow,cut);

  C.clear();
  st_node v;
  forall(v,cut) C.append(V_orig[v]);

  return best_value; 
}

//------------------------------------------------------------------------
// Stoer/Wagner
//-----------------------------------------------------------------------
  
static void combine_s_and_t(GRAPH<list<node>,int>& G, 
                            node_array<edge>& s_edge, node s, node t)
{ G[s].conc(G[t]);

  edge e;
  forall_adj_edges(e,s) s_edge[G.opposite(s,e)] =  e; 

  //assert(s_edge[t] != nil);


  forall_adj_edges(e,t)
  { node v = G.opposite(t,e);
    if ( v == s) continue;
    edge x = s_edge[v];
    if (x == nil)
       G.move_edge(e,s,v);
    else 
       G[x] += G[e]; 
  }

  node u;
  forall_adj_nodes(u,s)  s_edge[u] =  nil; 
 
  G.del_node(t); 
}

  
int MIN_CUT1(const graph& G0, const edge_array<int>& weight, list<node>& C)
{ 
  bool use_heuristic = true;

  edge e;
  forall_edges(e,G0) 
    if ( weight[e] < 0 ) 
      LEDA_EXCEPTION(1,"MIN_CUT1: no negative weights");

  GRAPH<list<node>,int> G;
  G.make_undirected();
    
  node_array<node> partner(G0);
  node v; 
  forall_nodes(v,G0)
  { node u = G.new_node(); 
    partner[v] = u;
    G[u].append(v); 
  }

  forall_edges(e,G0) 
  { node v = G0.source(e);
    node w = G0.target(e); 
    if (v != w) 
    { edge x = G.new_edge(partner[v], partner[w],0);
      G[x] = weight[e];
     }
   }

  node_array<edge> s_edge(G,nil);

  node a = G.first_node();   
  int best_value = MAXINT; 
  int cut_weight = MAXINT; 

  int hcount1 = 0;
  int hcount2 = 0;

  while ( G.number_of_nodes() >= 2 ) 
  { node t = a; 
    node s = nil; 

    node_pq<int> PQ(G); 

    node v;
    forall_nodes(v,G) 
    { if (v == a) continue; 
      PQ.insert(v,0); 
     }
                                        
    edge e;
    forall_adj_edges(e,a)
    { node b = G.opposite(a,e);
      int p = PQ.prio(b) - G[e];
      PQ.decrease_inf(b,p); 
     }
                                           
    while (!PQ.empty())
    { s = t; 
      t = PQ.find_min();
      cut_weight =  -PQ.prio(t); 
      PQ.del_min(); 
      forall_adj_edges(e,t)
      { node v = G.opposite(t,e); 
        if (PQ.member(v)) PQ.decrease_p(v,PQ.prio(v) - G[e]); 
       }
     }

    bool new_best_cut = false;
    if ( cut_weight < best_value )
    { C = G[t]; 
      best_value = cut_weight; 
      new_best_cut = true;
     }

    combine_s_and_t(G,s_edge,s,t); 

    if (use_heuristic)
    { 
      if (!new_best_cut)
      { edge e = G.first_adj_edge(s); 
        while (e) 
        { node t = G.opposite(s,e);    
          if (G[e] >= best_value) 
          { combine_s_and_t(G,s_edge,s,t); 
            e = G.first_adj_edge(s);
            hcount1++;
           }  
          else e = G.adj_succ(e,s);
        }
      }
      else
      { edge e = G.first_edge();
        while (e)
        { node s = G.source(e);
          node t = G.target(e);    
          if (G[e] >= best_value) 
          { combine_s_and_t(G,s_edge,s,t); 
            e = G.first_edge();
            hcount2++;
           }  
          else e = G.succ_edge(e);
         }
       }

     }// heuristic

  }

/*
  if (use_heuristic) cout << string("(%d/%d)",hcount1,hcount2) << flush;
*/

  return best_value; 
}





int main()
{
 int n = 1000;
 int m = 100000;

 graph G;
 random_simple_undirected_graph(G,n,m);

 edge_array<int> weight(G);
 edge e;
 forall_edges(e,G) weight[e] = rand_int(1,1000);


 float T = used_time(); 

 cout << "MIN_CUT:    " << flush;
 list<node> C0;
 int ct0 = MIN_CUT(G,weight,C0);
 cout << string("|C| = %d  w = %d   %5.2f sec", C0.length(), ct0, used_time(T));
 cout << endl;

 cout << "MIN_CUT1:   " << flush;
 list<node> C1;
 int ct1 = MIN_CUT1(G,weight,C1);
 cout << string("|C| = %d  w = %d   %5.2f sec", C1.length(), ct1, used_time(T));
 cout << endl;

 cout << "MIN_CUT_MF: " << flush;
 list<node> C2;
 int ct2 = MIN_CUT_MF(G,weight,C2);
 cout << string("|C| = %d  w = %d   %5.2f sec", C2.length(), ct2, used_time(T));
 cout << endl;

 assert(ct0 == ct1);
 assert(ct1 == ct2);

 cout << endl;

 return 0;

}

