#include <LEDA/graph/dimacs.h>
#include <LEDA/core/array2.h>
#include <LEDA/system/assert.h>

using namespace leda;

#if defined(LEDA_STD_IO_HEADERS)
using std::cout;
using std::cerr;
using std::endl;
#endif



void construct_cluster(graph& G, array<node>& V, edge_array<int>& weight, 
                                                 int n, int w)
{
  for(int i=0; i< n; i++) V[i] = G.new_node();

  for(int i=0; i<n; i++) 
    for(int j=i+1; j<n; j++) 
    { edge e = G.new_edge(V[i],V[j]);
      weight[e] = w;
     }
}



void min_cut_gen1(graph& G, edge_array<int>& weight, 
                           int cols, int rows, int c_sz)
{
  int c_num = cols*rows;

  int n = c_sz*c_num;
  int m = c_sz*(c_sz-1)*c_num/2 + 2*c_num;

  weight.init(G,m,0);

  array2<node> north(rows,cols);
  array2<node> south(rows,cols);
  array2<node> west(rows,cols);
  array2<node> east(rows,cols);

  array<node> V(c_sz);

  int w = 0;
  for(int i=0; i<cols; i++)
  { for(int j=0; j<cols; j++)
    { construct_cluster(G,V,weight,c_sz,w++);
      north(i,j) = V[0];
      south(i,j) = V[1];
      west(i,j)  = V[2];
      east(i,j)  = V[3];
     }
   }


  for(int i=0; i<cols; i++)
  { for(int j=0; j<cols; j++)
    { node a = north(i,j);
      node b = south((i+1)%rows,j);
      node c = east(i,j);
      node d = west(i,(j+1)%cols);
      edge x = G.new_edge(a,b);
      edge y = G.new_edge(c,d);
      weight[x] = rand_int(1,w);
      weight[y] = rand_int(1,w);
     }
   }

  assert(G.number_of_nodes() == n);
  assert(G.number_of_edges() == m);

}


int main(int argc, char** argv)
{
  if (argc < 4)
  { cerr << "usage: " << argv[0] << " cols rows csize" << endl;
    return 1;
   }

  graph G;
  edge_array<int> weight(G);

  int cols = atoi(argv[1]);
  int rows = atoi(argv[2]);
  int c_sz = atoi(argv[3]); 
  
  min_cut_gen1(G,weight, cols,rows,c_sz);

  return 0;
}

