
template<class graph_t, class node_int_array>
class BICONNECTED_COMPONENTS0 {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;

int count1;
int count2;

float cputime;

template<class compnum_array>
void dfs(const graph_t& G, node v, compnum_array& compnum,
                                   node_int_array& dfsnum,
                                   node_array<int,graph_t>& lowpt,
                                   node_array<node,graph_t>& father,
                                   b_stack<node>& current)
{
  lowpt[v] = dfsnum[v];

  edge e;
  forall_inout_edges(e,v)
  { node w = G.opposite(e,v);
    if (dfsnum[w] == -1)
    { dfsnum[w] = ++count1;
      current.push(w);
      father[w] = v;
      dfs(G,w,compnum,dfsnum,lowpt,father,current);
      lowpt[v] = leda_min(lowpt[v],lowpt[w]);
     }
    else
     lowpt[v] = leda_min(lowpt[v],dfsnum[w]);
   }

  if (father[v] && (lowpt[v] == dfsnum[father[v]]))
  { node w;
    do { w = current.pop();
         edge e;
         forall_inout_edges(e,w)
         if (dfsnum[w] > dfsnum[G.opposite(e,w)]) compnum[e] = count2;
        } while (w != v);

    count2++;
  }

 }

public:

float cpu_time() const { return cputime; }

template<class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }

template<class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{
  float t = used_time();

  int n = G.number_of_nodes();

  b_stack<node> current(n);

  node_int_array dfsnum(G);
  node_array<int,graph_t> lowpt(G);
  node_array<node,graph_t> father(G);

  count1 = 0; 
  count2 = 0;

  int num_isolated = 0;

  node v;
  forall_nodes(v,G) 
  { dfsnum[v] = -1;
    lowpt[v] = 0;
    father[v] = 0;
   }

  forall_nodes(v,G)
  { if (dfsnum[v] != -1) continue;
    dfsnum[v] = ++count1;
    bool is_isolated = true;
    edge e;
    forall_inout_edges(e,v)
      if ( G.opposite(e,v) != v ) { is_isolated = false; break; }

    if ( is_isolated ) 
       num_isolated++;
    else
       { current.push(v);
         dfs(G,v,compnum,dfsnum,lowpt,father,current);
         current.pop();
        }
   }

  cputime = used_time(t);



  return(count2 + num_isolated);

 }

};

//------------------------------------------------------------------------------

template<class graph_t, class node_int_array>
class BICONNECTED_COMPONENTS1 {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;

int count1; 
int count2;

float cputime;

template<class compnum_array>
int dfs(const graph_t& G, node v, node f, compnum_array& compnum,
                                          node_int_array& dfsnum,
                                          b_stack<node>& current)
{ dfsnum[v] = ++count1;
  int lowpnt  = count1;
  current.push(v);

  edge e;
  forall_inout_edges(e,v)
  { node w = G.opposite(e,v);
    int d = dfsnum[w];
    if (d == -1) d = dfs(G,w,v,compnum,dfsnum,current);
    if (d < lowpnt) lowpnt = d;
   }

  if (f && lowpnt == dfsnum[f])
  { node w;
    do { w = current.pop();
         int dw = dfsnum[w];
         edge e;
         forall_inout_edges(e,w)
         { node u = G.opposite(e,w);
           if (dfsnum[u] <= dw) compnum[e] = count2;
          }
       } while (w != v);
    count2++;
  }

  return lowpnt;
}


public:

float cpu_time() const { return cputime; }

template<class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }

template<class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{
  float t = used_time();

  int n = G.number_of_nodes();

  b_stack<node> current(n);
  node_int_array dfsnum(G);

  count1 = 0; 
  count2 = 0;

  int isolated = 0;

  node v;
  forall_nodes(v,G) dfsnum[v] = -1;

  forall_nodes(v,G)
  { if (G.degree(v) == 0) isolated++;
    if (dfsnum[v] == -1)
    { dfs(G,v,0,compnum,dfsnum,current);
      //assert(current.size() == 0);
      assert(current.size() == 1);
      current.pop();
    }
   }

  cputime = used_time(t);

  return count2 + isolated;

}

};



//------------------------------------------------------------------------------

template<class graph_t, class node_int_array>
class BICONNECTED_COMPONENTS2 {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;

int count1; 
int count2;

float cputime;

template<class compnum_array>
int dfs(const graph_t& G, node v, node parent, compnum_array& compnum,
                                               node_int_array& dfsnum,
                                              b_stack<node>& current)
{ int dfs_v = count1++;
  int lowpnt = dfs_v;
  dfsnum[v] = dfs_v;
  current.push(v);

  edge e;
  forall_inout_edges(e,v)
  { node w = G.opposite(e,v);
    int d = dfsnum[w];
    if (d >= 0) continue;
    if (d == -1) d = dfs(G,w,v,compnum,dfsnum,current);
    if (d < lowpnt) lowpnt = d;
   }


  if (parent && lowpnt == dfsnum[parent])
  { node w;
    do { w = current.pop();
         int dw = dfsnum[w];
         edge e;
         forall_out_edges(e,w)
         { node u = G.opposite(e,w);
           if (compnum[e] == -1) compnum[e] = count2;
          }
         dfsnum[w] = count2;
       } while (w != v);

     // root
     forall_out_edges(e,parent)
     { node u = G.opposite(e,parent);
       if (u == parent || dfsnum[u] == count2) compnum[e] = count2;
      }

    count2++;
  }


  return lowpnt;
}



public:

float cpu_time() const { return cputime; }

template<class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }


template<class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{
  float t = used_time();

  int n = G.number_of_nodes();

  b_stack<node> current(n);
  node_int_array dfsnum(G);

  count1 = -(n+1);  // dfs numbers: -(n+1) ... -2 
  count2 = 0;       // comp numbers: 0,1,2,  ..
  int isolated = 0;


  edge e;
  forall_edges(e,G) compnum[e] = -1; // unlabeled

  node v;
  forall_nodes(v,G) dfsnum[v] = -1;

  forall_nodes(v,G)
  { if (G.degree(v) == 0) isolated++;
    if (dfsnum[v] == -1)
    { dfs(G,v,NULL,compnum,dfsnum,current);
      assert(current.size() == 1);
      current.pop();
    }
   }

  cputime = used_time(t);

  return count2 + isolated;

}

};


//------------------------------------------------------------------------------

template<class graph_t, class node_int_array>
class BICONNECTED_COMPONENTS3 {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;

int count1; 
int count2;

float cputime;

template<class compnum_array>
int dfs(const graph_t& G, node v, node parent, compnum_array&  compnum,
                                               node_int_array& dfsnum,
                                               b_stack<node>&  edgeStack,
                                               b_stack<node>&  current)
{ int dfs_v = count1++;
  int lowpnt = dfs_v;
  dfsnum[v] = dfs_v;
  current.push(v);

  int sz = edgeStack.size();

   edge e;
  forall_inout_edges(e,v) edgeStack.push(G.opposite(e,v));
/*
  for(e = G.last_in_edge(v); e; e = G.in_pred(e)) edgeStack.push(G.source(e));
  for(e = G.last_out_edge(v); e; e = G.out_pred(e)) edgeStack.push(G.target(e));
*/

  while (edgeStack.size() > sz)
  { node w = edgeStack.pop();
    int d = dfsnum[w];
    if (d >= 0) continue;
    if (d == -1) d = dfs(G,w,v,compnum,dfsnum,edgeStack,current);
    if (d < lowpnt) lowpnt = d;
   }

  if (parent && lowpnt == dfsnum[parent])
  { node w;
    do { w = current.pop();
         int dw = dfsnum[w];
         edge e;
         forall_out_edges(e,w)
         { node u = G.opposite(e,w);
           if (compnum[e] == -1) compnum[e] = count2;
          }
         dfsnum[w] = count2;
       } while (w != v);

     edge e;
     forall_out_edges(e,parent)
     { node u = G.opposite(e,parent);
       if (u == parent || dfsnum[u] == count2) compnum[e] = count2;
      }
     count2++;
   }

  return lowpnt;
}



public:

float cpu_time() const { return cputime; }

template<class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }


template <class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{
  float t = used_time();

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  b_stack<node> edgeStack(2*m+1);

  b_stack<node> current(n);
  node_int_array dfsnum(G);

  count1 = -(n+1);  // dfs numbers: -(n+1) ... -2 
  count2 = 0;       // comp numbers: 0,1,2,  ..

  int isolated = 0;


  edge e;
  forall_edges(e,G) compnum[e] = -1; // unlabeled

  node v;
  forall_nodes(v,G) dfsnum[v] = -1;

  forall_nodes(v,G)
  { if (G.degree(v) == 0) isolated++;
    if (dfsnum[v] == -1)
    { dfs(G,v,NULL,compnum,dfsnum,edgeStack,current);
      assert(current.size() == 1);
      current.pop();
     }
   }

  cputime = used_time(t);

  return count2 + isolated;

}

};


//------------------------------------------------------------------------------

template<class graph_t, class node_int_array>
class BICONNECTED_COMPONENTS4 {

typedef typename graph_t::node node;
typedef typename graph_t::edge edge;

int count1; 
int count2;

float cputime;

template<class compnum_array>
int dfs(const graph_t& G, node v, int dfs_p, compnum_array& compnum,
                                             b_stack<node>&   edgeStack,
                                             b_stack<node>&   current)
{ int dfsnum = count1++;
  int lowpnt = dfsnum;
  compnum[v] = dfsnum;
  current.push(v);

  int sz = edgeStack.size();

   edge e;
  forall_inout_edges(e,v) edgeStack.push(G.opposite(e,v));
/*
  for(e = G.last_in_edge(v); e; e = G.in_pred(e)) edgeStack.push(G.source(e));
  for(e = G.last_out_edge(v); e; e = G.out_pred(e)) edgeStack.push(G.target(e));
*/

  while (edgeStack.size() > sz)
  { node w = edgeStack.pop();
    int d = compnum[w];
    if (d >= 0) continue;
    if (d == -1) d = dfs(G,w,dfsnum,compnum,edgeStack,current);
    if (d < lowpnt) lowpnt = d;
   }

  if (/*dfs_p == MAXINT || */lowpnt == dfs_p)
  { node w;
    do { w = current.pop();
         compnum[w] = count2;
       } while (w != v);
    count2++;
   }

  return lowpnt;
}


public:

float cpu_time() const { return cputime; }

template<class compnum_array>
int operator()(const graph_t& G, compnum_array& compnum)
{ return run(G,compnum); }


template <class compnum_array>
int run(const graph_t& G, compnum_array& compnum)
{
  float t = used_time();

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  b_stack<node> edgeStack(2*m+1);

  b_stack<node> current(n);


  count1 = -(n+1);  // dfs numbers: -(n+1) ... -2 
  count2 = 0;       // comp numbers: 0,1,2,  ..

  int isolated = 0;

  node v;
  forall_nodes(v,G) compnum[v] = -1;

  forall_nodes(v,G)
  { if (G.degree(v) == 0) { compnum[v] = MAXINT; isolated++; }
    if (compnum[v] == -1)
    { dfs(G,v,MAXINT,compnum,edgeStack,current);
      //assert(current.size() == 0);
      assert(current.size() == 1);
      current.pop();
     }
   }

  cputime = used_time(t);

  return count2 + isolated;

}

};




