#include <LEDA/core/basic_alg.h>
#include <LEDA/system/assert.h>
#include <LEDA/core/string.h>
#include <LEDA/internal/std/string.h>

using namespace leda;
using std::cout;
using std::endl;

//LEDA_BEGIN_NAMESPACE

typedef long long  int64;
typedef unsigned long long  uint64;

static int count[0x10000];

template<class integer_t>
inline unsigned access(const integer_t* p, int i)
{ unsigned short* q = (unsigned short*)p;
  return q[i];
}

template<class integer_t>
inline void counting_sort(integer_t*& A, integer_t*& B, size_t n, 
                                                   unsigned mask, 
                                                   unsigned mask1, 
                                                   size_t shift)
{
  integer_t* A_stop = A+n;
  integer_t* p = A;
  integer_t  x = (A[0] >> shift) & mask;

  while (p < A_stop && ((*p >> shift) & mask) == x) p++;

  if (p == A_stop)
  { leda_swap(A,B);
    return;
   }

  memset(count,0,int((mask+1)*sizeof(int)));

  if (mask1 == 0)
  { count[x] = int(p-A);
    while (p < A_stop) 
    { int x = (*p++ >> shift) & mask;
      count[x]++;
     }
   }
  else
  { // flip sign bit
    count[x ^ mask1] = int(p-A);
    while (p < A_stop) 
    { int x = (*p++ >> shift) & mask;
      count[x ^ mask1]++;
     }
   }


  int sum = 0;
  for(unsigned i=0; i<=mask; i++)
  { int r = count[i];
    count[i] = sum;
    sum += r;
   }


  if (mask1 == 0)
    for(p=A; p < A_stop; p++)
    { integer_t x = *p;
      int r = count[((x >> shift) & mask)]++;
      B[r] = x;
     }
  else
    for(p=A; p < A_stop; p++)
    { integer_t x = *p;
      int r = count[((x >> shift) & mask) ^ mask1]++;
      B[r] = x;
     }
}



void INT32_SORT(int* A, int N)
{
  int* B = new int[N];
  int* p = A;
  int* q = B;

  counting_sort(p,q,N,0xffff,0x0000, 0);
  counting_sort(q,p,N,0xffff,0xA000,16);

  if (p != A) memcpy(A,B,int(N*sizeof(int)));

  delete[] B;
}


void INT64_SORT(int64* A, int N)
{
  int64* B = new int64[N];
  int64* p = A;
  int64* q = B;

  counting_sort(p,q,N,0xffff,0x0000, 0);
  counting_sort(q,p,N,0xffff,0x0000,16);
  counting_sort(p,q,N,0xffff,0x0000,32);
  counting_sort(q,p,N,0xffff,0xA000,48);

  if (p != A) memcpy(A,B,N*sizeof(int64));

  delete[] B;
}


void INT64_SORT1(int64* A, int N)
{
  int64* B = new int64[N];
  int64* p = A;
  int64* q = B;

  counting_sort(p,q,N,0xfff,0x000, 0);
  counting_sort(q,p,N,0xfff,0x000,12);
  counting_sort(p,q,N,0xfff,0x000,24);
  counting_sort(q,p,N,0xfff,0x000,36);
  counting_sort(p,q,N,0xfff,0x000,48);
  counting_sort(q,p,N,0x00f,0x00A,60);

  if (p != A) memcpy(A,B,N*sizeof(int64));

  delete[] B;
}



void INT64_SORT2(int64* A, int N)
{
  int64* B = new int64[N];
  int64* p = A;
  int64* q = B;

  counting_sort(p,q,N,0xff,0x00, 0);
  counting_sort(q,p,N,0xff,0x00, 8);
  counting_sort(p,q,N,0xff,0x00,16);
  counting_sort(q,p,N,0xff,0x00,24);
  counting_sort(p,q,N,0xff,0x00,32);
  counting_sort(q,p,N,0xff,0x00,40);
  counting_sort(p,q,N,0xff,0x00,48);
  counting_sort(q,p,N,0xff,0xA0,56);

  if (p != A) memcpy(A,B,N*sizeof(int64));

  delete[] B;
}



//LEDA_END_NAMESPACE
   

int main () 
{
  int i;
  int N = read_int("N = "); 

  int D = 1000000000;

  int err_count = 0;

{
  int* A = new int[N];

  rand_int.set_seed(123456789);
  for (i = 0; i < N; i++) A[i] = rand_int(-D,D);

  float T = used_time();
  cout << "sort32    ";
  cout.flush();
  INTSORT(A,N);
  cout << string("  %5.3f sec",used_time(T));
  cout << endl;
/*
  for(i=1;i<N;i++)
    if (A[i-1] > A[i]) err_count++;
*/
  delete[] A;
}

{
  int* A = new int[N];

  rand_int.set_seed(123456789);
  for (i = 0; i < N; i++) A[i] = rand_int(-D,D);

  float T = used_time();
  cout << "sort32    ";
  cout.flush();
  INT32_SORT(A,N);
  cout << string("  %5.3f sec",used_time(T));
  cout << endl;
/*
  for(i=1;i<N;i++)
    if (A[i-1] > A[i]) err_count++;
*/
  delete[] A;
}

{
  int64* A = new int64[N];

  rand_int.set_seed(123456789);
  for (i = 0; i < N; i++) A[i] = rand_int(-D,D);

  float T = used_time();
  cout << "sort64    ";
  cout.flush();
  INT64_SORT(A,N);
  cout << string("  %5.3f sec",used_time(T));
  cout << endl;
  for(i=1;i<N;i++)
    if (A[i-1] > A[i]) err_count++;
  delete[] A;
}

{
  int64* A = new int64[N];

  rand_int.set_seed(123456789);
  for (i = 0; i < N; i++) A[i] = rand_int(-D,D);

  float T = used_time();
  cout << "sort64    ";
  cout.flush();
  INT64_SORT1(A,N);
  cout << string("  %5.3f sec",used_time(T));
  cout << endl;
  for(i=1;i<N;i++)
    if (A[i-1] > A[i]) err_count++;
  delete[] A;
}


{
  int64* A = new int64[N];

  rand_int.set_seed(123456789);
  for (i = 0; i < N; i++) A[i] = rand_int(-D,D);

  float T = used_time();
  cout << "sort64    ";
  cout.flush();
  INT64_SORT2(A,N);
  cout << string("  %5.3f sec",used_time(T));
  cout << endl;
  for(i=1;i<N;i++)
    if (A[i-1] > A[i]) err_count++;
  delete[] A;
}


  cout << endl;
  cout << string("%d errors",err_count);
  cout << endl;

  return 0;

}

