#include <LEDA/core/basic_alg.h>
#include <LEDA/core/array.h>
#include <LEDA/system/assert.h>
#include <LEDA/core/string.h>
/*
#include <LEDA/internal/std/string.h>
*/


#include <limits.h>
#include <string.h>

using namespace leda;
using std::cout;
using std::endl;

//LEDA_BEGIN_NAMESPACE

typedef long long  int64;
typedef unsigned long long  uint64;

static int count[0x10000];


template<typename integer_t>
inline integer_t cyclic_shift(const integer_t& x, int s)
{ const int bits = sizeof(integer_t) * CHAR_BIT;
  return (x >> s) | (x << (bits-s));
}



template<class integer_t>
inline void counting_sort(integer_t*& A, integer_t*& B, size_t n,  
                                                   size_t shift,
                                                   bool flip_sign = 0)
{
  unsigned mask = (1<<shift)-1;
  unsigned mask1 = 0;
  if (flip_sign) mask1 = 1 << (shift-1);


  integer_t* A_stop = A+n;
  integer_t* p = A;
  integer_t  x = A[0] & mask;

  while (p < A_stop && (*p & mask) == x) p++;

  if (p == A_stop)
  { //leda_swap(A,B);
    integer_t* q = B;
    for(p=A; p < A_stop; p++) *q++ = cyclic_shift(*p,shift);
    return;
   }

  //int* count = new int[mask+1];

  //memset(count,0,int((mask+1)*sizeof(int)));
  for(size_t j=0; j<=mask; j++) count[j] = 0;

  if (mask1 == 0)
  { count[x] = int(p-A);
    while (p < A_stop) 
    { int x = *p++ & mask;
      count[x]++;
     }
   }
  else
  { // flip sign bit
    count[x ^ mask1] = int(p-A);
    while (p < A_stop) 
    { int x = *p++ & mask;
      count[x ^ mask1]++;
     }
   }


  int sum = 0;
  for(unsigned i=0; i<=mask; i++)
  { int r = count[i];
    count[i] = sum;
    sum += r;
   }


  if (mask1 == 0)
    for(p=A; p < A_stop; p++)
    { integer_t x = *p;
      int r = count[(x & mask)]++;
      B[r] = cyclic_shift(x,shift);
     }
  else
    for(p=A; p < A_stop; p++)
    { integer_t x = *p;
      int r = count[(x & mask) ^ mask1]++;
      B[r] = cyclic_shift(x,shift);
     }

  //delete[] count;
}



void INT32_SORT(int* A, int N,int s)
{ unsigned int* B = new unsigned int[N];
  unsigned int* a = (unsigned int*)A;
  unsigned int* b = B;

  int bits = 32;
  while (bits > s)
  { counting_sort(a,b,N,s,false);
    leda_swap(a,b);
    bits -= s;
   }

  counting_sort(a,b,N,bits,true);

  if (b == B) memcpy(a,b,int(N*sizeof(int)));
  delete[] B;
}


void INT64_SORT(int64* A, int N,int s)
{ uint64* B = new uint64[N];
  uint64* a = (uint64*)A;
  uint64* b = B;

  int bits = 64;
  while (bits > s)
  { counting_sort(a,b,N,s,false);
    leda_swap(a,b);
    bits -= s;
   }

  counting_sort(a,b,N,bits,true);

  if (b == B) memcpy(a,b,int(N*sizeof(int64)));
  delete[] B;
}



//LEDA_END_NAMESPACE
   

int main () 
{
  int i;
  int N = read_int("N = "); 

  int Dmax = 0x3fffffff;
  int Dmin = -Dmax;


{
  array<int> A(N);
  int err_count = 0;

  rand_int.set_seed(123456789);
  for (i = 0; i < N; i++) A[i] = rand_int(Dmin,Dmax);

  float T = used_time();
  cout << "A.quicksort  ";
  cout.flush();
  A.sort();
  cout << string(" %6.3f sec",used_time(T));

  for(i=1;i<N;i++) if (A[i-1] > A[i]) err_count++;

  cout << string("  %d errors",err_count) << endl;
}

{
  int* A = new int[N];
  int err_count = 0;

  rand_int.set_seed(123456789);
  for (i = 0; i < N; i++) A[i] = rand_int(Dmin,Dmax);

  float T = used_time();
  cout << "QUICKSORT    ";
  cout.flush();
  QUICKSORT_C(A,A+N-1);
  cout << string(" %6.3f sec",used_time(T));

  for(i=1;i<N;i++) if (A[i-1] > A[i]) err_count++;

  delete[] A;
  cout << string("  %d errors",err_count) << endl;
}


{
  int* A = new int[N];
  int err_count = 0;

  rand_int.set_seed(123456789);
  for (i = 0; i < N; i++) A[i] = rand_int(0,100000);

  smaller_default<int> smaller;

  float T = used_time();
  cout << "MERGESORT    ";
  cout.flush();
  mergesort(A,A+N-1,smaller);
  cout << string(" %6.3f sec",used_time(T));

  for(i=1;i<N;i++) if (A[i-1] > A[i]) err_count++;

  delete[] A;
  cout << string("  %d errors",err_count) << endl;
}

cout << endl;


{
  int* A = new int[N];
  int err_count = 0;

  rand_int.set_seed(123456789);
  for (i = 0; i < N; i++) A[i] = rand_int(Dmin,Dmax);

  float T = used_time();
  cout << "INTSORT     ";
  cout.flush();
  INTSORT(A,N);
  cout << string(" %6.3f sec",used_time(T));

  for(i=1;i<N;i++) if (A[i-1] > A[i]) err_count++;

  delete[] A;
  cout << string("  %d errors",err_count) << endl;
}

cout << endl;

//for(int k=4; k<=16; k+=1)
for(int k=8; k<=8; k++)
{
  int* A = new int[N];

  int err_count = 0;

  rand_int.set_seed(123456789);
  for (i = 0; i < N; i++) A[i] = rand_int(Dmin,Dmax);

  int iterations = 32/k;
  if (32 % k) iterations++;

  float T = used_time();
  cout << string("sort32  k = %2d  it = %2d  ",k,iterations);
  cout.flush();

  INT32_SORT(A,N,k);

  cout << string(" %6.3f sec",used_time(T));
  for(i=0;i<N;i++)
  { //cout << A[i] << endl;
    if (i > 0  && A[i-1] > A[i]) err_count++;
   }
  delete[] A;
  cout << string("  %d errors",err_count) << endl;
}


cout << endl;

for(int k=8; k<=8; k++)
{
  int64* A = new int64[N];

  int err_count = 0;

  rand_int.set_seed(123456789);
  for (i = 0; i < N; i++) 
  { A[i] = rand_int(Dmin,Dmax);
    A[i] *= rand_int(Dmin,Dmax);
   }

  int iterations = 64/k;
  if (64 % k) iterations++;

  float T = used_time();
  cout << string("sort64  k = %2d  it = %2d  ",k,iterations);
  cout.flush();

  INT64_SORT(A,N,k);

  cout << string(" %6.3f sec",used_time(T));
  for(i=0;i<N;i++)
  { //cout << A[i] << endl;
    if (i > 0  && A[i-1] > A[i]) err_count++;
   }
  cout << string("  %d errors",err_count) << endl;

/*
  if (err_count > 0))
  { for(int i=0;i<N;i++) 
    { cout << A[i] << "  ";
      cout << string("%08x%08x",(unsigned long)(A[i]>>32),
                                (unsigned long)(A[i]&0xffffffff)) << endl;
    }
    cout << endl;
  }
*/

  delete[] A;
}


  return 0;

}

